package cn.rankin.apiweb.assist.resolver;

import cn.rankin.apiweb.entity.GlobalHeader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.MethodParameter;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

import java.util.Optional;

/**
 * @author Tomas
 * @date 2016-07-17
 * @what
 */
@Component
public class RequestHeaderResolver implements HandlerMethodArgumentResolver {
    private static final Logger logger = LoggerFactory.getLogger(RequestHeaderResolver.class);
    public boolean supportsParameter(MethodParameter methodParameter) {
        return  methodParameter.hasParameterAnnotation(RequestHeader.class);
    }

    public Object resolveArgument(MethodParameter methodParameter, ModelAndViewContainer modelAndViewContainer, NativeWebRequest nativeWebRequest, WebDataBinderFactory webDataBinderFactory) throws Exception {
        logger.info(String.format("Resolve argument %s.", methodParameter.getParameterName()));
        if(methodParameter.hasParameterAnnotation(RequestHeader.class)){
            GlobalHeader globalHeader = new GlobalHeader();
            globalHeader.setUid(nativeWebRequest.getHeader("uid"));
            globalHeader.setEid(nativeWebRequest.getHeader("eid"));
            globalHeader.setRequestId(nativeWebRequest.getHeader("requestId"));
            globalHeader.setTerminal(nativeWebRequest.getHeader("terminal"));

            //To be compatible with 2.x app version, if sign not exist, use Authenciation
            String sign = nativeWebRequest.getHeader("sign");
            if (null != sign && !sign.isEmpty()){
                globalHeader.setSign(sign);
            }
            else{
                globalHeader.setSign(Optional.ofNullable(nativeWebRequest.getHeader("Authentication")).orElse(""));
            }

            return globalHeader;
        }
        return null;
    }

}
