package cn.rankin.apiweb.configuration;

import cn.rankin.apiweb.assist.resolver.NeedUserResolver;
import cn.rankin.apiweb.assist.resolver.RequestHeaderResolver;
import cn.rankin.apiweb.intercepter.RequestSignatureInterceptor;
import cn.rankin.common.utils.web.intercepter.HeaderProcessIntercepter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.EnvironmentAware;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;

import java.util.List;

/**
 * 类名：WebConfiguration  <br />
 *
 * 功能：Web相关配置
 *
 * @author tomas <br />
 * 创建时间：2016年7月27日 下午3:57:19  <br />
 * @version 2016年7月27日*/
@Configuration
@EnableWebMvc
public class FrontConfiguration extends WebMvcConfigurerAdapter implements EnvironmentAware {
	// 日志记录器
	private static final Logger logger = LoggerFactory.getLogger(FrontConfiguration.class);

	@Autowired
	private RequestSignatureInterceptor requestSignatureInterceptor;

	// 当前的环境对象
	protected Environment environment;


	public void addInterceptors(InterceptorRegistry registry) {
		registry.addInterceptor(new HeaderProcessIntercepter());
		registry.addInterceptor(requestSignatureInterceptor).excludePathPatterns("*/favicon.ico");
	}

	public void addArgumentResolvers(List<HandlerMethodArgumentResolver> argumentResolvers) {
		argumentResolvers.add(new RequestHeaderResolver());
		argumentResolvers.add(new NeedUserResolver());
		super.addArgumentResolvers(argumentResolvers);
	}

	public void addResourceHandlers(ResourceHandlerRegistry registry) {
		registry.addResourceHandler("swagger-ui.html")
				.addResourceLocations("classpath:/META-INF/resources/");
		registry.addResourceHandler("favicon.ico")
				.addResourceLocations("classpath:/static/favicon.ico");
		registry.addResourceHandler("/webjars/**")
				.addResourceLocations("classpath:/META-INF/resources/webjars/");
	}
	/**
	 * Set the {@code Environment} that this object runs in.
	 *
	 * @param environment
	 */
	@Override
	public void setEnvironment(Environment environment) {
		this.environment = environment;
	}
}