package cn.rankin.apiweb.controller;

import cn.rankin.apiweb.assist.resolver.NeedUser;
import cn.rankin.apiweb.service.auth.AuthService;
import cn.rankin.apiweb.service.collection.CollectionService;
import cn.rankin.apiweb.service.product.ProductService;
import cn.rankin.apiweb.service.shopcart.ShopCartService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.app.vo.*;
import cn.rankin.data.api.auth.vo.AuthResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(value = "/course")
public class CourseController {

    @Autowired
    private CollectionService collectionService;

    @Autowired
    private ProductService productService;

    @Autowired
    private AuthService authService;

    @Autowired
    private ShopCartService shopCartService;

    @RequestMapping(value = "/{courseId}/lessons", method = RequestMethod.GET)
    public APIResult<CourseVo> getCourseLessons(@NeedUser DeviceUserVo user, @PathVariable("courseId") String courseId) {
        String userId = user.getUid();
        String merchantId = user.getMerchantId();
        APIResult<CourseVo> apiResult = productService.getCourseLessons(courseId);
        if (!apiResult.getSuccess()) {
            return apiResult;
        }
        CourseVo courseVo = apiResult.getData();
        Boolean auth = authService.isValid(userId, merchantId, courseId);
        courseVo.setAuth(auth);

        Boolean isCollected = collectionService.isCollected(userId, courseId);
        courseVo.setCollected(isCollected);
        return apiResult;
    }

    @RequestMapping(value = "/{courseId}/detail", method = RequestMethod.GET)
    public APIResult<CourseDetailVo> getCourseDetail(@NeedUser DeviceUserVo user, @PathVariable("courseId") String courseId) {
        String merchantId = user.getMerchantId();
        String userId = user.getUid();
        APIResult<CourseDetailVo> courseDetailVoAPIResult = productService.getCourseDetail(courseId, merchantId);
        CourseDetailVo courseDetailVo = courseDetailVoAPIResult.getData();
        if (!courseDetailVoAPIResult.getSuccess() || courseDetailVo == null) {
            return courseDetailVoAPIResult;
        }

        courseDetailVo.setCollected(collectionService.isCollected(userId, courseId));
        APIResult<AuthResult> authApiResult = authService.auth(userId, merchantId, courseId);
        if (authApiResult.getSuccess()) {
            AuthResult authResult = authApiResult.getData();
            courseDetailVo.setAuth(authResult.isValid());
            courseDetailVo.setBeginTime(authResult.getStartTime());
            courseDetailVo.setEndTime(authResult.getEndTime());
        }

        List<ItemGoodsVo> itemGoodsVoList = courseDetailVo.getGoods();
        shopCartService.isInCart(userId, itemGoodsVoList);

        return courseDetailVoAPIResult;
    }

    @RequestMapping(value = "/{courseId}/supports", method = RequestMethod.GET)
    public APIResult<List<ItemVo>> getSupportByCourseId(@NeedUser DeviceUserVo user, @PathVariable("courseId") String courseId) {
        String merchantId = user.getMerchantId();
        return productService.getSupportByCourseId(courseId, merchantId);
    }
}
