package cn.rankin.apiweb.controller;

import cn.rankin.apiweb.assist.resolver.RequestHeader;
import cn.rankin.apiweb.entity.GlobalHeader;
import cn.rankin.apiweb.service.user.UserService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.util.HttpUtil;
import cn.rankin.data.api.app.dto.DeviceLoginDTO;
import cn.rankin.data.api.app.dto.LoginInfoDTO;
import cn.rankin.data.api.app.vo.UserInfoVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
@Slf4j
@RestController
@RequestMapping(value = "/token")
public class LoginController {

    @Autowired
    private UserService userService;

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<UserInfoVo> login(HttpServletRequest request, @Valid @RequestBody LoginInfoDTO loginInfoDTO) {
        String ip = HttpUtil.getClientIp(request);
        loginInfoDTO.setIp(ip);
        loginInfoDTO.setTerminal(request.getHeader("terminal"));
        loginInfoDTO.setMerchant(request.getHeader("merchant"));

        log.info("login request header : ip={},terminal={},merchant={}",ip , request.getHeader("terminal"), request.getHeader("merchant"));
        return userService.login(loginInfoDTO);
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<UserInfoVo> refresh(HttpServletRequest request, @RequestBody DeviceLoginDTO deviceLoginDTO) {
        String deviceCode = deviceLoginDTO.getDeviceCode();
        String merchant = request.getHeader("merchant");
        log.info("refresh request header : deviceCode={}, merchant={}", deviceCode, merchant);
        return userService.refresh(deviceCode, merchant, Boolean.TRUE);
    }

    @RequestMapping(method = RequestMethod.DELETE)
    public APIResult<Boolean> logout(@RequestHeader GlobalHeader globalHeader) {
        String userId = globalHeader.getUid();
        return userService.logout(userId);
    }
}
