package cn.rankin.apiweb.controller;

import cn.rankin.apiweb.assist.resolver.NeedUser;
import cn.rankin.apiweb.service.product.ProductClient;
import cn.rankin.apiweb.service.product.ProductService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.app.vo.DeviceUserVo;
import cn.rankin.data.api.product.entity.Product;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

@RestController
@RequestMapping(value="/package")
@Slf4j
public class PackageController {

    @Autowired
    private ProductService productService;

    @Autowired
    private ProductClient productClient;

    @RequestMapping(value = "/{pid}")
    public APIResult getPackage(@NeedUser DeviceUserVo user, @PathVariable("pid") String pid){

        String uid = user.getUid();
        String merchantId = user.getMerchantId();

        //get merchant product, with goods field
        Product product = productClient.getProduct(merchantId, pid);

        //get package, with children field
        Product pkg = productClient.getPackage(pid);

        if (null == product || null == pkg){
            log.error("No Package Found, pid={}, merchantId={}", pid, merchantId);
            return APIResult.ok(new HashMap<>());
        }

        pkg.setGoods(product.getGoods());

        Map<String, Object> data = new HashMap<>();

        data.put("id", pkg.getPid());
        data.put("name", pkg.getName());
        data.put("code", pkg.getCode());
        data.put("type", pkg.getType());
        data.put("goods",product.getGoods());
        data.put("totalNum", Optional.ofNullable(pkg.getChildrens()).map(v->v.size()).orElse(0));
        data.put("recs", pkg.getChildrens());

        return APIResult.ok(data);
    }
}
