package cn.rankin.apiweb.controller;

import cn.rankin.apiweb.assist.resolver.NeedUser;
import cn.rankin.apiweb.service.product.ProductClient;
import cn.rankin.apiweb.service.product.ProductService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.app.vo.DeviceUserVo;
import cn.rankin.data.api.product.entity.Product;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;


@RestController
@RequestMapping(value = "/product")
@Slf4j
public class ProductController {

    @Autowired
    private ProductService productService;

    @Autowired
    private ProductClient productClient;

    @RequestMapping(value = "/{pid}/relatedPkg", method = RequestMethod.GET)
    public APIResult getRelatedPkg(@NeedUser DeviceUserVo user, @PathVariable("pid") String pid) {
        String merchantId = user.getMerchantId();

        //get Pkg
        List<Product> packages = productClient.getRelatedPackages(pid, merchantId, 0L, 10, "id", Sort.Direction.ASC);

        if(null == packages){
            packages = new ArrayList<>();
        }

        //if no package, self is a package
        /*
        if (packages.isEmpty()){
            log.info("No RelatedPackage Found, Use Product itself, pid={}", pid);
            Product targetProduct = productClient.getProduct(merchantId, pid);

            if(null != targetProduct){
                packages.add(targetProduct);
            }
        }
        */

        //get each product in Pkg
        List<Map<String, Object>> recs = new ArrayList<>();

        for(Product pkg : packages){

            if(null == pkg.getPid()){
                log.error("Related Package pid is Null, pid={}", pid);
                continue;
            }

            Product targetPkg =  productClient.getPackage(pkg.getPid());

            List<Product> pkgContents = Optional.ofNullable(targetPkg).map(v -> v.getChildrens()).orElse(new ArrayList<>());

            if(null == pkgContents){
                pkgContents = new ArrayList<>();
            }

            List<Map<String, Object>> lightProducts = new ArrayList<>();
            for(Product content : pkgContents){
               Map<String, Object> tmp = new HashMap<>();
               tmp.put("id", content.getPid());
               tmp.put("name", content.getName());
               tmp.put("type", content.getType());
               tmp.put("code", content.getCode());
               lightProducts.add(tmp);
            }

            Map<String, Object> rec = new HashMap<String, Object>();
            rec.put("id", pkg.getPid());
            rec.put("name", pkg.getName());
            rec.put("code", pkg.getCode());
            rec.put("goods", pkg.getGoods());
            rec.put("recs", lightProducts);

            recs.add(rec);
        }


        Map<String, Object> data = new HashMap<>();

        data.put("totalNum", recs.size());
        data.put("recs", recs);

        return APIResult.ok(data);

    }
}
