package cn.rankin.apiweb.controller;

import cn.rankin.apiweb.assist.resolver.NeedUser;
import cn.rankin.apiweb.service.product.ProductService;
import cn.rankin.apiweb.service.tag.TagService;
import cn.rankin.apiweb.service.user.UserService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.app.vo.*;
import cn.rankin.data.api.user.entity.UserTag;
import cn.rankin.data.api.user.entity.UserTagProductRelation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
@Slf4j
@RestController
public class TagController {

    @Autowired
    private TagService tagService;

    @Autowired
    private UserService userService;

    @Autowired
    private ProductService productService;

    @RequestMapping(value = "/tagGroup/{code}", method = RequestMethod.GET)
    public APIResult<TagGroupVo> getTagGroup(@NeedUser DeviceUserVo user, @PathVariable("code") String code) {
        String merchantId = user.getMerchantId();
        return tagService.findTagGroupByCode(code, merchantId);
    }

    @RequestMapping(value = "/tag/{tagId}", method = RequestMethod.GET)
    public APIResult<Page<ItemVo>> findItemByTagId(@NeedUser DeviceUserVo user, @PathVariable("tagId") String tagId,
                                                   @RequestParam("pageNo") Integer pageNo, @RequestParam("pageSize") Integer pageSize) {
        String merchantId = user.getMerchantId();
        return tagService.findPageByTagId(tagId, merchantId, pageNo, pageSize);
    }

    @RequestMapping(value = "/tagType/{code}", method = RequestMethod.GET)
    public APIResult<List<TagVo>> getTagType(@NeedUser DeviceUserVo user,@PathVariable("code") String code) {
        List<TagVo> result = new ArrayList<>();

        //先去取用户标签，没有用户标签再使用渠道标签
        String uid = user.getUid();
        String merchantId = user.getMerchantId();

        //用户标签列表
        List<UserTag> userTagList = userService.findUserTagByCodeUid(code,uid);
        if(null != userTagList && userTagList.size() > 0){
            log.info("get userTag By code and uid, size={}", userTagList.size());
            //组装数据   用户标签下   课程数据
            userTagList.forEach(userTag -> {
                List<UserTagProductRelation> relationList = userTag.getRelationList();
                log.info("userTag relationList , size={}", relationList);
                if(!CollectionUtils.isEmpty(relationList)){
                    List<String> productIdList = new ArrayList<>();
                    relationList.forEach(relation -> productIdList.add(relation.getPid()) );
                    List<ItemVo> itemVoList = productService.findItemByPids(productIdList,merchantId);

                    TagVo vo = new TagVo();
                    vo.setId(userTag.getId());
                    vo.setName(userTag.getName());
                    vo.setRecs(itemVoList);
                    result.add(vo);
                }
            });
            return APIResult.ok(result);
        }

        //没有用户标签数据，使用渠道标签数据
        return tagService.findTagTypeByCode(code, merchantId);
    }

}
