package cn.rankin.apiweb.controller;

import cn.rankin.apiweb.assist.resolver.NeedUser;
import cn.rankin.apiweb.code.ApiWebCode;
import cn.rankin.apiweb.service.product.ProductService;
import cn.rankin.apiweb.service.user.UserService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.app.vo.DeviceUserVo;
import cn.rankin.data.api.app.vo.UserInfoVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

@RestController
@RequestMapping(value = "/user")
@Slf4j
public class UserController {

    @Autowired
    private UserService userService;

    @Autowired
    private ProductService productService;

    @RequestMapping(method = RequestMethod.GET)
    public APIResult<UserInfoVo> getUserInfo(@NeedUser DeviceUserVo deviceUserVo) {
        String uid = deviceUserVo.getUid();
        if (StringUtils.isEmpty(uid)) {
            return APIResult.error(ApiWebCode.NOT_EXISTS);
        }

        UserInfoVo userInfoVo = userService.getUserInfo(uid);
        return APIResult.ok(userInfoVo);
    }

    @RequestMapping(value="/product/valid", method = RequestMethod.GET)
    public APIResult getProductValid(@NeedUser DeviceUserVo user) {
        String uid = user.getUid();
        if (StringUtils.isEmpty(uid)) {
            log.error("Uid-Empty, uid={}", uid);
            return APIResult.error(ApiWebCode.NOT_EXISTS);
        }

        List<Map<String, Object>> products =  userService.getProductValid(uid);


        if (null == products){
            products = new ArrayList<>();
        }

        log.error("Valid Product Get From Service, product={}, len={}", products, products.size());

        List<Map<String, Object> > recs = new ArrayList<>();

        for(Map<String, Object> item : products){
            Map<String, Object> rec = new HashMap<>();
            rec.put("id", item.get("pid"));
            rec.put("title", item.get("name"));
            rec.put("beginTime", item.get("beginTime"));
            rec.put("endTime", item.get("endTime"));

            recs.add(rec);
        }

        HashMap<String, Object> data = new HashMap<String, Object>();
        data.put("totalNum", recs.size());
        data.put("recs", recs);

        return APIResult.ok(data);
    }

    @RequestMapping(value="/messages/productExpiredAlert", method = RequestMethod.GET)
    public APIResult getMessagesProductExpired(@NeedUser DeviceUserVo user){

        String uid = user.getUid();

        //get Valid Product
        List<Map<String, Object>> validItems =  userService.getProductValid(uid);

        List<Map<String, Object> > recs = new ArrayList<>();

        Long now = new Date().getTime();

        // Five Days
        Long remindTime = 5*24*3600*1000L;

        for(Map<String, Object> item : validItems){

            Long endTime = (Long) item.get("endTime");

            if (null == endTime){
                log.error("Illegal Product endTime, endTime=null");
                continue;
            }

            if (endTime  > now + remindTime){
               continue;
            }

            Map<String, Object> rec = new HashMap<>();
            rec.put("id", item.get("pid"));
            rec.put("title", item.get("name"));
            rec.put("beginTime", item.get("beginTime"));
            rec.put("endTime", item.get("endTime"));
            rec.put("type", item.get("type"));
            recs.add(rec);
        }

        Map<String, Object> data = new HashMap<>();
        data.put("totalNum", recs.size());
        data.put("recs", recs);

        return APIResult.ok(data);
    }

}
