package cn.rankin.apiweb.service.collection;

import cn.rankin.apiweb.code.ApiWebCode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.user.dto.CollectionDTO;
import cn.rankin.data.api.user.entity.Collection;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@FeignClient(name = "${service.user.name}", fallback = CollectionClient.CollectionServiceHystrix.class)
public interface CollectionClient {

    @RequestMapping(value = "/user/collection", method = RequestMethod.POST)
    APIResult<Boolean> put(@RequestBody CollectionDTO collectionDTO);

    @RequestMapping(value = "/user/collection", method = RequestMethod.GET)
    APIResult<Collection> getCollection(@RequestParam("userId") String userId, @RequestParam("pid") String productId);

    @RequestMapping(value = "/user/collection/list", method = RequestMethod.GET)
    APIResult<List<Collection>> getUserCollections(@RequestParam("userId") String userId, @RequestParam("size") Integer size);

    @RequestMapping(value = "/user/collection", method = RequestMethod.DELETE)
    APIResult<Boolean> delete(@RequestParam("userId") String userId, @RequestParam("pid") String productId);

    @RequestMapping(value = "/user/collection/page", method = RequestMethod.GET)
    APIResult<Page<Collection>> getPage(@RequestParam("userId") String userId, @RequestParam("pageNo") Integer pageNo, @RequestParam("pageSize") Integer pageSize);

    @Component
    class CollectionServiceHystrix implements CollectionClient {

        @Override
        public APIResult<Collection> getCollection(String userId, String productId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult put(CollectionDTO collectionDTO) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult getUserCollections(String userId, Integer size) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult delete(String userId, String productId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult getPage(String userId, Integer pageNo, Integer pageSize) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

    }
}
