package cn.rankin.apiweb.service.product;

import cn.rankin.apiweb.code.ApiWebCode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.app.dto.ItemSearchDTO;
import cn.rankin.data.api.app.vo.GoodsVo;
import cn.rankin.data.api.app.vo.ItemVo;
import cn.rankin.data.api.product.entity.*;
import cn.rankin.data.api.product.vo.CourseItemVo;
import cn.rankin.data.api.product.vo.SupportItemVo;
import cn.rankin.data.api.product.vo.TrainingItemVo;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@FeignClient(name = "${service.product.name}")
public interface ProductClient {

    @RequestMapping(value = "/recommend/courses", method = RequestMethod.GET)
    APIResult<List<Recommend>> getRecommendCourses(@RequestParam("merchantId") String merchantId);

    @RequestMapping(value = "/merchant/product/detail", method = RequestMethod.GET)
    APIResult<MerchantProduct> getMerchantProduct(@RequestParam("pid") String pid, @RequestParam("merchantId") String merchantId);

    @RequestMapping(value = "/app/item/ids", method = RequestMethod.POST)
    APIResult<List<ItemVo>> findItemByIds(@RequestParam("merchantId") String merchantId, @RequestBody List<String> productIdList);

    @RequestMapping(value = "/app/item/ids/page", method = RequestMethod.POST)
    APIResult<Page<ItemVo>> findItemPageByIds(@RequestBody ItemSearchDTO itemSearchDTO);

    @RequestMapping(value = "/app/course/{courseId}/lessons", method = RequestMethod.GET)
    APIResult<Course> getCourseLessons(@PathVariable("courseId") String courseId);

    @RequestMapping(value = "/app/course/{courseId}/detail", method = RequestMethod.GET)
    APIResult<CourseItemVo> getCourseDetail(@PathVariable("courseId") String courseId, @RequestParam("merchantId") String merchantId);

    @RequestMapping(value = "/app/course/{courseId}/supports", method = RequestMethod.GET)
    APIResult<List<ItemVo>> getSupportByCourseId(@PathVariable("courseId") String courseId, @RequestParam("merchantId") String merchantId);

    @RequestMapping(value = "/app/support/{supportId}", method = RequestMethod.GET)
    APIResult<SupportItemVo> getSupport(@PathVariable("supportId") String supportId, @RequestParam("merchantId") String merchantId);

    @RequestMapping(value = "/training/{id}", method = RequestMethod.GET)
    APIResult<TrainingItemVo> getTraining(@PathVariable("id") String id, @RequestParam("merchantId") String merchantId);

    @RequestMapping(value = "/app/lesson/{lessonId}", method = RequestMethod.GET)
    APIResult<Lesson> getLesson(@PathVariable("lessonId") String lessonId, @RequestParam("courseId") String courseId);

    @RequestMapping(value = "/app/goods", method = RequestMethod.POST)
    APIResult<List<GoodsVo>> findGoodsByIds(@RequestBody List<String> goodsIdList, @RequestParam("merchantId") String merchantId);

    @RequestMapping(value = "/goods/{goodsId}", method = RequestMethod.GET)
    APIResult<Goods> findGoods(@PathVariable("goodsId") String goodsId);

    @RequestMapping(value = "/merchant/{merchantId}/posters", method = RequestMethod.GET)
    List<Poster> getPosters(@PathVariable("merchantId") String merchantId, @RequestParam("start") Long start,
                                   @RequestParam("offset") Integer offset, @RequestParam("sortKey") String sortKey,
                                   @RequestParam("direction") Sort.Direction direction);

    @RequestMapping(value = "/user/{uid}/product/valid", method = RequestMethod.GET)
    List<Map<String, Object>> getProductValid(@PathVariable("uid") String uid, @RequestParam("start") Long start,
                                              @RequestParam("offset") Integer offset, @RequestParam("sortKey") String sortKey,
                                              @RequestParam("direction") Sort.Direction direction);

    @RequestMapping(value = "/product/{pid}/relatedPkg", method = RequestMethod.GET)
    List<Product> getRelatedPackages(@PathVariable("pid") String pid, @RequestParam("merchantId") String merchantId, @RequestParam("start") Long start,
                                              @RequestParam("offset") Integer offset, @RequestParam("sortKey") String sortKey,
                                              @RequestParam("direction") Sort.Direction direction);

    @RequestMapping(value = "/package/{pid}", method = RequestMethod.GET)
    Product getPackage(@PathVariable("pid") String pid);

    @RequestMapping(value = "/merchant/{merchantId}/product/{pid}", method = RequestMethod.GET)
    Product getProduct(@PathVariable("merchantId") String merchantId, @PathVariable("pid") String pid);

    @RequestMapping(value = "/app/course/{id}", method = RequestMethod.GET)
    Course getCourse(@PathVariable("id") String id);

    @RequestMapping(value = "/item/pids", method = RequestMethod.POST)
    List<ItemVo> findItemByPids(@RequestBody List<String> productIdList,
                                @RequestParam("merchantId") String merchantId);


    @Component
    class ProductClientHystrix implements ProductClient {

        @Override
        public APIResult getRecommendCourses(String merchantId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult getMerchantProduct(String pid, String merchantId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult findItemByIds(String merchantId, List<String> productIdList) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult findItemPageByIds(ItemSearchDTO itemSearchDTO) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult<Course> getCourseLessons(String courseId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult<CourseItemVo> getCourseDetail(String courseId, String merchantId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult<List<ItemVo>> getSupportByCourseId(String courseId, String merchantId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult<Lesson> getLesson(String lessonId, String courseId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult getSupport(String supportId, String merchantId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult getTraining(String id, String merchantId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult<List<GoodsVo>> findGoodsByIds(List<String> goodsIdList, String merchantId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        @Override
        public APIResult<Goods> findGoods(String goodsId) {
            return APIResult.error(ApiWebCode.SERVER_ERROR);
        }

        public List<Poster> getPosters(String merchantId, Long start,
                                Integer offset, String sortKey, Sort.Direction direction){
            return new ArrayList<>();
        }


        @Override
        public List<Map<String, Object>> getProductValid(String uid, Long start,
                                                         Integer offset, String sortKey, Sort.Direction direction){
            return new ArrayList<>();
        }

        @Override
        public List<Product> getRelatedPackages(String pid, String merchantId, Long start,
                                                         Integer offset, String sortKey, Sort.Direction direction){
            return new ArrayList<>();
        }

        @Override
        public Product getPackage(String pid){
            return null;
        }


        @Override
        public Product getProduct(String merchantId, String pid) {
            return null;
        }

        @Override
        public Course getCourse(String id) {
            return null;
        }

        @Override
        public List<ItemVo> findItemByPids(List<String> productIdList, String merchantId) {
            return null;
        }

    }
}
