package cn.rankin.apiweb.service.resource;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.constant.ResourceType;
import cn.rankin.common.utils.vo.resource.ResourceVo;
import cn.rankin.data.api.app.vo.CourseWareVo;
import cn.rankin.data.api.app.vo.LessonVo;
import cn.rankin.data.api.app.vo.ResourceItemVo;
import cn.rankin.data.api.product.entity.CourseWare;
import cn.rankin.data.api.product.entity.Lesson;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.rankin.apiweb.utils.DTOConverter.convert;

@Service
@Slf4j
public class ResourceService {

    @Autowired
    private ResourceClient resourceClient;

    public Map<String, ResourceVo> getResourceMap(List<String> idList) {
        Map<String, ResourceVo> resourceVoMap = new HashMap<>();
        if (CollectionUtils.isEmpty(idList)) {
            return resourceVoMap;
        }

        APIResult<Map<String, ResourceVo>> apiResult = resourceClient.getResourceMap(idList);
        if (!apiResult.getSuccess()) {
            log.warn("get resource map error!");
            return resourceVoMap;
        }else if (apiResult.getData() == null) {
            log.warn("get resource map empty");
            return resourceVoMap;
        }

        return apiResult.getData();
    }


    public LessonVo setWareUrl(Lesson lesson) {
        LessonVo lessonVo = convert(lesson);

        List<String> wareIdList = new ArrayList<>();
        List<CourseWare> courseWareList = lesson.getWareList();
        courseWareList.forEach(e -> {
            List<String> resourceIdList = e.getResourceList();
            if (!CollectionUtils.isEmpty(resourceIdList)) {
                wareIdList.addAll(resourceIdList);
            }
        });

        Map<String, ResourceVo> resourceVoMap = getResourceMap(wareIdList);
        List<CourseWareVo> courseWareVoList = new ArrayList<>();
        for (CourseWare courseWare : courseWareList) {
            CourseWareVo courseWareVo = convert(courseWare);
            List<String> resourceIdList = courseWare.getResourceList();
            if (CollectionUtils.isEmpty(resourceIdList)) {
                courseWareVoList.add(courseWareVo);
                continue;
            }

            List<ResourceItemVo> resourceVoList = new ArrayList<>();
            Integer type = ResourceType.IMG;
            for (String resourceId : resourceIdList) {
                ResourceVo resourceVo = resourceVoMap.get(resourceId);
                if (resourceVo != null) {
                    ResourceItemVo resourceItemVo = new ResourceItemVo();
                    resourceItemVo.setId(resourceVo.getId());
                    resourceItemVo.setUrl(resourceVo.getUrl());
                    Integer resourceType = resourceVo.getType();
                    resourceItemVo.setType(resourceType);
                    if (resourceType.equals(ResourceType.VIDEO)) {
                        type = ResourceType.VIDEO;
                    }else if (resourceType.equals(ResourceType.AUDIO_IMG)) {
                        type = ResourceType.AUDIO_IMG;
                        resourceItemVo.setImg(resourceVo.getImg());
                        resourceItemVo.setAudio(resourceVo.getAudio());
                    }
                    resourceVoList.add(resourceItemVo);
                }
            }
            courseWareVo.setList(resourceVoList);
            courseWareVo.setType(type);
            courseWareVoList.add(courseWareVo);
        }
        lessonVo.setList(courseWareVoList);
        return lessonVo;
    }
}
