package cn.rankin.cmsweb.controller.product;

import cn.rankin.cmsweb.assist.resolver.NeedUser;
import cn.rankin.cmsweb.entity.UserDetails;
import cn.rankin.cmsweb.service.product.GroupService;
import cn.rankin.cmsweb.service.user.MerchantInfoSetService;
import cn.rankin.cmsweb.service.user.MerchantService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.model.BaseCode;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.GroupDTO;
import cn.rankin.common.utils.dto.search.GroupSearchDTO;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.data.api.product.vo.GroupVo;
import cn.rankin.data.api.user.vo.MerchantVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@Slf4j
@RestController
@RequestMapping(value = "/group")
public class GroupController {

    @Autowired
    private GroupService groupService;

    @Autowired
    private MerchantService merchantService;

    @Autowired
    private MerchantInfoSetService<GroupVo> merchantInfoSetService;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<GroupVo>> search(@NeedUser UserDetails user, GroupSearchDTO searchDTO) {
        if (!user.isPlatForm()) {
            searchDTO.setMerchantId(user.getMerchantId());
        }
        APIResult<Page<GroupVo>> pageAPIResult = groupService.search(BeanUtil.convertToMap(searchDTO));
        if (pageAPIResult.getSuccess()) {
            merchantInfoSetService.setMerchantInfo(pageAPIResult.getData().getList());
        }
        return pageAPIResult;
    }

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<GroupVo> create(@Valid @RequestBody GroupDTO groupDTO) {
        // 转大写
        String merchantId = groupDTO.getMerchantId();
        APIResult<MerchantVo> merchantVoAPIResult = merchantService.getMerchant(merchantId);
        if (merchantVoAPIResult.getSuccess()) {
            MerchantVo merchantVo = merchantVoAPIResult.getData();
            String code = merchantVo.getSimple() + "_" + groupDTO.getCode();
            groupDTO.setCode(code.toUpperCase());
        }else {
            return APIResult.error(new BaseCode(merchantVoAPIResult.getCode(), merchantVoAPIResult.getMessage()));
        }

        APIResult<GroupVo> groupVoAPIResult = groupService.create(groupDTO);
        if (groupVoAPIResult.getSuccess()) {
            merchantInfoSetService.setMerchantInfo(groupVoAPIResult.getData());
        }
        return groupVoAPIResult;
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<GroupVo> update(@RequestBody GroupDTO groupDTO) {
        APIResult<GroupVo> groupVoAPIResult = groupService.update(groupDTO);
        if (groupVoAPIResult.getSuccess()) {
            merchantInfoSetService.setMerchantInfo(groupVoAPIResult.getData());
        }
        return groupVoAPIResult;
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@PathVariable("id") String id) {
        return groupService.delete(id);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<GroupVo> getGroup(@PathVariable("id") String id) {
        APIResult<GroupVo> groupVoAPIResult = groupService.getGroup(id);
        if (groupVoAPIResult.getSuccess()) {
            merchantInfoSetService.setMerchantInfo(groupVoAPIResult.getData());
        }
        return groupVoAPIResult;
    }
}
