package cn.rankin.cmsweb.controller.product;

import cn.rankin.cmsweb.assist.resolver.NeedUser;
import cn.rankin.cmsweb.entity.UserDetails;
import cn.rankin.cmsweb.service.product.MerchantProductService;
import cn.rankin.cmsweb.service.user.MerchantInfoSetService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.data.api.product.dto.MerchantProductDTO;
import cn.rankin.data.api.product.dto.MerchantProductSearchDTO;
import cn.rankin.data.api.product.vo.MerchantProductVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(value = "/merchant/product")
public class MerchantProductController {

    @Autowired
    private MerchantProductService merchantProductService;

    @Autowired
    private MerchantInfoSetService merchantInfoSetService;

    @RequestMapping(method = RequestMethod.GET)
    public APIResult<Page<MerchantProductVo>> search(@NeedUser UserDetails user, MerchantProductSearchDTO searchDTO) {
        if (!user.isPlatForm()) {
            searchDTO.setMerchantId(user.getMerchantId());
        }
        APIResult<Page<MerchantProductVo>> apiResult = merchantProductService.search(BeanUtil.convertToMap(searchDTO));
        if (apiResult.getSuccess()) {
            List<MerchantProductVo> merchantProductVoList = apiResult.getData().getList();
            merchantInfoSetService.setMerchantInfo(merchantProductVoList);
        }
        return apiResult;
    }

    @RequestMapping(value = "/detail", method = RequestMethod.GET)
    public APIResult<MerchantProductVo> getProduct(@RequestParam("pid") String productId, @RequestParam("merchantId") String merchantId) {
        return merchantProductService.getProduct(productId, merchantId);
    }

    @RequestMapping(value = "/status", method = RequestMethod.PUT)
    public APIResult<Boolean> change(@RequestBody MerchantProductDTO merchantProductDTO) {
        return merchantProductService.change(merchantProductDTO);
    }

    @RequestMapping(value = "/tags", method = RequestMethod.PUT)
    public APIResult<MerchantProductVo> mountTags(@RequestBody MerchantProductDTO productDTO) {
        return merchantProductService.mountTags(productDTO);
    }
}
