package cn.rankin.cmsweb.controller.product;

import cn.rankin.cmsweb.service.product.ProductService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.data.api.product.dto.*;
import cn.rankin.data.api.product.vo.*;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
@Slf4j
@RestController
@RequestMapping(value = "/product")
public class ProductController {

    @Autowired
    private ProductService productService;

    @RequestMapping(method = RequestMethod.GET)
    public APIResult<Page<ProductVo>> search(ProductSearchDTO searchDTO) {
        log.info("GET /product request params ,params = {}",JSON.toJSONString(searchDTO));
        return productService.search(BeanUtil.convertToMap(searchDTO));
    }

    @RequestMapping(value = "/{productId}", method = RequestMethod.GET)
    public APIResult getProduct(@PathVariable("productId") String productId) {
        return productService.getProduct(productId);
    }

    @RequestMapping(value = "/course", method = RequestMethod.PUT)
    public APIResult<CourseVo> updateCourse(@RequestBody CourseDTO courseDTO) {
        return productService.updateCourse(courseDTO);
    }

    @RequestMapping(value = "/course", method = RequestMethod.POST)
    public APIResult<CourseVo> createCourse(@RequestBody CourseDTO courseDTO) {
        return productService.createCourse(courseDTO);
    }

    @RequestMapping(value = "/support", method = RequestMethod.POST)
    public APIResult<SupportVo> createSupport(@RequestBody SupportDTO supportDTO) {
        return productService.createSupport(supportDTO);
    }

    @RequestMapping(value = "/support", method = RequestMethod.PUT)
    public APIResult<SupportVo> updateSupport(@RequestBody SupportDTO supportDTO) {
        return productService.updateSupport(supportDTO);
    }

    @RequestMapping(value = "/package", method = RequestMethod.POST)
    public APIResult<PackageVo> createPackage(@RequestBody PackageDTO packageDTO) {
        return productService.createPackage(packageDTO);
    }

    @RequestMapping(value = "/package", method = RequestMethod.PUT)
    public APIResult<PackageVo> updatePackage(@RequestBody PackageDTO packageDTO) {
        return productService.updatePackage(packageDTO);
    }

    @RequestMapping(value = "/training", method = RequestMethod.POST)
    public APIResult<TrainingItemVo> createTraining(@RequestBody TrainingDTO trainingDTO) {
        return productService.createTraining(trainingDTO);
    }

    @RequestMapping(value = "/training", method = RequestMethod.PUT)
    public APIResult<TrainingItemVo> updateTraining(@RequestBody TrainingDTO trainingDTO) {
        return productService.updateTraining(trainingDTO);
    }


    @RequestMapping(value = "/{productId}", method = RequestMethod.DELETE)
    public APIResult delete(@PathVariable("productId") String productId) {
        return productService.delete(productId);
    }
}
