package cn.rankin.cmsweb.controller.product;

import cn.rankin.cmsweb.assist.resolver.NeedUser;
import cn.rankin.cmsweb.entity.UserDetails;
import cn.rankin.cmsweb.service.product.TagService;
import cn.rankin.cmsweb.service.user.MerchantInfoSetService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.TagDTO;
import cn.rankin.common.utils.dto.search.TagSearchDTO;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.data.api.product.vo.TagVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping(value = "/tag")
public class TagController {

    @Autowired
    private TagService tagService;

    @Autowired
    private MerchantInfoSetService<TagVo> merchantInfoSetService;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<TagVo>> search(@NeedUser UserDetails user, TagSearchDTO searchDTO) {
        if (!user.isPlatForm()) {
            searchDTO.setMerchantId(user.getMerchantId());
        }
        APIResult<Page<TagVo>> apiResult = tagService.search(BeanUtil.convertToMap(searchDTO));
        if (apiResult.getSuccess()) {
            merchantInfoSetService.setMerchantInfo(apiResult.getData().getList());
        }
        return apiResult;
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<TagVo> getTag(@PathVariable("id") String id) {
        APIResult<TagVo> apiResult = tagService.getTag(id);
        if (apiResult.getSuccess()) {
            merchantInfoSetService.setMerchantInfo(apiResult.getData());
        }
        return apiResult;
    }

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<TagVo> create(@RequestBody TagDTO tagDTO) {
        APIResult<TagVo> apiResult = tagService.create(tagDTO);
        if (apiResult.getSuccess()) {
            merchantInfoSetService.setMerchantInfo(apiResult.getData());
        }
        return apiResult;
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<TagVo> update(@RequestBody TagDTO tagDTO) {
        APIResult<TagVo> apiResult = tagService.update(tagDTO);
        if (apiResult.getSuccess()) {
            merchantInfoSetService.setMerchantInfo(apiResult.getData());
        }
        return apiResult;
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult delete(@PathVariable("id") String id) {
        return tagService.delete(id);
    }
}
