package cn.rankin.cmsweb.controller.resource;

import cn.rankin.cmsweb.code.CmsWebAPICode;
import cn.rankin.cmsweb.service.resource.ResourceService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.dto.resource.ResourceDTO;
import cn.rankin.common.utils.dto.resource.ResourceSearchDTO;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.common.utils.vo.resource.ResourceVo;
import cn.rankin.data.api.resource.dto.AudioImgDTO;
import cn.rankin.data.api.resource.entity.AudioImg;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping(value = "/resource")
@Slf4j
public class ResourceController {

    @Autowired
    private ResourceService resourceService;

    @RequestMapping(value = "/list")
    public APIResult<Page<ResourceVo>> search(ResourceSearchDTO resourceSearchDTO) {
        return resourceService.search(BeanUtil.convertToMap(resourceSearchDTO));
    }

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<ResourceVo> create(@RequestBody ResourceDTO resourceDTO) {
        return resourceService.create(resourceDTO);
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<ResourceVo> update(@RequestBody ResourceDTO resourceDTO) {
        String id = resourceDTO.getId();
        if (null == id || id.isEmpty()){
            log.error("id Is Null");
            return APIResult.error(CmsWebAPICode.PARAMETER_ERROR);
        }
        return resourceService.update(id,resourceDTO);
    }


    //创建有声读物
    @RequestMapping(value="/audioImg",method = RequestMethod.POST)
    public APIResult<AudioImg> createAudioImg(@RequestBody AudioImgDTO dto) {
        AudioImg audioImg = resourceService.createAudioImg(dto);
        if(null == audioImg){
            return APIResult.error(CmsWebAPICode.PARAMETER_ERROR);
        }else{
            return APIResult.ok(audioImg);
        }
    }

    //修改有声读物
    @RequestMapping(value="/audioImg",method = RequestMethod.PUT)
    public APIResult<AudioImg> updateAudioImg(@RequestBody AudioImgDTO dto) {
        String id = dto.getId();
        if (null == id || id.isEmpty()){
            log.error("id Is Null");
            return APIResult.error(CmsWebAPICode.PARAMETER_ERROR);
        }

        AudioImg audioImg = resourceService.updateAudioImg(dto);
        if(null == audioImg){
            return APIResult.error(CmsWebAPICode.PARAMETER_ERROR);
        }else{
            return APIResult.ok(audioImg);
        }
    }
}
