package cn.rankin.cmsweb.controller.trade;

import cn.rankin.cmsweb.assist.resolver.NeedUser;
import cn.rankin.cmsweb.code.CmsWebAPICode;
import cn.rankin.cmsweb.entity.UserDetails;
import cn.rankin.cmsweb.service.buymsg.BuyMsgClient;
import cn.rankin.cmsweb.service.trade.order.OrderService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.cms.OrderDetailWebSearchDTO;
import cn.rankin.data.api.cms.OrderSearchWebDTO;
import cn.rankin.data.api.cms.OrderSnapshotWebSearchDTO;
import cn.rankin.data.api.trade.dto.OrderDTO;
import cn.rankin.data.api.trade.dto.OrderSendDTO;
import cn.rankin.data.api.trade.entity.OrderProductSnapshot;
import cn.rankin.data.api.trade.vo.OrderDetailVo;
import cn.rankin.data.api.trade.vo.OrderVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.Date;

@Slf4j
@RestController
@RequestMapping(value = "/order")
public class OrderController {

    @InitBinder
    protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
        binder.registerCustomEditor(Date.class,
                new CustomDateEditor(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"), true));
    }

    @Autowired
    private OrderService orderService;

    @Autowired
    private BuyMsgClient buyMsgClient;

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<OrderVo> create(@RequestBody OrderDTO orderDTO) {
        String userId = orderDTO.getUid();
        APIResult<OrderVo> orderResult = orderService.create(orderDTO);
        if (orderResult.getSuccess() && Boolean.TRUE.equals(orderDTO.getIsFromCart())) {
            APIResult<Boolean> buyMsgResult = buyMsgClient.delete(userId);
            if (!buyMsgResult.getSuccess()) {
                log.error("delete user buy msg error: {}", userId);
            }
        }
        return orderResult;
    }

    @RequestMapping(value = "/{orderId}", method = RequestMethod.GET)
    public APIResult<OrderVo> getOrder(@PathVariable String orderId) {
        return orderService.getOrder(orderId);
    }

    @RequestMapping(value = {"/list", ""}, method = RequestMethod.GET)
    public APIResult<Page<OrderVo>> search(@NeedUser UserDetails user, OrderSearchWebDTO searchDTO) {
        if (!user.isPlatForm()) {
            searchDTO.setMerchantId(user.getMerchantId());
        }
        return orderService.search(searchDTO);
    }

    @RequestMapping(value = "/pay/{orderId}", method = RequestMethod.POST)
    public APIResult<Boolean> pay(@PathVariable("orderId") String orderId) {
        return orderService.pay(orderId);
    }

    @RequestMapping(value = "/{orderId}", method = RequestMethod.DELETE)
    public APIResult<Boolean> cancel(@PathVariable("orderId") String orderId) {
        return orderService.cancel(orderId);
    }

    @RequestMapping(value = "/snapshot", method = RequestMethod.GET)
    public APIResult<Page<OrderProductSnapshot>> search(@NeedUser UserDetails user, OrderSnapshotWebSearchDTO searchDTO) {
        if (!user.isPlatForm()) {
            searchDTO.setMerchantId(user.getMerchantId());
        }
        return orderService.search(searchDTO);
    }

    @RequestMapping(value = "/detail", method = RequestMethod.GET)
    public APIResult<Page<OrderDetailVo>> search(@NeedUser UserDetails user, OrderDetailWebSearchDTO searchDTO) {
        if (user.isMerchant()) {
            searchDTO.setMerchantId(user.getMerchantId());
        }else if (user.isCp()) {
            searchDTO.setCpId(user.getMerchantId());
        }
        return orderService.search(searchDTO);
    }

    @RequestMapping(value = "/detail/{detailId}", method = RequestMethod.GET)
    public APIResult<OrderDetailVo> getDetailOrder(@PathVariable("detailId") String detailId) {
        return orderService.getDetailOrder(detailId);
    }

    @RequestMapping(value = "/send/{detailId}", method = RequestMethod.POST)
    public APIResult<Boolean> send(@NeedUser UserDetails user, @PathVariable("detailId") String detailId, @RequestBody OrderSendDTO sendDTO) {
        if (user.isCp() || user.isPlatForm()) {
            return orderService.send(detailId, sendDTO);
        }
        return APIResult.error(CmsWebAPICode.ACCESS_DENIED);
    }


    @RequestMapping(value = "/receive/{detailId}", method = RequestMethod.POST)
    public APIResult<Boolean> receive(@NeedUser UserDetails user, @PathVariable("detailId") String detailId) {
        if (user.isMerchant() || user.isPlatForm()) {
            return orderService.receive(detailId);
        }
        return APIResult.error(CmsWebAPICode.ACCESS_DENIED);
    }
}
