package cn.rankin.cmsweb.controller.user;

import cn.rankin.cmsweb.service.product.PosterServiceInterface;
import cn.rankin.cmsweb.service.product.RecommendService;
import cn.rankin.cmsweb.service.user.MerchantService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.entity.Poster;
import cn.rankin.data.api.product.vo.RecommendVo;
import cn.rankin.data.api.user.dto.MerchantDTO;
import cn.rankin.data.api.user.dto.MerchantSearchDTO;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.data.api.user.vo.MerchantVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@RestController
@RequestMapping(value = "/merchant")
public class MerchantController {

    @Autowired
    private MerchantService merchantService;

    @Autowired
    private RecommendService recommendService;

    @Autowired
    private PosterServiceInterface posterServiceInterface;

    @RequestMapping(value = {"", "/list"}, method = RequestMethod.GET)
    public APIResult<Page<MerchantVo>> getPage(MerchantSearchDTO searchDTO) {
        return merchantService.search(BeanUtil.convertToMap(searchDTO));
    }

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<MerchantVo> create(@RequestBody @Valid MerchantDTO merchantDTO) {
        return merchantService.create(merchantDTO);
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<MerchantVo> update(@RequestBody MerchantDTO merchantDTO) {
        return merchantService.update(merchantDTO);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@PathVariable String id) {
        return merchantService.delete(id);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<MerchantVo> getMerchant(@PathVariable String id) {
        return merchantService.getMerchant(id);
    }

    // 推荐位设置
    @RequestMapping(value = "/recommend/{merchantId}", method = RequestMethod.GET)
    public APIResult<List<RecommendVo>> getRecommend(@PathVariable("merchantId") String merchantId) {
        return recommendService.get(merchantId);
    }

    @RequestMapping(value = "/recommend/{merchantId}", method = RequestMethod.PUT)
    public APIResult<List<RecommendVo>> updateRecommend(@PathVariable("merchantId") String merchantId, @RequestBody List<String> recommendIdList) {
        return recommendService.put(merchantId, recommendIdList);
    }

    //海报设置
    @RequestMapping(value = "/poster/{merchantId}", method = RequestMethod.GET)
    public APIResult<List<Poster>> getPoster(@PathVariable("merchantId") String merchantId) {
        return posterServiceInterface.getPosters(merchantId);
    }

    @RequestMapping(value = "/poster/{merchantId}", method = RequestMethod.PUT)
    public APIResult<List<Poster>> updatePoster(@PathVariable("merchantId") String merchantId, @RequestBody List<String> posterIdList) {
        return posterServiceInterface.updatePosters(merchantId, posterIdList);
    }

}
