package cn.rankin.cmsweb.controller.user;

import cn.rankin.cmsweb.assist.resolver.NeedUser;
import cn.rankin.cmsweb.entity.UserDetails;
import cn.rankin.cmsweb.service.product.ProductService;
import cn.rankin.cmsweb.service.user.TerminalUserService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.vo.ProductVo;
import cn.rankin.data.api.user.dto.TerminalUserDTO;
import cn.rankin.data.api.user.dto.TerminalUserSearchDTO;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.data.api.user.vo.TerminalUserAuthVo;
import cn.rankin.data.api.user.vo.TerminalUserVo;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping(value = "/user")
public class TerminalUserController {

    @Autowired
    private TerminalUserService terminalUserService;

    @Autowired
    private ProductService productService;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<TerminalUserVo>> search(@NeedUser UserDetails user, TerminalUserSearchDTO searchDTO) {
        log.info("/list request params ,params = {}",JSON.toJSONString(searchDTO));

        if (!user.isPlatForm()) {
            String merchantId = user.getMerchantId();
            searchDTO.setMerchantId(merchantId);
        }

        return terminalUserService.search(BeanUtil.convertToMap(searchDTO));
    }

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<TerminalUserVo> create(@RequestBody TerminalUserDTO terminalUserDTO) {
        return terminalUserService.create(terminalUserDTO);
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<TerminalUserVo> update(@RequestBody TerminalUserDTO terminalUserDTO) {
        return terminalUserService.update(terminalUserDTO);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@PathVariable("id") String id) {
        return terminalUserService.delete(id);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<TerminalUserVo> getUser(@PathVariable("id") String id) {
        return terminalUserService.getUser(id);
    }

    @RequestMapping(value = "/auth/list", method = RequestMethod.GET)
    public APIResult<Page<TerminalUserAuthVo>> findAuthList(TerminalUserSearchDTO searchDTO) {
        log.info("/auth/list request params ,params = {}",JSON.toJSONString(searchDTO));


        Page<TerminalUserAuthVo> authVoPage = terminalUserService.findAuthList(BeanUtil.convertToMap(searchDTO));

        List<TerminalUserAuthVo> terminalUserAuthVos = authVoPage.getList();
        setProductInfo(terminalUserAuthVos);

        return APIResult.ok(authVoPage);
    }

    private void setProductInfo(List<TerminalUserAuthVo> terminalUserAuthVos) {
        if(CollectionUtils.isEmpty(terminalUserAuthVos)){
            return ;
        }

        List<ProductVo> productVos = productService.productList();
        if(CollectionUtils.isEmpty(productVos)){
            return ;
        }

        Map<String, ProductVo> productVoMap = ListUtil.convert(productVos, "pid", ProductVo.class);

        terminalUserAuthVos.forEach(vo -> {
            ProductVo productVo = productVoMap.get(vo.getPid());
            if(null != productVo){
                vo.setPCode(productVo.getCode());
                vo.setPName(productVo.getName());
            }else{
                log.info("not found product by pid ,pid={}",vo.getPid());
            }
        });

    }

}
