package cn.rankin.cmsweb.controller.user;

import cn.rankin.cmsweb.service.product.MerchantProductService;
import cn.rankin.cmsweb.service.product.ProductService;
import cn.rankin.cmsweb.service.user.UserRecommendService;
import cn.rankin.cmsweb.service.user.UserTagServiceInterface;
import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.product.entity.MerchantProduct;
import cn.rankin.data.api.product.vo.ProductVo;
import cn.rankin.data.api.product.vo.UserRecommendVo;
import cn.rankin.data.api.user.entity.UserRecommend;
import cn.rankin.data.api.user.entity.UserTag;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@RestController
@RequestMapping(value = "/user")
public class UserController {

    @Autowired
    private UserRecommendService userRecommendService;

    @Autowired
    private UserTagServiceInterface userTagService;

    @Autowired
    private ProductService productService;

    @Autowired
    private MerchantProductService merchantProductService;

    // 用户推荐位设置
    @RequestMapping(value = "/userRecommend/uid/{uid}", method = RequestMethod.GET)
    public APIResult<List<UserRecommendVo>> getUserRecommend(@PathVariable("uid") String uid) {
        if (StringUtils.isEmpty(uid)) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }
        List<UserRecommend> userRecommendList = userRecommendService.get(uid);

        setProductInfo(userRecommendList);
        return APIResult.ok(userRecommendList);
    }



    @RequestMapping(value = "/userRecommend/uid/{uid}", method = RequestMethod.PUT)
    public APIResult<List<UserRecommendVo>> updateUserRecommend(@PathVariable("uid") String uid, @RequestBody List<String> productIdList) {
        if (StringUtils.isEmpty(uid) || CollectionUtils.isEmpty(productIdList)) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }
        List<MerchantProduct> merchantProducts = new ArrayList<>();
        productIdList.forEach(pid -> {
            MerchantProduct merchantProduct = merchantProductService.findMerchantProductByPid(pid);
            if(null != merchantProduct){
                merchantProducts.add(merchantProduct);
            }else{
                log.info("Not found MerchantProduct by pid , pid={}",pid);
            }
        });

        List<UserRecommend> userRecommendList = userRecommendService.put(uid, merchantProducts);

        setProductInfo(userRecommendList);
        return APIResult.ok(userRecommendList);
    }

    //用户标签配置
    @RequestMapping(value = "/userTag/uid/{uid}", method = RequestMethod.GET)
    public APIResult<UserTag> getUserTagsByUid(@PathVariable("uid") String uid) {
        List<UserTag> userTags = userTagService.findUserTagByUid(uid);
        if (userTags != null && userTags.size() > 0) {
            /*userTags.forEach(userTag -> {
                List<UserTagProductRelation> relationList = userTagProductRelationServiceInterface.findByUserTagId(userTag.getId());
                List<String>  productIdList = new ArrayList<>();
                if(null != relationList && relationList.size() > 0){
                    relationList.forEach(relation -> {
                        productIdList.add(relation.getPid());
                    });
                    List<ItemVo> itemVoList = productService.findItemByPids(productIdList, merchantId);
                    userTag.setRecs(itemVoList);
                }

            });*/
            return APIResult.ok(userTags);
        }else{
            return APIResult.error(APICode.NOT_EXISTS);
        }
    }



    private void setProductInfo(List<UserRecommend> userRecommendList) {
        userRecommendList.forEach(userRecommend -> {
            ProductVo productVo = productService.findProductByPid(userRecommend.getPid());
            if(null != productVo){
                userRecommend.setName(productVo.getName());
                userRecommend.setCode(productVo.getCode());
            }else{
                log.info("Not found Product by pid , pid={}",userRecommend.getPid());
            }
        });
    }

}
