package cn.rankin.cmsweb.controller.user;

import cn.rankin.cmsweb.assist.resolver.NeedUser;
import cn.rankin.cmsweb.entity.UserDetails;
import cn.rankin.cmsweb.service.product.MerchantProductService;
import cn.rankin.cmsweb.service.product.TagService;
import cn.rankin.cmsweb.service.user.UserTagProductRelationServiceInterface;
import cn.rankin.cmsweb.service.user.UserTagServiceInterface;
import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.data.api.product.entity.MerchantProduct;
import cn.rankin.data.api.product.vo.TagVo;
import cn.rankin.data.api.user.dto.UserTagDTO;
import cn.rankin.data.api.user.dto.UserTagSearchDTO;
import cn.rankin.data.api.user.entity.UserTag;
import cn.rankin.data.api.user.entity.UserTagProductRelation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@RestController
@RequestMapping(value = "/userTag")
public class UserTagController {

    @Autowired
    private TagService tagService;

    @Autowired
    private UserTagServiceInterface userTagService;

    @Autowired
    private UserTagProductRelationServiceInterface userTagProductRelationServiceInterface;

    @Autowired
    private MerchantProductService merchantProductService;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<UserTag>> search(@NeedUser UserDetails user, UserTagSearchDTO searchDTO) {
        if (!user.isPlatForm()) {
            searchDTO.setUid(user.getId());
        }
        Page<UserTag> userTagPage = userTagService.search(BeanUtil.convertToMap(searchDTO));
        if (userTagPage != null) {
            return APIResult.ok(userTagPage);
        }else{
            return APIResult.error(APICode.NOT_EXISTS);
        }
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<UserTag> getUserTag(@PathVariable("id") String id) {
        UserTag userTag = userTagService.getUserTag(id);
        if (userTag != null) {
            List<UserTagProductRelation> relationList = userTagProductRelationServiceInterface.findByUserTagId(userTag.getId());
            if( !CollectionUtils.isEmpty(relationList)){
                //开始组装产品数据
                List<MerchantProduct> merchantProductList = new ArrayList<>();
                relationList.forEach(relation -> {
                    String pid = relation.getPid();
                    MerchantProduct merchantProduct = merchantProductService.findMerchantProductByPid(pid);
                    if(null != merchantProduct){
                        merchantProductList.add(merchantProduct);
                    }else{
                        log.error("not found MerchantProduct by pid, pid={}", pid);
                    }
                });

                userTag.setProductList(merchantProductList);
            }
            return APIResult.ok(userTag);
        }else{
            log.error("not found UserTag by id, id={}", id);
            return APIResult.error(APICode.NOT_EXISTS);
        }
    }

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<UserTag> create(@RequestBody UserTagDTO dto) {
        UserTag userTag = userTagService.create(dto);
        if (userTag != null) {
            return APIResult.ok(userTag);
        }else{
            return APIResult.error(APICode.NOT_EXISTS);
        }
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<UserTag> update(@RequestBody UserTagDTO dto) {
        if(StringUtils.isBlank(dto.getId())){
            return APIResult.error(APICode.PARAMETER_ERROR);
        }
        UserTag userTag = userTagService.update(dto);
        if (userTag != null) {
            log.error("update UserTag is error, id={}", dto.getId());
            return APIResult.ok(userTag);
        }else{
            return APIResult.error(APICode.NOT_EXISTS);
        }
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult delete(@PathVariable("id") String id) {
        if(StringUtils.isBlank(id)){
            return APIResult.error(APICode.PARAMETER_ERROR);
        }
        boolean flag = userTagService.delete(id);
        if(flag){
            return APIResult.ok();
        }else{
            log.error("delete UserTag is error, id={}", id);
            return APIResult.error(APICode.OPERATE_ERROR);
        }
    }

    @RequestMapping(value = "/copy" , method = RequestMethod.GET)
    public APIResult<UserTag> copyTag(@RequestParam("userTagId") String userTagId,@RequestParam("tagId") String tagId) {
        TagVo tagVo = tagService.getTagInfo(tagId);
        if(null == tagVo){
            log.error("{copyTag} not found Tag by id, id={}", tagId);
            return APIResult.error(APICode.NOT_EXISTS);
        }
        List<MerchantProduct> productList = tagVo.getProductList();
        List<String> productIdList = new ArrayList<>();

        productList.forEach(product -> productIdList.add(product.getPid()));

        UserTagDTO dto =new UserTagDTO();
        dto.setId(userTagId);
        dto.setProductList(productIdList);

        UserTag userTag = userTagService.update(dto);
        if (userTag == null) {
            log.error("{copyTag}  update UserTag is error, id={}", dto.getId());
            return APIResult.error(APICode.OPERATE_ERROR);
        }else{
            List<UserTagProductRelation> relationList = userTagProductRelationServiceInterface.findByUserTagId(userTag.getId());
            if( !CollectionUtils.isEmpty(relationList)){
                //开始组装产品数据
                List<MerchantProduct> merchantProductList = new ArrayList<>();
                relationList.forEach(relation -> {
                    String pid = relation.getPid();
                    MerchantProduct merchantProduct = merchantProductService.findMerchantProductByPid(pid);
                    if(null != merchantProduct){
                        merchantProductList.add(merchantProduct);
                    }else{
                        log.error("not found MerchantProduct by pid, pid={}", pid);
                    }
                });

                userTag.setProductList(merchantProductList);
            }
            return APIResult.ok(userTag);
        }
    }
}
