package cn.rankin.cmsweb.controller.user;

import cn.rankin.cmsweb.assist.resolver.NeedUser;
import cn.rankin.cmsweb.code.CmsWebAPICode;
import cn.rankin.cmsweb.entity.UserDetails;
import cn.rankin.cmsweb.service.user.WhiteUserService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.user.dto.WhiteUserDTO;
import cn.rankin.data.api.user.dto.WhiteUserSearchDTO;
import cn.rankin.data.api.user.vo.WhiteUserVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.Date;


@RestController
@RequestMapping(value = "/white/user")
public class WhiteUserController {

    @InitBinder
    protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
        binder.registerCustomEditor(Date.class,
                new CustomDateEditor(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"), true));
    }

    @Autowired
    private WhiteUserService whiteUserService;

    @RequestMapping(value = "/{userId}", method = RequestMethod.GET)
    public APIResult<WhiteUserVo> getWhiteUser(@PathVariable String userId) {
        return whiteUserService.findUserByUserId(userId);
    }

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<WhiteUserVo> create(@NeedUser UserDetails user, @RequestBody WhiteUserDTO whiteUserDTO) {
        if (!user.isPlatForm()) {
            return APIResult.error(CmsWebAPICode.ACCESS_DENIED);
        }
        return whiteUserService.addWhiteUser(whiteUserDTO);
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<WhiteUserVo> update(@NeedUser UserDetails user, @RequestBody WhiteUserDTO whiteUserDTO) {
        if (!user.isPlatForm()) {
            return APIResult.error(CmsWebAPICode.ACCESS_DENIED);
        }
        return whiteUserService.updateWhiteUser(whiteUserDTO);
    }

    @RequestMapping(method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@NeedUser UserDetails user, @RequestBody WhiteUserDTO whiteUserDTO) {
        if (!user.isPlatForm()) {
            return APIResult.error(CmsWebAPICode.ACCESS_DENIED);
        }
        return whiteUserService.deleteWhiteUser(whiteUserDTO);
    }

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<WhiteUserVo>> search(WhiteUserSearchDTO searchDTO) {
        return whiteUserService.search(searchDTO);
    }
}
