package cn.rankin.cmsweb.security;

import cn.rankin.cmsweb.entity.UserDetails;
import cn.rankin.cmsweb.service.user.CmsUserService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.user.vo.CmsUserVo;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Component;

import java.util.HashSet;
import java.util.Set;

@Component
public class CmsUserDetailsService implements UserDetailsService {

    protected final Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private CmsUserService cmsUserService;

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        logger.info("loadUserByUsername: {}", username);
        //TODO: 将来加入权限验证
        Set<GrantedAuthority> grantedAuthorities = new HashSet<>();

        APIResult<CmsUserVo> cmsUserApiResult = cmsUserService.findCmsUserByName(username);
        if (cmsUserApiResult == null || !cmsUserApiResult.getSuccess() || cmsUserApiResult.getData() == null) {
            throw new UsernameNotFoundException("用户不存在");
        }

        CmsUserVo cmsUser = cmsUserApiResult.getData();
        if (null != cmsUser.getResources()) {
            cmsUser.getResources().forEach(resouce -> {
                if (!StringUtils.isEmpty(resouce)) {
                    GrantedAuthority grantedAuthority = new SimpleGrantedAuthority(resouce.toUpperCase());
                    grantedAuthorities.add(grantedAuthority);
                }
            });
        }
        logger.info("grantedAuthorities --> {}", grantedAuthorities);
        return new UserDetails(grantedAuthorities,true,true,true,true,
                cmsUser.getId(), cmsUser.getName(), cmsUser.getAvatar(), cmsUser.getBirthday(), null, cmsUser.getDomain(), cmsUser.getMerchantId(), cmsUser.getMerchantName(), cmsUser.getMobile(), cmsUser.getNickName(), cmsUser.getPassword(), cmsUser.getIsAdmin());
    }
}