package cn.rankin.cmsweb.service.buymsg;

import cn.rankin.cmsweb.service.product.MerchantProductService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.model.BaseCode;
import cn.rankin.data.api.product.entity.Goods;
import cn.rankin.data.api.product.vo.MerchantProductVo;
import cn.rankin.data.api.user.entity.BuyMsg;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class BuyMsgService {

    @Autowired
    private BuyMsgClient buyMsgClient;

    @Autowired
    private MerchantProductService merchantProductService;

    public APIResult<List<MerchantProductVo>> getListByBuyMsg(String userId, String merchantId) {
        APIResult<List<BuyMsg>> buyMsgResult = buyMsgClient.getBuyMsg(userId);
        if (!buyMsgResult.getSuccess()) {
            return APIResult.error(new BaseCode(buyMsgResult.getCode(), buyMsgResult.getMessage()));
        }

        List<BuyMsg> buyMsgList = buyMsgResult.getData();
        List<String> productIdList = new ArrayList<>();
        Map<String, BuyMsg> buyMsgMap = new HashMap<>();
        for (BuyMsg buyMsg : buyMsgList) {
            String productId = buyMsg.getPid();
            String goodsId = buyMsg.getGoodsId();
            productIdList.add(productId);
            buyMsgMap.put(goodsId, buyMsg);
        }

        if (CollectionUtils.isEmpty(productIdList)) {
            return APIResult.ok(new ArrayList<>());
        }

        APIResult<List<MerchantProductVo>> productResult = merchantProductService.getList(productIdList, merchantId);
        if (!productResult.getSuccess()) {
            return APIResult.error(new BaseCode(productResult.getCode(), productResult.getMessage()));
        }

        List<MerchantProductVo> merchantProductVoList = productResult.getData();
        for (MerchantProductVo merchantProductVo : merchantProductVoList) {
            List<Goods> goodsList = merchantProductVo.getGoods();
            if (CollectionUtils.isEmpty(goodsList)) {
                continue;
            }
            for (Goods goods : goodsList) {
                String goodsId = goods.getId();
                if (buyMsgMap.containsKey(goodsId)) {
                    goods.setIsInCart(Boolean.TRUE);
                }else {
                    goods.setIsInCart(Boolean.FALSE);
                }
            }
        }

        return APIResult.ok(merchantProductVoList);
    }
}
