package cn.rankin.cmsweb.service.product.proxy;

import cn.rankin.cmsweb.service.product.WareService;
import cn.rankin.cmsweb.service.resource.ResourceService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.CourseWareDTO;
import cn.rankin.common.utils.dto.search.SearchDTO;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.data.api.product.entity.CourseWare;
import cn.rankin.data.api.product.vo.CourseWareVo;
import cn.rankin.common.utils.vo.resource.ResourceVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class WareServiceProxy {

    @Autowired
    private WareService wareService;

    @Autowired
    private ResourceService resourceService;

    public APIResult<Page<CourseWareVo>> search(SearchDTO searchDTO) {
        Map<String, Object> params = BeanUtil.convertToMap(searchDTO);
        APIResult<Page<CourseWare>> result = wareService.search(params);
        Page<CourseWare> page = result.getData();

        Page<CourseWareVo> voPage = new Page(page.getTotalSize(), page.getPageNo(), page.getPageSize());
        if (result.getSuccess()) {
            List<CourseWare> courseWareList = result.getData().getList();
            voPage.setList(setWareUrl(courseWareList));
        }
        return APIResult.ok(voPage);
    }

    public APIResult<CourseWareVo> create(CourseWareDTO courseWareDTO) {
        CourseWareVo courseWareVo = null;
        APIResult<CourseWare> result = wareService.create(courseWareDTO);
        if (result.getSuccess()) {
            CourseWare courseWare = result.getData();
            courseWareVo = setWareUrl(courseWare);
        }
        return APIResult.ok(courseWareVo);
    }

    public APIResult<CourseWareVo> update(CourseWareDTO courseWareDTO) {
        CourseWareVo courseWareVo = null;
        APIResult<CourseWare> result = wareService.update(courseWareDTO);
        if (result.getSuccess()) {
            CourseWare courseWare = result.getData();
            courseWareVo = setWareUrl(courseWare);
        }
        return APIResult.ok(courseWareVo);
    }

    public APIResult delete(String id) {
        return wareService.delete(id);
    }

    public APIResult<CourseWareVo> getWare(String id) {
        CourseWareVo courseWareVo = null;
        APIResult<CourseWare> result = wareService.getWare(id);
        if (result.getSuccess()) {
            CourseWare courseWare = result.getData();
            courseWareVo = setWareUrl(courseWare);
        }
        return APIResult.ok(courseWareVo);
    }

    public Map<String, ResourceVo> getResourceMap(List<String> idList) {
        Map<String, ResourceVo> resourceVoMap = new HashMap<>();
        if (CollectionUtils.isEmpty(idList)) {
            return resourceVoMap;
        }

        APIResult<Map<String, ResourceVo>> apiResult = resourceService.getBatch(idList);
        if (!apiResult.getSuccess()) {
            log.warn("get resource map error!");
            return resourceVoMap;
        }else if (apiResult.getData() == null) {
            log.warn("get resource map empty");
            return resourceVoMap;
        }

        return apiResult.getData();
    }

    public List<CourseWareVo> setWareUrl(List<CourseWare> courseWareList) {
        List<CourseWareVo> courseWareVoList = new ArrayList<>();
        if (CollectionUtils.isEmpty(courseWareList)) {
            return courseWareVoList;
        }

        List<String> idList = new ArrayList<>();
        courseWareList.forEach(e -> {
            List<String> resourceIdList = e.getResourceList();
            if (!CollectionUtils.isEmpty(resourceIdList)) {
                idList.addAll(resourceIdList);
            }
        });

        Map<String, ResourceVo> resourceVoMap = getResourceMap(idList);

        for (CourseWare courseWare : courseWareList) {
            CourseWareVo courseWareVo = new CourseWareVo();
            BeanUtils.copyProperties(courseWare, courseWareVo);
            List<String> resourceIdList = courseWare.getResourceList();
            if (CollectionUtils.isEmpty(resourceIdList)) {
                courseWareVoList.add(courseWareVo);
                continue;
            }

            List<ResourceVo> resourceVoList = new ArrayList<>();
            for (String resourceId : resourceIdList) {
                ResourceVo resourceVo = resourceVoMap.get(resourceId);
                if (resourceVo != null) {
                    resourceVoList.add(resourceVo);
                }
            }
            courseWareVo.setResourceList(resourceVoList);
            courseWareVoList.add(courseWareVo);
        }

        return courseWareVoList;
    }

    public CourseWareVo setWareUrl(CourseWare courseWare) {
        CourseWareVo courseWareVo = new CourseWareVo();
        BeanUtils.copyProperties(courseWare, courseWareVo);

        if (courseWare == null || courseWare.getResourceList() == null) {
            return courseWareVo;
        }

        List<String> resourceIdList = courseWare.getResourceList();
        if (CollectionUtils.isEmpty(resourceIdList)) {
            return courseWareVo;
        }

        Map<String, ResourceVo> resourceVoMap = getResourceMap(resourceIdList);

        List<ResourceVo> resourceVoList = new ArrayList<>();
        for (String resourceId : resourceIdList) {
            ResourceVo resourceVo = resourceVoMap.get(resourceId);
            if (resourceVo != null) {
                resourceVoList.add(resourceVo);
            }

        }

        courseWareVo.setResourceList(resourceVoList);

        return courseWareVo;
    }

}
