package cn.rankin.cmsweb.service.trade.order;

import cn.rankin.cmsweb.code.CmsWebAPICode;
import cn.rankin.cmsweb.service.product.GoodsService;
import cn.rankin.cmsweb.service.product.ProductService;
import cn.rankin.cmsweb.service.user.CampusService;
import cn.rankin.cmsweb.service.user.MerchantService;
import cn.rankin.cmsweb.service.user.TerminalUserService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.model.BaseCode;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.LedgerTypeEnum;
import cn.rankin.common.utils.enums.ProductTypeEnum;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.data.api.cms.OrderDetailWebSearchDTO;
import cn.rankin.data.api.cms.OrderSearchWebDTO;
import cn.rankin.data.api.cms.OrderSnapshotWebSearchDTO;
import cn.rankin.data.api.product.entity.Goods;
import cn.rankin.data.api.product.vo.GoodsVo;
import cn.rankin.data.api.product.vo.ProductVo;
import cn.rankin.data.api.trade.dto.*;
import cn.rankin.data.api.trade.entity.OrderDetail;
import cn.rankin.data.api.trade.entity.OrderGoods;
import cn.rankin.data.api.trade.entity.Order;
import cn.rankin.data.api.trade.entity.OrderProductSnapshot;
import cn.rankin.data.api.trade.vo.OrderDetailVo;
import cn.rankin.data.api.trade.vo.OrderGoodsVo;
import cn.rankin.data.api.trade.vo.OrderVo;
import cn.rankin.data.api.user.dto.CampusSearchDTO;
import cn.rankin.data.api.user.dto.PayDTO;
import cn.rankin.data.api.user.vo.CampusVo;
import cn.rankin.data.api.user.vo.TerminalUserVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.rankin.cmsweb.utils.DTOConverter.convert;

@Slf4j
@Service
public class OrderService {

    @Autowired
    private OrderClient orderClient;

    @Autowired
    private GoodsService goodsService;

    @Autowired
    private TerminalUserService terminalUserService;

    @Autowired
    private CampusService campusService;

    @Autowired
    private PayingService payingService;

    @Autowired
    private MerchantService merchantService;

    @Autowired
    private ProductService productService;

    // 支付
    public APIResult<Boolean> pay(String orderId) {
        APIResult<Order> userOrderAPIResult = orderClient.getOrder(orderId);
        Order order = userOrderAPIResult.getData();
        if (order == null || !userOrderAPIResult.getSuccess()) {
            log.error("查询订单错误, orderId={}, {}", orderId, userOrderAPIResult.getMessage());
            return APIResult.error(new BaseCode(userOrderAPIResult.getCode(), userOrderAPIResult.getMessage()));
        }

        String uid = order.getUid();
        APIResult<TerminalUserVo> userVoAPIResult = terminalUserService.getUser(uid);
        TerminalUserVo userVo = userVoAPIResult.getData();
        if (!userVoAPIResult.getSuccess() || userVo == null) {
            log.error("查询用户错误, orderId={}, {}", orderId, userVoAPIResult.getMessage());
            return APIResult.error(new BaseCode(userVoAPIResult.getCode(), userVoAPIResult.getMessage()));
        }

        // 开始扣款
        String merchantId = userVo.getMerchantId();
        PayDTO payDTO = new PayDTO();
        payDTO.setQuantity(order.getFinalPrice());
        payDTO.setType(LedgerTypeEnum.COURSE);
        payDTO.setReceiptId(order.getId());
        payDTO.setMerchantId(merchantId);

        APIResult<Boolean> payAPIResult = payingService.pay(payDTO);
        if (!payAPIResult.getSuccess()) {
            log.error("扣款失败, orderId={}, {}", orderId, payAPIResult.getMessage());
            return APIResult.error(new BaseCode(payAPIResult.getCode(), payAPIResult.getMessage()));
        }

        // 更新订单状态
        APIResult<Boolean> orderAPIResult = orderClient.pay(orderId);
        if (!orderAPIResult.getSuccess()) {
            return APIResult.error(new BaseCode(orderAPIResult.getCode(), orderAPIResult.getMessage()));
        }

        // 生成交易商品快照, 还要插入鉴权记录, 放入循环任务中进行

        return APIResult.ok();
    }

    // 取消
    public APIResult<Boolean> cancel(String orderId) {
        return orderClient.cancel(orderId);
    }

    // 发货
    public APIResult<Boolean> send(String detailId, OrderSendDTO sendDTO) {
        return orderClient.send(detailId, sendDTO);
    }

    // 收货
    public APIResult<Boolean> receive(String detailId) {
        return orderClient.receive(detailId);
    }

    // 查询详单
    public APIResult<OrderDetailVo> getDetailOrder(String detailId) {
        APIResult<OrderDetail> orderAPIResult = orderClient.getDetailOrder(detailId);
        if (!orderAPIResult.getSuccess()) {
            return APIResult.error(new BaseCode(orderAPIResult.getCode(), orderAPIResult.getMessage()));
        }

        OrderDetail orderDetail = orderAPIResult.getData();
        OrderDetailVo orderDetailVo = convert(orderDetail);

        List<OrderGoods> orderGoodsList = orderDetail.getGoods();
        if (CollectionUtils.isEmpty(orderGoodsList)) {
            return APIResult.ok(orderDetailVo);
        }

        String uid = orderDetailVo.getUid();
        APIResult<TerminalUserVo> userAPIResult = terminalUserService.getUser(uid);
        TerminalUserVo terminalUserVo = userAPIResult.getData();

        if (userAPIResult.getSuccess()) {
            orderDetailVo.setMerchantId(terminalUserVo.getMerchantId());
            orderDetailVo.setMerchantName(terminalUserVo.getMerchantName());
            orderDetailVo.setUserCode(terminalUserVo.getCode());
            orderDetailVo.setClassroomName(terminalUserVo.getName());
        }

        String campusId = terminalUserVo.getCampusId();
        APIResult<CampusVo> campusAPIResult = campusService.getCampus(campusId);
        CampusVo campusVo = campusAPIResult.getData();

        if (campusVo != null) {
            orderDetailVo.setProvinceCode(campusVo.getProvinceCode());
            orderDetailVo.setCityName(campusVo.getCityName());
            orderDetailVo.setZoneName(campusVo.getZoneName());
        }

        orderDetailVo.setGoods(this.toOrderGoodsVo(orderGoodsList));
        return APIResult.ok(orderDetailVo);
    }

    // 创建订单
    public APIResult<OrderVo> create(OrderDTO orderDTO) {
        String uid = orderDTO.getUid();
        APIResult<TerminalUserVo> userVoAPIResult = terminalUserService.getUser(uid);
        if (!userVoAPIResult.getSuccess()) {
            return APIResult.error(CmsWebAPICode.error("用户不存在"));
        }

        TerminalUserVo terminalUserVo = userVoAPIResult.getData();

        String merchantId = terminalUserVo.getMerchantId();
        orderDTO.setMerchantId(merchantId);

        List<OrderGoodsDTO> orderGoodsDTOList = orderDTO.getGoods();
        if (CollectionUtils.isEmpty(orderGoodsDTOList)) {
            return APIResult.error(CmsWebAPICode.error("商品为空"));
        }

        List<String> goodsIdList = new ArrayList<>();
        Map<String, Integer> goodsCountMap = new HashMap<>();
        orderGoodsDTOList.forEach(goods -> {
            String goodsId = goods.getGoodsId();
            goodsIdList.add(goodsId);
            goodsCountMap.put(goodsId, goods.getQuantity());
        });

        APIResult<List<GoodsVo>> goodsVoAPIResult = goodsService.findByIds(goodsIdList);
        if (!goodsVoAPIResult.getSuccess()) {
            return APIResult.error(CmsWebAPICode.error("查询商品错误"));
        }

        List<GoodsVo> goodsVoList = goodsVoAPIResult.getData();
        if (goodsVoList.size() != orderGoodsDTOList.size()) {
            return APIResult.error(CmsWebAPICode.error("包含不可售卖的商品"));
        }

        BigDecimal totalPrice = BigDecimal.ZERO;
        List<OrderGoodsDTO> finalGoodsList = new ArrayList<>();
        for (GoodsVo goodsVo : goodsVoList) {
            String goodsMerchantId = goodsVo.getMerchantId();
            if (!terminalUserVo.getMerchantId().equals(goodsMerchantId)) {
                return APIResult.error(CmsWebAPICode.error("不可以购买其他渠道商品"));
            }
//            if (goodsVo.getType().equals(ProductTypeEnum.PACKAGE)) {
//                return APIResult.error(CmsWebAPICode.error("包含课程包"));
//            }
            String goodsId = goodsVo.getId();
            Integer count = goodsCountMap.get(goodsId);
            OrderGoodsDTO orderGoodsDTO = new OrderGoodsDTO();
            orderGoodsDTO.setGoodsId(goodsId);
            orderGoodsDTO.setPid(goodsVo.getPid());
            orderGoodsDTO.setType(goodsVo.getType());
            orderGoodsDTO.setQuantity(count);
            finalGoodsList.add(orderGoodsDTO);
            totalPrice = totalPrice.add(goodsVo.getMerchantPrice().multiply(new BigDecimal(count)));
        }
        orderDTO.setFinalPrice(totalPrice.subtract(orderDTO.getAdjustPrice()));
        orderDTO.setOriginPrice(totalPrice);
        orderDTO.setGoods(finalGoodsList);

        APIResult userOrderAPIResult = orderClient.create(orderDTO);
        return userOrderAPIResult;
    }

    public APIResult<OrderVo> getOrder(String orderId) {
        APIResult<Order> orderAPIResult = orderClient.getOrder(orderId);
        if (!orderAPIResult.getSuccess()) {
            return APIResult.error(new BaseCode(orderAPIResult.getCode(), orderAPIResult.getMessage()));
        }

        Order order = orderAPIResult.getData();
        OrderVo orderVo = convert(order);
        List<OrderGoods> orderGoodsList = order.getGoods();
        if (CollectionUtils.isEmpty(orderGoodsList)) {
            return APIResult.ok(orderVo);
        }

        String uid = order.getUid();
        APIResult<TerminalUserVo> userAPIResult = terminalUserService.getUser(uid);
        TerminalUserVo terminalUserVo = userAPIResult.getData();

        if (userAPIResult.getSuccess()) {
            orderVo.setMerchantId(terminalUserVo.getMerchantId());
            orderVo.setMerchantName(terminalUserVo.getMerchantName());
            orderVo.setUserCode(terminalUserVo.getCode());
            orderVo.setClassroomName(terminalUserVo.getName());
        }

        String campusId = terminalUserVo.getCampusId();
        APIResult<CampusVo> campusAPIResult = campusService.getCampus(campusId);
        CampusVo campusVo = campusAPIResult.getData();

        if (campusVo != null) {
            orderVo.setProvinceCode(campusVo.getProvinceCode());
            orderVo.setCityName(campusVo.getCityName());
            orderVo.setZoneName(campusVo.getZoneName());
        }

        orderVo.setGoods(this.toOrderGoodsVo(orderGoodsList));
        return APIResult.ok(orderVo);
    }

    public List<OrderGoodsVo> toOrderGoodsVo(List<OrderGoods> orderGoodsList) {
        if (CollectionUtils.isEmpty(orderGoodsList)) {
            return new ArrayList<>();
        }
        List<String> goodsIdList = new ArrayList<>();
        List<String> productIdList = new ArrayList<>();
        orderGoodsList.forEach(orderGoods -> {
            goodsIdList.add(orderGoods.getGoodsId());
            productIdList.add(orderGoods.getPid());
        });

        APIResult<List<GoodsVo>> goodsAPIResult = goodsService.findByIds(goodsIdList);
        if (!goodsAPIResult.getSuccess()) {
            log.error("goods api error: {}", goodsAPIResult.getMessage());
            return null;
        }

        APIResult<List<ProductVo>> productAPIResult = productService.findProductByIds(productIdList);
        if (!productAPIResult.getSuccess()) {
            log.error("product api error: {}", productAPIResult.getMessage());
            return null;
        }

        List<ProductVo> productVoList = productAPIResult.getData();
        Map<String, ProductVo> productVoMap = new HashMap<>();
        for (ProductVo productVo : productVoList) {
            productVoMap.put(productVo.getPid(), productVo);
        }

        List<GoodsVo> goodsVoList = goodsAPIResult.getData();
        Map<String, GoodsVo> goodsVoMap = new HashMap<>();
        for (GoodsVo goodsVo : goodsVoList) {
            goodsVoMap.put(goodsVo.getId(), goodsVo);
        }

        List<OrderGoodsVo> orderGoodsVoList = new ArrayList<>();
        for (OrderGoods orderGoods : orderGoodsList) {
            String goodsId = orderGoods.getGoodsId();
            String productId = orderGoods.getPid();

            OrderGoodsVo orderGoodsVo = new OrderGoodsVo();
            BeanUtils.copyProperties(orderGoods, orderGoodsVo);

            GoodsVo goodsVo = goodsVoMap.get(goodsId);
            if (goodsVo == null) {
                log.error("goods not exists, goodsId={}", goodsId);
                continue;
            }
            BeanUtils.copyProperties(goodsVo, orderGoodsVo);
            goodsVo.setId(orderGoods.getId());

            ProductVo productVo = productVoMap.get(productId);
            if (productVo != null) {
                orderGoodsVo.setName(productVo.getName());
                orderGoodsVo.setCode(productVo.getCode());
            }

            orderGoodsVoList.add(orderGoodsVo);
        }
        return orderGoodsVoList;
    }

    public APIResult<Page<OrderVo>> search(OrderSearchWebDTO searchWebDTO) {
        String userCode = searchWebDTO.getUserCode();
        String campusCode = searchWebDTO.getCampusCode();
        String campusName = searchWebDTO.getCampusName();
        String merchantId = searchWebDTO.getMerchantId();

        List<String> userIdList = new ArrayList<>();

        List<TerminalUserVo> terminalUserVoList = findTerminalUsers(userCode, campusCode, campusName, merchantId);
        terminalUserVoList.forEach(terminalUserVo -> userIdList.add(terminalUserVo.getId()));

        OrderSearchDTO searchDTO = new OrderSearchDTO();
        searchDTO.setPageNo(searchWebDTO.getPageNo());
        searchDTO.setPageSize(searchWebDTO.getPageSize());
        searchDTO.setBeginTime(searchWebDTO.getBeginTime());
        searchDTO.setEndTime(searchWebDTO.getEndTime());
        searchDTO.setStatus(searchWebDTO.getStatus());
        searchDTO.setUid(userIdList);
        searchDTO.setMerchantId(merchantId);

        APIResult<Page<Order>> orderAPIResult = orderClient.search(searchDTO);
        if (!orderAPIResult.getSuccess()) {
            log.error("order query error: {}", orderAPIResult.getMessage());
            return APIResult.error(new BaseCode(orderAPIResult.getCode(), orderAPIResult.getMessage()));
        }

        Page<Order> userOrderPage = orderAPIResult.getData();
        Page<OrderVo> page = new Page<>(userOrderPage.getTotalSize(), userOrderPage.getPageNo(), userOrderPage.getPageSize());

        List<Order> orderList = orderAPIResult.getData().getList();
        List<OrderVo> orderVoList = new ArrayList<>();
        if (CollectionUtils.isEmpty(orderList)) {
            return APIResult.ok(page);
        }

        if (CollectionUtils.isEmpty(userIdList)) {
            orderList.forEach(order -> userIdList.add(order.getUid()));
            APIResult<List<TerminalUserVo>> userAPIResult = terminalUserService.findByIds(userIdList);
            if (!userAPIResult.getSuccess()) {
                return APIResult.error(new BaseCode(userAPIResult.getCode(), userAPIResult.getMessage()));
            }
            terminalUserVoList = userAPIResult.getData();
        }

        // 用户信息
        Map<String, TerminalUserVo> terminalUserVoMap = new HashMap<>();
        // 加载校区信息
        List<String> campusIdList = new ArrayList<>();
        for (TerminalUserVo terminalUserVo : terminalUserVoList) {
            String campusId = terminalUserVo.getCampusId();
            if (!campusIdList.contains(campusId)) {
                campusIdList.add(campusId);
            }
            terminalUserVoMap.put(terminalUserVo.getId(), terminalUserVo);
        }

        Map<String, CampusVo> campusVoMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(campusIdList)) {
            APIResult<Map<String, CampusVo>> campusAPIResult = campusService.findByIds(campusIdList);
            if (!campusAPIResult.getSuccess()) {
                log.error("load campus info error");
            }else {
                campusVoMap = campusAPIResult.getData();
            }
        }

        for (Order order : orderList) {
            OrderVo orderVo = convert(order);
            String uid = orderVo.getUid();

            TerminalUserVo terminalUserVo = terminalUserVoMap.get(uid);
            if (terminalUserVo == null) {
                log.error("用户不存在: {}", uid);
                orderVoList.add(orderVo);
                continue;
            }
            orderVo.setMerchantId(terminalUserVo.getMerchantId());
            orderVo.setMerchantName(terminalUserVo.getMerchantName());
            orderVo.setUserCode(terminalUserVo.getCode());
            orderVo.setClassroomName(terminalUserVo.getName());

            String campusId = terminalUserVo.getCampusId();
            CampusVo campusVo = campusVoMap.get(campusId);

            if (campusVo == null) {
                log.error("campus is null, uid={}", uid);
                orderVoList.add(orderVo);
                continue;
            }
            orderVo.setProvinceCode(campusVo.getProvinceCode());
            orderVo.setCityName(campusVo.getCityName());
            orderVo.setZoneName(campusVo.getZoneName());

            orderVoList.add(orderVo);
        }

        page.setList(orderVoList);
        return APIResult.ok(page);
    }

    // 查拆单
    public APIResult<Page<OrderDetailVo>> search(OrderDetailWebSearchDTO searchWebDTO) {
        String userCode = searchWebDTO.getUserCode();
        String campusCode = searchWebDTO.getCampusCode();
        String campusName = searchWebDTO.getCampusName();
        String merchantId = searchWebDTO.getMerchantId();

        List<String> userIdList = new ArrayList<>();

        List<TerminalUserVo> terminalUserVoList = findTerminalUsers(userCode, campusCode, campusName, merchantId);
        terminalUserVoList.forEach(terminalUserVo -> userIdList.add(terminalUserVo.getId()));

        OrderDetailSearchDTO searchDTO = new OrderDetailSearchDTO();
        searchDTO.setPageNo(searchWebDTO.getPageNo());
        searchDTO.setPageSize(searchWebDTO.getPageSize());
        searchDTO.setBeginTime(searchWebDTO.getBeginTime());
        searchDTO.setEndTime(searchWebDTO.getEndTime());
        searchDTO.setStatus(searchWebDTO.getStatus());
        searchDTO.setUid(userIdList);
        searchDTO.setMerchantId(merchantId);
        searchDTO.setCpId(searchWebDTO.getCpId());

        APIResult<Page<OrderDetail>> orderAPIResult = orderClient.searchDetail(searchDTO);
        if (!orderAPIResult.getSuccess()) {
            log.error("order query error: {}", orderAPIResult.getMessage());
            return APIResult.error(new BaseCode(orderAPIResult.getCode(), orderAPIResult.getMessage()));
        }

        Page<OrderDetail> orderPage = orderAPIResult.getData();
        Page<OrderDetailVo> page = new Page<>(orderPage.getTotalSize(), orderPage.getPageNo(), orderPage.getPageSize());

        List<OrderDetail> orderDetailList = orderAPIResult.getData().getList();
        if (CollectionUtils.isEmpty(orderDetailList)) {
            return APIResult.ok(page);
        }

        if (CollectionUtils.isEmpty(userIdList)) {
            orderDetailList.forEach(order -> userIdList.add(order.getUid()));
            APIResult<List<TerminalUserVo>> userAPIResult = terminalUserService.findByIds(userIdList);
            if (!userAPIResult.getSuccess()) {
                return APIResult.error(new BaseCode(userAPIResult.getCode(), userAPIResult.getMessage()));
            }
            terminalUserVoList = userAPIResult.getData();
        }

        // 用户信息
        Map<String, TerminalUserVo> terminalUserVoMap = new HashMap<>();
        // 加载校区信息
        List<String> campusIdList = new ArrayList<>();
        for (TerminalUserVo terminalUserVo : terminalUserVoList) {
            String campusId = terminalUserVo.getCampusId();
            if (!campusIdList.contains(campusId)) {
                campusIdList.add(campusId);
            }
            terminalUserVoMap.put(terminalUserVo.getId(), terminalUserVo);
        }

        Map<String, CampusVo> campusVoMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(campusIdList)) {
            APIResult<Map<String, CampusVo>> campusAPIResult = campusService.findByIds(campusIdList);
            if (!campusAPIResult.getSuccess()) {
                log.error("load campus info error");
            }else {
                campusVoMap = campusAPIResult.getData();
            }
        }

        List<OrderDetailVo> orderDetailVoList = new ArrayList<>();
        for (OrderDetail order : orderDetailList) {
            OrderDetailVo orderDetailVo = convert(order);
            String uid = orderDetailVo.getUid();

            TerminalUserVo terminalUserVo = terminalUserVoMap.get(uid);
            if (terminalUserVo == null) {
                log.error("用户不存在: {}", uid);
                orderDetailVoList.add(orderDetailVo);
                continue;
            }
            orderDetailVo.setMerchantId(terminalUserVo.getMerchantId());
            orderDetailVo.setMerchantName(terminalUserVo.getMerchantName());
            orderDetailVo.setUserCode(terminalUserVo.getCode());
            orderDetailVo.setClassroomName(terminalUserVo.getName());

            String campusId = terminalUserVo.getCampusId();
            CampusVo campusVo = campusVoMap.get(campusId);

            if (campusVo == null) {
                log.error("campus is null, uid={}", uid);
                continue;
            }
            orderDetailVo.setProvinceCode(campusVo.getProvinceCode());
            orderDetailVo.setCityName(campusVo.getCityName());
            orderDetailVo.setZoneName(campusVo.getZoneName());

            orderDetailVoList.add(orderDetailVo);
        }

        page.setList(orderDetailVoList);
        return APIResult.ok(page);
    }

    public APIResult<Page<OrderProductSnapshot>> search(OrderSnapshotWebSearchDTO searchDTO) {
        return orderClient.searchSnapshot(convert(searchDTO));
    }

    // 根据搜索条件查找用户
    public List<TerminalUserVo> findTerminalUsers(String userCode, String campusCode, String campusName, String merchantId) {
        List<TerminalUserVo> terminalUserVoList = new ArrayList<>();

        if (!StringUtils.isEmpty(userCode)) {
            // 按用户查询
            Map<String, Object> searchDTO = new HashMap<>();
            searchDTO.put("code", userCode);
            searchDTO.put("merchantId", merchantId);
            APIResult<List<TerminalUserVo>> userAPIResult = terminalUserService.findByCondition(searchDTO);
            if (!userAPIResult.getSuccess()) {
                log.error("load user id list error");
                return terminalUserVoList;
            }
            terminalUserVoList = userAPIResult.getData();

        } else if (!StringUtils.isEmpty(campusCode) || !StringUtils.isEmpty(campusName)) {
            // 按校区查询
            CampusSearchDTO campusSearchDTO = new CampusSearchDTO();
            campusSearchDTO.setCode(campusCode);
            campusSearchDTO.setName(campusName);
            campusSearchDTO.setMerchantId(merchantId);
            APIResult<List<CampusVo>> campusAPIResult = campusService.findByCondition(BeanUtil.convertToMap(campusSearchDTO));
            if (!campusAPIResult.getSuccess()) {
                log.error("load user id list error when find campus");
                return terminalUserVoList;
            }

            List<CampusVo> campusVoList = campusAPIResult.getData();
            if (CollectionUtils.isEmpty(campusVoList)) {
                return terminalUserVoList;
            }

            List<String> campusIdList = new ArrayList<>();
            for (CampusVo campusVo : campusVoList) {
                campusIdList.add(campusVo.getId());
            }

            APIResult<List<TerminalUserVo>> userAPIResult = terminalUserService.findByCampusIds(campusIdList);
            if (!userAPIResult.getSuccess()) {
                log.error("load user id list error");
                return terminalUserVoList;
            }
            terminalUserVoList = userAPIResult.getData();
        }
        return terminalUserVoList;
    }
}
