package cn.rankin.cmsweb.service.user;

import cn.rankin.cmsweb.service.user.MerchantService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.user.vo.MerchantVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class MerchantInfoSetService<V> {

    @Autowired
    private MerchantService merchantService;

    public Map<String, MerchantVo> findByIds(List<String> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            log.error("merchant idList is empty!");
            return new HashMap<>();
        }

        APIResult<Map<String, MerchantVo>> merchantAPIResult = merchantService.findByIds(idList);
        if (!merchantAPIResult.getSuccess()) {
            log.error("api error: {}", merchantAPIResult.getMessage());
            return new HashMap<>();
        }

        return merchantAPIResult.getData();
    }

    public MerchantVo findById(String merchantId) {
        APIResult<MerchantVo> merchantVoAPIResult = merchantService.getMerchant(merchantId);
        if (!merchantVoAPIResult.getSuccess()) {
            log.error("api error: {}", merchantVoAPIResult.getMessage());
            return null;
        }
        return merchantVoAPIResult.getData();
    }

    public void setMerchantInfo(V v) {
        try {
            Class clazz = v.getClass();
            Field idField = clazz.getDeclaredField("merchantId");
            idField.setAccessible(true);
            String merchantId = (String) idField.get(v);
            idField.setAccessible(false);

            Field nameField = clazz.getDeclaredField("merchantName");
            nameField.setAccessible(true);
            MerchantVo merchantVo = findById(merchantId);
            nameField.set(v, merchantVo.getName());
            nameField.setAccessible(false);

        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    public void setMerchantInfo(List<V> list) {
        if (CollectionUtils.isEmpty(list)) return;

        List<String> merchantIdList = new ArrayList<>();
        for (V v : list) {
            try {
                Class clazz = v.getClass();
                Field idField = clazz.getDeclaredField("merchantId");
                idField.setAccessible(true);
                String merchantId = (String) idField.get(v);
                idField.setAccessible(false);
                merchantIdList.add(merchantId);
            } catch (Exception e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }

        Map<String, MerchantVo> merchantVoMap = findByIds(merchantIdList);
        for (V v : list) {
            try {
                Class clazz = v.getClass();
                Field idField = clazz.getDeclaredField("merchantId");
                idField.setAccessible(true);
                String merchantId = (String) idField.get(v);
                idField.setAccessible(false);

                MerchantVo merchantVo = merchantVoMap.get(merchantId);
                Field nameField = clazz.getDeclaredField("merchantName");
                nameField.setAccessible(true);
                nameField.set(v, merchantVo.getName());
                nameField.setAccessible(false);
            } catch (Exception e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
    }
}
