package cn.rankin.common.utils.api.page;

import com.alibaba.fastjson.annotation.JSONField;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;

public class Page<E> implements Pageable<List<E>> ,Serializable {

	// 当前页
	private int pageNo = 1;
	
	// 当前页大小
	private int pageSize = 10;
	
	// 总条数
	private long totalSize = 0;

	// 数据内容
	private List<E> list;

	public Page() {
		// 默认构造方法
	}
	
	public Page(Long totalSize) {
	    this(totalSize, null, null);
	}
	
	public Page(Long totalSize, Integer pageNo) {
	    this(totalSize, pageNo, null);
	}
	
	public Page(Long totalSize, Integer pageNo, Integer pageSize) {
	    if (totalSize != null && totalSize >= 0) {
	        this.totalSize = totalSize;
        }

        if (pageNo != null && pageNo > 0) {
	        this.pageNo = pageNo;
        }

        if (pageSize != null && pageSize > 0) {
	        this.pageSize = pageSize;
        }
    }

	@Override
	public int getTotalNo() {
		if (totalSize == 0 || pageSize == 0) {
		    return 1;
        }else {
		    return (int) ((totalSize / pageSize) + (totalSize % pageSize == 0 ? 0 : 1));
        }
	}

	@Override
	public long getTotalSize() {
		return totalSize;
	}

	@Override
	public int getPageNo() {
		return pageNo > getTotalNo() ? getTotalNo() : pageNo;
	}

	@Override
	public int getPageSize() {
		return pageSize;
	}

	@Override
    @JSONField(name = "hasNext")
	public boolean hasNext() {
		return getPageNo() < getTotalNo();
	}

	@Override
    @JSONField(name = "hasPrevious")
	public boolean hasPrevious() {
		return getPageNo() > 1;
	}

	public void setPageNo(int pageNo) {
		this.pageNo = pageNo;
	}

	public void setPageSize(int pageSize) {
		this.pageSize = pageSize;
	}

	public void setTotalSize(int totalSize) {
		this.totalSize = totalSize;
	}

	@Override
	public List<E> getList() {
		return null == list ? Collections.emptyList() : list;
	}

	public void setList(List<E> list) {
	    this.list = list;
    }

	@Override
	public long getStart() {
		return (getPageNo() - 1) * getPageSize();
	}

//	public static void main(String[] args) {
//	    Page<Integer> page = new Page(100L, 1, 20);
//	    System.out.print(JSON.toJSONString(page));
//    }
}
