/**
 * 文件名：@RedisConfiguration.java <br/>
 * 包名： configuration <br/>
 * @author tomas <br/>
*/


package cn.rankin.common.utils.configuration;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.CachingConfigurerSupport;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cache.interceptor.KeyGenerator;
import org.springframework.context.EnvironmentAware;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.data.redis.cache.DefaultRedisCachePrefix;
import org.springframework.data.redis.cache.RedisCacheManager;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.StringRedisSerializer;

import java.lang.reflect.Method;

/**
 * 类名：RedisConfiguration  <br />
 *
 * 功能：redis相关配置
 *
 * @author tomas <br />
 * 创建时间：2016年8月16日 上午10:00:53  <br />
 * @version 2016年8月16日
*/
@Configuration
@EnableCaching
public class RedisConfiguration extends CachingConfigurerSupport implements EnvironmentAware {

	// 应用环境信息
	protected Environment environment;

	@Bean
	public KeyGenerator keyGenerator() {
		return new KeyGenerator() {
			@Override
			public Object generate(Object target, Method method, Object... params) {
				StringBuilder sb = new StringBuilder();
				sb.append(target.getClass().getName());
				sb.append(method.getName());
				for (Object obj : params) {
					sb.append(obj.toString());
				}
				return sb.toString();
			}
		};
	}

	@Bean
	public CacheManager cacheManager(RedisTemplate redisTemplate) {
		RedisCacheManager rcm = new RedisCacheManager(redisTemplate);
		rcm.setCachePrefix(new DefaultRedisCachePrefix());
		//设置缓存过期时间 30天
		rcm.setDefaultExpiration(2592000);//秒
		return rcm;
	}

	@Bean(name="redisTemplate")
	public RedisTemplate<?, ?> redisTemplate(JedisConnectionFactory factory){
		RedisTemplate<?, ?> template = new RedisTemplate<>();
		template.setConnectionFactory(factory);
		factory.setDatabase(2);

		//初始化序列化框架（StringRedisSerializer）
		/*StringRedisSerializer stringRedisSerializer = new StringRedisSerializer();
		JdkSerializationRedisSerializer redisSerializer = new JdkSerializationRedisSerializer();

		template.setDefaultSerializer(redisSerializer);
		//设置key的序列化方式
		template.setKeySerializer(stringRedisSerializer);
		//设置value的序列化方式
		template.setValueSerializer(redisSerializer);
		//设置hash key的序列化方式
		template.setHashKeySerializer(stringRedisSerializer);
		//设置hash value的序列化方式
		template.setHashValueSerializer(redisSerializer);

		template.afterPropertiesSet();
		return template;*/

		Jackson2JsonRedisSerializer jackson2JsonRedisSerializer=new Jackson2JsonRedisSerializer(Object.class);
		ObjectMapper om=new ObjectMapper();
		om.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.ANY);
		om.enableDefaultTyping(ObjectMapper.DefaultTyping.NON_FINAL);
		jackson2JsonRedisSerializer.setObjectMapper(om);
		template.setKeySerializer(new StringRedisSerializer());
		template.setValueSerializer(jackson2JsonRedisSerializer);
		template.setDefaultSerializer(jackson2JsonRedisSerializer);
		template.setHashValueSerializer(jackson2JsonRedisSerializer);
		template.setHashKeySerializer(new StringRedisSerializer());
		template.afterPropertiesSet();
		return template;

	}
	@Override
	public void setEnvironment(Environment environment) {
		this.environment = environment;
	}

}
