package cn.rankin.common.utils.util;

import cn.rankin.common.utils.api.model.BaseCode;
import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import com.alibaba.fastjson.JSON;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpMethod;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.security.web.util.matcher.RequestMatcher;
import org.springframework.util.StringUtils;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by tomas on 2017/6/4.
 */

public class HttpUtil {

    public  static List<RequestMatcher> ignoreRequest = new ArrayList<>();

    private static final Logger logger = LoggerFactory.getLogger(HttpUtil.class);

    public static void ok(HttpServletRequest request, HttpServletResponse response) {
        responseOutWithJson(request, response, APIResult.ok());
    }
    public static void error(HttpServletRequest request, HttpServletResponse response, BaseCode code) {
        if(code.getCode() == APICode._C_OK){
            responseOutWithJson(request,response, APIResult.ok());
        }else {
            responseOutWithJson(request,response, APIResult.error(code));
        }
    }
    public static void responseOutWithJson(HttpServletRequest request, HttpServletResponse response, Object data) {
        //将实体对象转换为JSON Object转换
        processHeader(request, response);
        PrintWriter out = null;
        try {
            out = response.getWriter();
            out.append(JSON.toJSONString(data));
            logger.info("url= {} 返回JSON ={} ",request.getRequestURI(), JSON.toJSONString(data));
        } catch (IOException e) {
            logger.error("request ={} 返回 失败 e={}" ,request.getRequestURI(),e);
        } finally {
            if (out != null) {
                out.close();
            }
        }
    }

    public static void processHeader(HttpServletRequest request, HttpServletResponse response) {
        String origin = request.getHeader("Origin");
        String headers = request.getHeader("Access-Control-Request-Headers");
        if (!StringUtils.isEmpty(headers)) {
            headers = ", " + headers;
        } else {
            headers = "";
        }

        response.setHeader("Access-Control-Allow-Origin", "*");
        response.setHeader("Content-Type", "application/json;charset=UTF-8");

        if (!StringUtils.isEmpty(origin)) {
            response.setHeader("Access-Control-Allow-Origin", origin);
            response.setHeader("Access-Control-Allow-Methods", "GET, POST, HEAD, OPTIONS, PUT, DELETE, TRACE, PATCH");
            response.setHeader("Access-Control-Allow-Headers", "Origin, X-Token, Authentication, Accept, X-Requested-With, Content-Type, Access-Control-Request-Method, Access-Control-Request-Headers, Authorization, Cache-control "+headers);
            response.setHeader("Access-Control-Allow-Credentials", "true");
        }
    }

    public static void delCookies(HttpServletRequest request, HttpServletResponse response, String...  cookies) {
        for (String cookieName : cookies) {
            Cookie cookie = new Cookie(cookieName, null);
            String cookiePath = request.getContextPath();
            if (!StringUtils.hasLength(cookiePath)) {
                cookiePath = "/";
            }
            cookie.setPath(cookiePath);
            cookie.setMaxAge(0);
            response.addCookie(cookie);
        }
    }

    public static void antMatchers(HttpMethod httpMethod, String... antPatterns) {
        String method = httpMethod == null?null:httpMethod.toString();
        List<RequestMatcher> matchers = new ArrayList();
        String[] patterns = antPatterns;
        int length = antPatterns.length;

        for(int i = 0; i < length; ++i) {
            String pattern = patterns[i];
            matchers.add(new AntPathRequestMatcher(pattern, method));
        }
        ignoreRequest.addAll(matchers);
    }
    public static RequestMatcher[] getMatchersArray() {
       return ignoreRequest.toArray(new RequestMatcher[ignoreRequest.size()]);
    }

    /**
     * 获取请求主机  真实IP
     * @param request
     * @return
     */
    public static String getClientIp(HttpServletRequest request) {
        // 获取请求主机IP地址,如果通过代理进来，则透过防火墙获取真实IP地址
        String ip = request.getHeader("X-Forwarded-For");
        if (logger.isInfoEnabled()) {
            logger.info("getIpAddress(HttpServletRequest) - X-Forwarded-For - String ip=" + ip);
        }

        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
            if (logger.isInfoEnabled()) {
                logger.info("getIpAddress(HttpServletRequest) - Proxy-Client-IP - String ip=" + ip);
            }
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
            if (logger.isInfoEnabled()) {
                logger.info("getIpAddress(HttpServletRequest) - WL-Proxy-Client-IP - String ip=" + ip);
            }
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_CLIENT_IP");
            if (logger.isInfoEnabled()) {
                logger.info("getIpAddress(HttpServletRequest) - HTTP_CLIENT_IP - String ip=" + ip);
            }
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_X_FORWARDED_FOR");
            if (logger.isInfoEnabled()) {
                logger.info("getIpAddress(HttpServletRequest) - HTTP_X_FORWARDED_FOR - String ip=" + ip);
            }
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
            if (logger.isInfoEnabled()) {
                logger.info("getIpAddress(HttpServletRequest) - getRemoteAddr - String ip=" + ip);
            }
        }


        return ip.split(",")[0];
    }

}
