package cn.rankin.common.utils.util;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import cn.rankin.common.utils.api.Cache;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class LocalCache {

    private static final long interval = 5 * 60 * 1000;

    private static final long timeout = 5 * 60 * 1000;

    private static Map<Object, Cache> pool = new ConcurrentHashMap<Object, Cache>();

    private static final Logger log = LoggerFactory.getLogger(LocalCache.class);


    // Create a timer to clean cache
	static {

		new Timer().schedule(new TimerTask() {
			@Override
			public void run() {
				for (Iterator<Object> it = LocalCache.pool.keySet().iterator(); it.hasNext();) {
					Object key = it.next();
					Cache cache = LocalCache.pool.get(key);
					if (null != cache && cache.getTimeOut() < System.currentTimeMillis()) {
						LocalCache.pool.remove(key);
						cache = null;
                        log.info("LocalCache remove key: {}", key);
					}
				}
			}
		}, new Date(), interval);
	}

	// timeout: seconds
	public static void add(String key, Object value, int timeout) {
		Cache cache = new Cache(key, value, timeout*1000L);
		pool.put(key, cache);
	}

	public static void add(String key, Object value) {
	    Cache cache = new Cache(key, value, timeout);
	    pool.put(key, cache);
    }

	public static Object get(String key, boolean expire) {

		Cache cache = pool.get(key);
		if (null == cache) {
		    return null;
        }

        // not strict to expire
        if (expire == false) {
		    return cache.getValue();
        }

		if (cache.getTimeOut() < System.currentTimeMillis()) {
			pool.remove(key);
			return null;
		}

		return cache.getValue();
	}

	public static void main(String[] args) throws Exception {
		LocalCache.add("a", "A", 1); //保存3秒
		LocalCache.add("b", "B", 1); //保存3秒
        while (true) {
			Thread.sleep(900);
			System.out.println(LocalCache.get("b", false));
            System.out.println(LocalCache.get("a", false));
		}
	}
}