package cn.rankin.data.api.product.entity;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.ProductTypeEnum;
import lombok.Data;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/*
| field | type | null | default | desc | extra |
|:----:|:-----|:-------|:------|:-----|:-----|
| id | varchar(512) | no | | 主键 | pri |
| pid | varchar(512) | no | | 产品id |  |
| product_type | tinyint | no | | 产品类型, 课程/周边/课程包 | |
| merchant_id | varchar(512) | no | | 渠道id | |
| duration | int | no | | 售卖时长 | |
| charge_unit | varchar(512) | no | | 计价单位, 如"季度", "件"等 | |
| cp_price | double(16,2) | no | 0 | 内容提供商成本价 | |
| merchant_price | double(16,2) | no | 0 | 给渠道价格 | |
| terminal_price | double(16,2) | no | 0 | 终端显示价格 | |
| status | smallint | no | | 状态 | |
| gmt_created | timestamp | no | now | 创建时间 | |
| gmt_modified | timestamp | no | now | 更新时间 | | |
*/

@Data
@ToString
@Entity
@DynamicInsert
@DynamicUpdate
@Table(name = "p_goods")
public class Goods implements Serializable {

    @Id
    private String id;

    private String pid;

    @Enumerated(EnumType.ORDINAL)
    private ProductTypeEnum type;

    @Column(name = "merchant_id")
    private String merchantId;

    @Column(name = "cp_id")
    private String cpId;

    private Integer duration;

    @Column(name = "charge_unit")
    private String chargeUnit;

    @Column(name = "cp_price")
    private BigDecimal cpPrice;

    @Column(name = "merchant_price")
    private BigDecimal merchantPrice;

    @Column(name = "terminal_Price")
    private BigDecimal terminalPrice;

    @Column(name = "sort", nullable = false)
    private Integer sort;

    @Column(name = "pkg_id")
    private String pkgId;

    @Enumerated(EnumType.ORDINAL)
    private BaseStatusEnum status;

    @Column(name = "gmt_created", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtCreated;

    @Column(name = "gmt_modified", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtModified;

    @Column(name = "shop_qr")
    //shop qr code
    private String shopQR;

    @Transient
    private Boolean isInCart;
}
