package cn.rankin.data.api.product.entity;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.ProductTypeEnum;
import cn.rankin.data.api.product.vo.PackageProductVo;
import lombok.Data;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

@Data
@ToString
@Table(name = "p_merchant_product", uniqueConstraints = {@UniqueConstraint(columnNames = {"pid", "merchant_id"})})
@Entity
@DynamicUpdate
@DynamicInsert
public class MerchantProduct implements Serializable {

    @Id
    private String id;

    @Column(name = "pid", nullable = false)
    private String pid;

    private String code;

    private String name;

    @Enumerated(EnumType.ORDINAL)
    private ProductTypeEnum type;

    @Column(name = "merchant_id", nullable = false)
    private String merchantId;

    private Integer sort;

    @Enumerated(EnumType.ORDINAL)
    private BaseStatusEnum status;

    @Column(name = "gmt_created", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtCreated;

    @Column(name = "gmt_modified", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtModified;

    @Transient
    private List<Goods> goods;

    @Transient
    private List<PackageProductVo> products;

    @Transient
    private List<Goods> specials;

    @Transient
    private List<Tag> tags;
}
