package cn.rankin.data.api.product.entity;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.data.api.product.vo.PackageProductVo;
import lombok.Data;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/*
| field | type | null | default | desc | extra |
|----|:-----|:-------|:------|:-----|:-----|
| id | varchar(512) | no | | 主键 | pri |
| code | varchar(1024) | yes | null | 课程包编号 | |
| name | varchar(1024) | no | '' | 课程包名 | |
| sort | int | no | 0 | 排序字段 | |
| status | smallint | no | | 状态 | |
| gmt_created | timestamp | no | now | 创建时间 | |
| gmt_modified | timestamp | no | now | 更新时间 | | |
 */
@Data
@ToString
@Entity
@DynamicInsert
@DynamicUpdate
@Table(name = "p_package")
public class Package implements Serializable {

    @Id
    private String id;

    private String code;

    private String name;

    private Integer sort;

    @Enumerated(EnumType.ORDINAL)
    private BaseStatusEnum status;

    @Column(name = "gmt_created", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtCreated;

    @Column(name = "gmt_modified", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtModified;

    @Transient
    private List<PackageProductVo> products;
}
