package cn.rankin.data.api.product.entity;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.ProductTypeEnum;
import cn.rankin.common.utils.jpa.ListAttributeConverter;
import lombok.Data;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

@Data
@ToString
@Entity
@Table(name = "p_support", indexes = {@Index(columnList = "code"), @Index(columnList = "name")})
@DynamicInsert
@DynamicUpdate
public class Support implements Serializable {

    @Id
    private String id;

    @Column(unique = true, nullable = false)
    private String code;

    @Column
    private String name;

    @Column
    private String title;

    @Column(name = "sub_title")
    private String subTitle;

    @Column
    private String digest;

    @Column
    private String detail;

    @Column(name = "cover_url")
    private String coverUrl;

    @Column(name = "img_list")
    @Convert(converter = ListAttributeConverter.class)
    private List<String> imgList;

    @Column(name = "cp_id")
    private String cpId;

    @Enumerated(EnumType.ORDINAL)
    private ProductTypeEnum type;

    @Column
    @Enumerated(EnumType.ORDINAL)
    private BaseStatusEnum status;

    @Column(name = "gmt_created", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtCreated;

    @Column(name = "gmt_modified", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtModified;

    @Transient
    private List<Support> supportList;
}
