package cn.rankin.data.api.product.entity;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.TagTypeEnum;
import cn.rankin.data.api.app.vo.ItemVo;
import lombok.Data;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

@Data
@ToString
@Entity
@Table(name = "p_tag", uniqueConstraints = {@UniqueConstraint(columnNames = {"name", "group_id"})},
        indexes = {@Index(columnList = "name")})
@DynamicInsert
@DynamicUpdate
public class Tag implements Serializable {

    @Id
    private String id;

    @Column
    private String name;

    @Column(name = "group_id", updatable = false, nullable = false)
    private String groupId;

    @Column(name = "type_code", nullable = false)
    private String typeCode;

    @Column
    private Integer sort;

    @Column
    @Enumerated(EnumType.ORDINAL)
    private BaseStatusEnum status;

    @Column(name = "gmt_created", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtCreated;

    @Column(name = "gmt_modified", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtModified;

    @Column(name = "merchant_id")
    private String merchantId;

    @Transient
    private String groupName;

    @Transient
    private String typeName;

    @Transient
    private List<MerchantProduct> productList;

    @Transient
    private List<ItemVo> recs;

}
