package cn.rankin.data.api.trade.entity;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.OrderStatusEnum;
import cn.rankin.common.utils.enums.OrderDetailTypeEnum;
import lombok.Data;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Data
@ToString
@Entity
@Table(name = "t_order_detail")
@DynamicInsert
@DynamicUpdate
public class OrderDetail implements Serializable {

    @Id
    private String id;

    @Column(name = "order_id")
    private String orderId;

    @Column(updatable = false)
    private String uid;

    @Column(name = "serial_no", updatable = false, nullable = false, unique = true)
    private String serialNo;

    @Column(name = "origin_price")
    private BigDecimal originPrice;

    @Column(name = "final_price")
    private BigDecimal finalPrice;

    @Column(name = "adjust_price")
    private BigDecimal adjustPrice;

    @Column(name = "order_status")
    @Enumerated(EnumType.ORDINAL)
    private OrderStatusEnum orderStatus;

    @Enumerated(EnumType.ORDINAL)
    private BaseStatusEnum status;

    @Enumerated(EnumType.ORDINAL)
    private OrderDetailTypeEnum type;

    private String note;

    private String address;

    private String mobile;

    private String name;

    @Column(name = "track_no")
    private String trackNo;

    @Column(name = "cp_id")
    private String cpId;

    @Column(name = "merchant_id")
    private String merchantId;

    @Column(name = "gmt_created", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtCreated;

    @Column(name = "gmt_modified", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtModified;

    @Transient
    private List<OrderGoods> goods;
}
