package cn.rankin.data.api.trade.entity;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.ProductTypeEnum;
import lombok.Data;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/*
| field | type | null | default | desc | extra |
|----|:-----|:-------|:------|:-----|:-----|
| id | varchar(512) | no | | 主键 | pri |
| uid | varchar(512) | no | | 下单终端用户id |  |
| userCode | varchar(512) | no | | 下单终端用户学号 |  |
| userName | varchar(512) | no | | 下单终端用户名称 |  |
| campusId | varchar(512) | no | | 下单终端学校 |  |
| campusCode | varchar(512) | no | | 下单终端学校编号 |  |
| campusName | varchar(512) | no | | 下单终端学校名称 |  |
| order_id | varchar(512) | no | | 订单id |  |
| goods_id | varchar(512) | no | | 订单id |  |
| pkg_id | varchar | no | '' | 课程包id, 不为空则表示以package方式购买 |  |
| pid | varchar(512) | no | | 商品id |  |
| product_type | smallint | no | | 商品类型,{课程包，配套, 课程} |  |
| product_code | varchar(512) | no | | 商品code |  |
| product_name | varchar(512) | no | | 商品名称 |  |
| merchant_id | varchar(512) | no | | 渠道id |  |
| duration | int | no | | 时长 |  |
| cp_id | varchar(512) | yes | | cp id |  |
| cp_price | double(16,2) | no | 0 | 此订单成交时,内容提供商可获得价格, 当pkg_id不为空时，值为cpPriceInPackage | |
| merchant_price | double(16,2) | no | 0 | 给渠道价格, 当pkg_id不为空时，值为0 | |
| terminal_price | double(16,2) | no | 0 | 终端显示价格, 当pkg_id不为空时，值为0 | |
| charge_unit | varchar(512) | no | 0 | 计价单位 | |
| quantity | int | no | 0 | 购买的数量 | |
| gmt_created | timestamp | no | | 快照创建时间 | |
 */
@Data
@ToString
@Entity
@Table(name = "t_order_product_snapshot")
@DynamicInsert
@DynamicUpdate
public class OrderProductSnapshot implements Serializable {

    @Id
    private String id;

    private String uid;

    @Column(name = "user_code")
    private String userCode;

    @Column(name = "user_name")
    private String userName;

    @Column(name = "campus_id")
    private String campusId;

    @Column(name = "campus_code")
    private String campusCode;

    @Column(name = "campus_name")
    private String campusName;

    @Column(name = "order_id")
    private String orderId;

    @Column(name = "detail_id")
    private String detailId;

    @Column(name = "goods_id")
    private String goodsId;

    @Column(name = "pkg_id")
    private String pkgId;

    private String pid;

    @Column(name = "product_type")
    @Enumerated(EnumType.ORDINAL)
    private ProductTypeEnum productType;

    @Column(name = "product_code")
    private String productCode;

    @Column(name = "product_name")
    private String productName;

    @Column(name = "merchant_id")
    private String merchantId;

    @Column(name = "merchant_name")
    private String merchantName;

    private Integer duration;

    @Column(name = "cp_id")
    private String cpId;

    @Column(name = "cp_name")
    private String cpName;

    @Column(name = "cp_price")
    private BigDecimal cpPrice;

    @Column(name = "merchant_price")
    private BigDecimal merchantPrice;

    @Column(name = "terminal_price")
    private BigDecimal terminalPrice;

    @Column(name = "charge_unit")
    private String chargeUnit;

    private Integer quantity;

    @Enumerated(EnumType.ORDINAL)
    private BaseStatusEnum status;

    @Column(name = "gmt_modified", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtModified;

    @Temporal(TemporalType.TIMESTAMP)
    @Column(name = "gmt_created", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP")
    private Date gmtCreated;
}
