package cn.rankin.data.api.user.entity;

import cn.rankin.common.utils.util.ProvinceUtil;
import lombok.Data;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import javax.validation.constraints.Pattern;
import java.io.Serializable;
import java.util.Date;

@Data
@ToString
@Entity
@Table(name = "u_campus", uniqueConstraints = {@UniqueConstraint(columnNames = {"merchant_id", "serial_no", "province_code"})})
@DynamicInsert
@DynamicUpdate
public class Campus implements Serializable {

    @Id
    private String id;

    @Column(updatable = false)
    private String code;

    @Column(name = "merchant_id", updatable = false, nullable = false)
    private String merchantId;

    @Column(name = "province_code", updatable = false, nullable = false)
    private String provinceCode;

    @Column(name = "city_name")
    private String cityName;

    @Column(name = "zone_name")
    private String zoneName;

    @Column(name = "serial_no", updatable = false, nullable = false)
    private Integer serialNo;

    @Column(name = "contact_name")
    private String contactName;

    @Column(nullable = false)
    @Pattern(regexp = "^[1][3,4,5,7,8,9][0-9]{9}$")
    private String mobile;

    private String address;

    @Column(name = "bank_account")
    private String bankAccount;

    // 开户行
    @Column(name = "deposit_bank")
    private String depositBank;

    @Column(name = "gmt_created", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtCreated;

    @Column(name = "gmt_modified", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtModified;

    private String name;

    @Transient
    private String merchantName;

    public void setName() {
        this.name = getName();
    }

    public String getName() {
        String pName = ProvinceUtil.get(provinceCode);
        if (pName != null) {
            return String.format("%s-%s-%s", pName, cityName, zoneName);
        }
        return null;
    }
}
