package cn.rankin.data.api.user.entity;

import cn.rankin.common.utils.enums.LedgerTypeEnum;
import lombok.Data;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

@Data
@ToString
@Entity
@Table(name = "u_ledger")
@DynamicInsert
@DynamicUpdate
public class Ledger implements Serializable {

    @Id
    private String id;

    @Column(name = "merchant_id")
    private String merchantId;

    private BigDecimal quantity;

    @Column(name = "balance_before")
    private BigDecimal balanceBefore;

    @Column(name = "balance_after")
    private BigDecimal balanceAfter;

    @Column(name = "receipt_id")
    private String receiptId;

    @Enumerated(EnumType.ORDINAL)
    private LedgerTypeEnum type;

    private String note;

    @Column(name = "gmt_created", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtCreated;

    public Ledger(String merchantId, BigDecimal balanceBefore, BigDecimal quantity, LedgerTypeEnum type,
                  String receiptId, String note) {
        this.merchantId = merchantId;
        this.balanceBefore = balanceBefore;
        this.quantity = quantity;
        this.balanceAfter = balanceBefore.add(quantity);
        this.type = type;
        this.receiptId = receiptId;
        this.note = note;
    }
}
