package cn.rankin.data.api.user.entity;

import cn.rankin.common.utils.constant.Constant;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.ReceiptTypeEnum;
import cn.rankin.common.utils.validator.Domain;
import lombok.Data;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import javax.validation.constraints.Pattern;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Data
@ToString
@Entity
@Table(name = "u_merchant")
@DynamicInsert
@DynamicUpdate
public class Merchant implements Serializable {

    @Id
    private String id;

    @Domain
    private Integer domain;

    @Column(unique = true, updatable = false)
    @Pattern(regexp = Constant.MRegEx)
    private String code;

    private String name;

    @Column(unique = true, nullable = false)
    private String simple;

    @Column(name = "contact_name")
    private String contactName;

    @Column(nullable = false)
    @Pattern(regexp = "^[1][3,4,5,7,8,9][0-9]{9}$")
    private String mobile;

    @Column(name = "license_id")
    private String licenseId;

    @Column(name = "tax_number")
    private String taxNumber;

    @Column(name = "bank_account", nullable = false)
    private String bankAccount;

    // 开户行
    @Column(name = "deposit_bank", nullable = false)
    private String depositBank;

    private BigDecimal balance;

    @Column(nullable = false)
    @Enumerated(EnumType.ORDINAL)
    private ReceiptTypeEnum receiptType;

    @Enumerated(EnumType.ORDINAL)
    private BaseStatusEnum status;

    @Column(name = "gmt_created", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtCreated;

    @Column(name = "gmt_modified", updatable = false, insertable = false, columnDefinition = "timestamp NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP")
    @Temporal(TemporalType.TIMESTAMP)
    private Date gmtModified;

    @Transient
    private List<CmsUser> adminList;

}
