package cn.rankin.data.api.user.entity;

import lombok.Data;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.io.Serializable;


/**
 * The persistent class for the resource database table.
 * 
 */
@Data
@ToString
@Entity
@Table(name="u_resource")
@DynamicInsert
@DynamicUpdate
public class Resource implements Serializable {
	private static final long serialVersionUID = 1L;

	public static final Integer ROOT = 0;
	public static final Integer MENU = 1;
	public static final Integer BUTTON = 2;

	@Id
	@Column(name = "id")
	private String id;

	@Column(name = "parent_id")
	private String parentId;
	/**
	 * 资源名称
	 */
	private String name;
	/**
	 * 授权Code (多个用逗号分隔，如：vo:list,vo:create)
	 * sys:config:list,sys:config:info,sys:config:save,sys:config:update,sys:config:delete
	 * 此处做的灵活一点 现在权限控制不用每个按钮都设置 只需要控制到菜单权限 然后菜单的 code 一次设置全部权限 增删改查code
	 * 给出两个角色 一个是管理权限 一个查看权限 然后授权给 初始化的 role
	 * 后面需求来了 可以控制到 按钮权限 但是页面上 需要授权 每个角色 具体权限 授权
	 */
	private String code;
	/**
	 * 排序
	 */
	private Integer sort;

	/**
	 * 类型 类型   0：目录   1：菜单   2：按钮
	 */
	private Integer type;


	private String url;

	@Transient
	private boolean own;

}