package cn.rankin.productservice.configuration;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.context.EnvironmentAware;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.StringRedisSerializer;


/**
 * 类名：PersistConfiguration  <br />
 *
 * 功能：持久化相关配置
 *
 * @author xtwin <br />
 * 创建时间：2016年7月26日 上午9:56:48  <br />
 * @version 2016年7月26日
 */
@Configuration
@EntityScan(basePackages={"cn.rankin.data.api.product"})
public class PersistConfiguration implements EnvironmentAware {
	
	// 日志记录器
	private static final Logger logger = LoggerFactory.getLogger(PersistConfiguration.class);
	
	// 应用环境信息
	private Environment environment;

	@Bean(name="redisTemplate")
	public RedisTemplate<String, Object> redisTemplate(JedisConnectionFactory factory){
		String index = environment.getProperty("spring.redis.database", "0");
		RedisTemplate<String, Object> template = new RedisTemplate<>();
		template.setConnectionFactory(factory);
		factory.setDatabase(Integer.parseInt(index));

		Jackson2JsonRedisSerializer jackson2JsonRedisSerializer=new Jackson2JsonRedisSerializer(Object.class);
		ObjectMapper om=new ObjectMapper();
		om.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.ANY);
		om.enableDefaultTyping(ObjectMapper.DefaultTyping.NON_FINAL);
		jackson2JsonRedisSerializer.setObjectMapper(om);
		template.setKeySerializer(new StringRedisSerializer());
		template.setValueSerializer(jackson2JsonRedisSerializer);
		template.setDefaultSerializer(jackson2JsonRedisSerializer);
		template.setHashValueSerializer(jackson2JsonRedisSerializer);
		template.setHashKeySerializer(new StringRedisSerializer());
		template.afterPropertiesSet();
		return template;
	}


	/**
	 * 功能: 由spring注入环境信息 <br/>
	 * 
	 * 重写：xtwin <br/>
	 * 
	 * @version ：2016年7月27日 上午10:02:15<br/>
	 * 
	 * @param environment <br/>
	 * @see EnvironmentAware#setEnvironment(Environment)
	 */
	@Override
	public void setEnvironment(Environment environment) {
		this.environment = environment;
	}
}
