package cn.rankin.productservice.controller;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.data.api.product.entity.Goods;
import cn.rankin.data.api.product.entity.MerchantProduct;
import cn.rankin.data.api.product.entity.Poster;
import cn.rankin.data.api.product.entity.Product;
import cn.rankin.productservice.repository.GoodsRepository;
import cn.rankin.productservice.repository.MerchantProductRepository;
import cn.rankin.productservice.repository.PosterRepository;
import cn.rankin.productservice.repository.ProductRepository;
import cn.rankin.productservice.service.ProductService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping(value = "/merchant")
@Slf4j
public class MerchantController {

    @Autowired
    private PosterRepository posterRepo;

    @Autowired
    private ProductService productService;

    @Autowired
    private ProductRepository productRepository;

    @Autowired
    private MerchantProductRepository merchantProductRepository;

    @Autowired
    private GoodsRepository goodsRepository;

    @RequestMapping(value = "/{merchantId}/posters", method = RequestMethod.GET)
    public List<Poster> getPosters(@PathVariable("merchantId") String merchantId, @RequestParam("start") Long start,
                                         @RequestParam("offset") Integer offset, @RequestParam("sortKey") String sortKey,
                                         @RequestParam("direction") Sort.Direction direction) {

        //get posters
        Poster findSample = new Poster();
        findSample.setMerchantId(merchantId);
        findSample.setStatus(BaseStatusEnum.NORMAL);

        List<Poster> posters = posterRepo.find(findSample, start, offset, new Sort(direction, sortKey));

        if (null == posters){
            posters = new ArrayList<>();
        }

        log.info("Find Posters, len={}, merchantId={}", posters.size(), merchantId);
        return posters;
    }

    @RequestMapping(value = "/{merchantId}/product/{pid}", method = RequestMethod.GET)
    public Product getProduct(@PathVariable("merchantId") String merchantId, @PathVariable("pid") String pid){

        //check merchant
        MerchantProduct merchantProduct =  merchantProductRepository.findByPidAndMerchantId(pid, merchantId);

        if (null == merchantProduct){
            log.error("Cannot Find Merchant Product By pid, pid={}, merchantId={}", pid, merchantId);
            return null;
        }

        Product product = productRepository.findByPid(pid);

        if (null == product){
            log.error("Cannot Find Product By pid, pid={}", pid);
            return null;
        }

        //get goods
        List<Goods> goods = goodsRepository.findByPidAndMerchantId(pid, merchantId);

        product.setGoods(goods);

        return product;
    }
}

