package cn.rankin.productservice.controller.app;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.app.vo.ItemVo;
import cn.rankin.data.api.product.entity.Course;
import cn.rankin.data.api.product.vo.CourseItemVo;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.productservice.service.CourseService;
import cn.rankin.productservice.service.app.ItemService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(value = "/app/course")
public class CourseController {

    @Autowired
    private CourseService courseService;

    @Autowired
    private ItemService itemService;

    @RequestMapping(value = "/{courseId}/lessons", method = RequestMethod.GET)
    public APIResult<Course> getCourseLessons(@PathVariable("courseId") String courseId) {
        return courseService.getCourseLessons(courseId);
    }

    @RequestMapping(value = "/{courseId}/detail", method = RequestMethod.GET)
    public APIResult<CourseItemVo> getCourseItem(@PathVariable("courseId") String courseId, @RequestParam("merchantId") String merchantId) {
        CourseItemVo courseItemVo = itemService.findCourseItem(courseId, merchantId);
        if (courseItemVo == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }
        return APIResult.ok(courseItemVo);
    }

    @RequestMapping(value = "/{courseId}/supports", method = RequestMethod.GET)
    public APIResult<List<ItemVo>> getSupportByCourseId(@PathVariable("courseId") String courseId, @RequestParam("merchantId") String merchantId) {
        List<ItemVo> itemVoList = itemService.findSupportByCourseId(courseId, merchantId);
        return APIResult.ok(itemVoList);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public Course getCourse(@PathVariable("id") String id) {
        return courseService.findByIdAndStatus(id);
    }
}
