package cn.rankin.productservice.controller.cms;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.data.api.product.entity.Recommend;
import cn.rankin.productservice.service.RecommendService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(value = "/recommend")
public class RecommendController {

    @Autowired
    private RecommendService recommendService;

    @RequestMapping(method = RequestMethod.GET)
    public APIResult<List<Recommend>> get(@RequestParam("merchantId") String merchantId) {
        if (StringUtils.isEmpty(merchantId)) {
            return APIResult.error(ProductServiceAPICode.PARAMETER_ERROR);
        }

        List<Recommend> recommendList = recommendService.get(merchantId);
        return APIResult.ok(recommendList);
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<List<Recommend>> put(@RequestParam("merchantId") String merchantId, @RequestBody List<String> itemIdList) {
        if (StringUtils.isEmpty(merchantId) || CollectionUtils.isEmpty(itemIdList)) {
            return APIResult.error(ProductServiceAPICode.PARAMETER_ERROR);
        }

        List<Recommend> recommendList = recommendService.put(merchantId, itemIdList);
        return APIResult.ok(recommendList);
    }

    @RequestMapping(value = "/courses", method = RequestMethod.GET)
    public APIResult<List<Recommend>> getRecommendCourses(@RequestParam("merchantId") String merchantId) {
        if (StringUtils.isEmpty(merchantId)) {
            return APIResult.error(ProductServiceAPICode.PARAMETER_ERROR);
        }

        List<Recommend> recommendList = recommendService.getRecommendCourses(merchantId);
        return APIResult.ok(recommendList);
    }
}
