package cn.rankin.productservice.controller.cms;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.dto.search.TagSearchDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.data.api.product.dto.TagDTO;
import cn.rankin.data.api.product.entity.Tag;
import cn.rankin.productservice.service.TagService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.LinkedHashMap;

@Slf4j
@RestController
@RequestMapping(value = "tag")
public class TagController {

    @Autowired
    private TagService tagService;

    @RequestMapping(value = "/{tagId}", method = RequestMethod.GET)
    public APIResult<Tag> getTag(@PathVariable("tagId") String tagId) {

        return tagService.getTag(tagId);
    }

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<Tag>> search(TagSearchDTO searchDTO) {
        Tag tag = new Tag();

        String name = searchDTO.getName();
        if (name != null) {
            tag.setName("%" + name + "%");
        }
        tag.setMerchantId(searchDTO.getMerchantId());
        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);
        return tagService.search(tag, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);
    }

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<Tag> create(@RequestBody TagDTO tagDTO) {
        return tagService.create(tagDTO);
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<Tag> update(@RequestBody TagDTO tagDTO) {
        return tagService.update(tagDTO);
    }

    @RequestMapping(value = "/{tagId}", method = RequestMethod.DELETE)
    public APIResult delete(@PathVariable("tagId") String tagId) {
        return tagService.delete(tagId);
    }

    @RequestMapping(value = "/v2/{tagId}", method = RequestMethod.GET)
    public Tag getTagInfo(@PathVariable("tagId") String tagId) {
        return tagService.getTagInfo(tagId);
    }

}
