package cn.rankin.productservice.controller.cms;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.GroupDTO;
import cn.rankin.common.utils.dto.search.GroupSearchDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.data.api.product.entity.TagGroup;
import cn.rankin.productservice.service.TagGroupService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.LinkedHashMap;

@RestController
@RequestMapping(value = "group")
public class TagGroupController {

    @Autowired
    private TagGroupService tagGroupService;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<TagGroup>> search(GroupSearchDTO searchDTO) {
        TagGroup tagGroup = new TagGroup();

        String code = searchDTO.getCode();
        if (!StringUtils.isEmpty(code)) {
            tagGroup.setCode("%" + code + "%");
        }

        String name = searchDTO.getName();
        if (!StringUtils.isEmpty(name)) {
            tagGroup.setName("%" + name + "%");
        }

        // others
        tagGroup.setMerchantId(searchDTO.getMerchantId());

        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);

        return tagGroupService.search(tagGroup, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);
    }

    @Transactional
    @RequestMapping(method = RequestMethod.POST)
    public APIResult<TagGroup> create(@Valid @RequestBody GroupDTO groupDTO) {
        return tagGroupService.create(groupDTO);
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<TagGroup> update(@RequestBody GroupDTO groupDTO) {
        return tagGroupService.update(groupDTO);
    }

    @RequestMapping(value = "/{groupId}", method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@PathVariable("groupId") String groupId) {
        return tagGroupService.delete(groupId);
    }

    @RequestMapping(value = "/{groupId}", method = RequestMethod.GET)
    public APIResult<TagGroup> getGroup(@PathVariable("groupId") String groupId) {
        return tagGroupService.getTagGroup(groupId);
    }
}
