package cn.rankin.productservice.controller.cms;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.dto.search.SearchDTO;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.CourseWareDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.data.api.product.entity.Lesson;
import cn.rankin.data.api.product.entity.LessonWareRelation;
import cn.rankin.data.api.product.entity.CourseWare;
import cn.rankin.productservice.service.LessonService;
import cn.rankin.productservice.service.LessonWareRelationService;
import cn.rankin.productservice.service.WareService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

@RestController
@RequestMapping(value = "ware")
public class WareController {

    @Autowired
    private WareService wareService;

    @Autowired
    private LessonWareRelationService relationService;

    @Autowired
    private LessonService lessonService;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<CourseWare>> search(SearchDTO searchDTO) {
        CourseWare CourseWare = new CourseWare();

        String code = searchDTO.getCode();
        if (!StringUtils.isEmpty(code)) {
            CourseWare.setCode("%" + code + "%");
        }

        String name = searchDTO.getName();
        if (!StringUtils.isEmpty(name)) {
            CourseWare.setTitle("%" + name + "%");
        }

        CourseWare.setStatus(searchDTO.getStatus());

        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap() { {
            this.put("gmtModified", BaseOrderEnum.DESC);
        }};

        return wareService.search(CourseWare, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);
    }

    // 创建资源
    @RequestMapping(method = RequestMethod.POST)
    public APIResult<CourseWare> create(@Valid @RequestBody CourseWareDTO courseWareDTO) {
        return wareService.create(courseWareDTO);
    }

    // 修改资源
    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<CourseWare> update(@Valid @RequestBody CourseWareDTO courseWareDTO) {
        if (courseWareDTO.getId() == null) {
            return APIResult.error(APICode.error("参数错误: id"));
        }

        return wareService.update(courseWareDTO);
    }

    // 删除资源
    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult delete(@NotNull @PathVariable("id") String id) {
        List<LessonWareRelation> existRel = relationService.findByWareId(id);
        if (existRel == null || existRel.size() == 0) {
            return wareService.delete(id);
        }

        List<String> lessonIds = new ArrayList<>();
        existRel.forEach( rel ->
            {
                String lessonId = rel.getLessonId();
                if (!lessonIds.contains(lessonId)) {
                    lessonIds.add(rel.getLessonId());
                }
            }
        );

        List<Lesson> lessonList = lessonService.findByIds(lessonIds).getData();
        APIResult<List<Lesson>> result = APIResult.error(APICode.CAN_NOT_DEL);
        result.setData(lessonList);

        return result;
    }

    // 取一个
    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<CourseWare> getWare(@PathVariable("id") String id) {
        return wareService.findById(id);
    }

}
