package cn.rankin.productservice.repository;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.jpa.BasicJpaRepository;
import cn.rankin.data.api.product.entity.LessonWareRelation;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

public interface LessonWareRelationRepository extends BasicJpaRepository<LessonWareRelation, String> {

    List<LessonWareRelation> findAllByWareId(String wareId);

    List<LessonWareRelation> findAllByLessonId(String lessonId);

    @Query(value = "select r from LessonWareRelation r where r.wareId = :wareId and r.status = :status")
    List<LessonWareRelation> findByWareId(@Param("wareId") String wareId, @Param("status") BaseStatusEnum status);

    @Query(value = "select r from LessonWareRelation r where r.lessonId = :lessonId and r.status = :status order by r.sort asc")
    List<LessonWareRelation> findByLessonId(@Param("lessonId") String lessonId, @Param("status") BaseStatusEnum status);

    @Transactional
    @Modifying
    @Query(value = "update LessonWareRelation r set r.status = :status where r.lessonId = :lessonId")
    Integer deleteByLessonId(@Param("lessonId") String lessonId, @Param("status") BaseStatusEnum status);
}
