package cn.rankin.productservice.repository;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.jpa.BasicJpaRepository;
import cn.rankin.data.api.product.entity.MerchantProduct;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

import java.util.List;

public interface MerchantProductRepository extends BasicJpaRepository<MerchantProduct, String> {

    List<MerchantProduct> findByPid(String pid);

    @Query(value = "select m from MerchantProduct m where m.pid in (:pids) and m.status = 0")
    List<MerchantProduct> findByPidIn(@Param("pids") List<String> pids);

    @Query(value = "select m from MerchantProduct m where m.pid in (?1) and m.merchantId = ?2 order by m.sort")
    List<MerchantProduct> findByPidsAndMerchantId(List<String> pids, String merchantId);

    @Query(value = "select m from MerchantProduct m where m.pid in (?1) order by m.sort")
    List<MerchantProduct> findByPids(List<String> pids);

    @Query(value = "select m from MerchantProduct m where m.pid in (?1) and m.merchantId = ?2 and m.status = ?3 order by m.sort")
    List<MerchantProduct> findByPidsAndMerchantId(List<String> pids, String merchantId, BaseStatusEnum status);

    MerchantProduct findByPidAndMerchantId(String pid, String merchantId);

    @Query(value = "select m from MerchantProduct m where m.pid = ?1 and m.merchantId = ?2 and m.status=?3 order by m.sort")
    MerchantProduct findByPidAndMerchantIdAndStatus(String pid, String merchantId, BaseStatusEnum status);

    @Modifying
    @Query(value = "update MerchantProduct m set m.status = 1 where m.pid = ?1")
    Integer deleteByPid(String pid);
}
