package cn.rankin.productservice.repository;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.jpa.BasicJpaRepository;
import cn.rankin.data.api.product.entity.MerchantProductTagRelation;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

import java.util.List;

public interface MerchantProductTagRelationRepository extends BasicJpaRepository<MerchantProductTagRelation, String> {

    @Query(value = "select p from MerchantProductTagRelation p where p.tagId = ?1 and p.status = ?2 order by p.sort")
    List<MerchantProductTagRelation> findByTagId(String tagId, BaseStatusEnum status);

    List<MerchantProductTagRelation> findByTagId(String tagId);

    List<MerchantProductTagRelation> findByTagIdIn(List<String> tagIds);

    @Query(value = "select p from MerchantProductTagRelation p where p.pid in (?1) and p.merchantId in (?2) and p.status = 0")
    List<MerchantProductTagRelation> findByPidsAndMerchantIds(List<String> pids, List<String> merchantIds);

    @Query(value = "select r from MerchantProductTagRelation r where r.pid = ?1 and r.merchantId = ?2 and r.status = ?3")
    List<MerchantProductTagRelation> findByPidAndMerchantId(String pid, String merchantId, BaseStatusEnum status);

    List<MerchantProductTagRelation> findByPidAndMerchantId(String pid, String merchantId);

    @Query(value = "select * from p_product_tag_relation r where r.tag_id = ?1 and r.pid in (?2) and r.status = 0 order by r.sort limit ?3, ?4", nativeQuery = true)
    List<MerchantProductTagRelation> findByTagIdAndPids(String tagId, List<String> productIdList, Long start, Integer size);

    @Modifying
    @Query(value = "update MerchantProductTagRelation r set r.status = 1 where r.tagId = :tagId")
    Integer deleteByTagId(@Param("tagId") String tagId);

    @Modifying
    @Query(value = "update MerchantProductTagRelation r set r.status = 1 where r.pid = :productId and r.merchantId = :merchantId")
    Integer deleteByPidAndMerchantId(@Param("productId") String productId, @Param("merchantId") String merchantId);
}
