package cn.rankin.productservice.service;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.entity.CourseSupportRelation;
import cn.rankin.productservice.repository.CourseSupportRelationRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class CourseSupportRelationService {

    @Autowired
    private CourseSupportRelationRepository relationRepository;

    public List<CourseSupportRelation> findByCourseId(String courseId) {
        return relationRepository.findByCourseId(courseId, BaseStatusEnum.NORMAL);
    }

    @Transactional
    public Integer deleteByCourseId(String courseId) {
        return relationRepository.deleteByCourseId(courseId, BaseStatusEnum.DEL);
    }

    @Transactional
    public void update(String courseId, List<String> supportIdList) {
        if (supportIdList == null) {
            return;
        }else if (supportIdList.size() == 0) {
            deleteByCourseId(courseId);
            return;
        }

        List<CourseSupportRelation> relationList = relationRepository.findAllByCourseId(courseId);
        List<String> existSupportIdList = new ArrayList<>();

        // 更新现存的
        relationList.forEach( relation -> {
            String supportId = relation.getSupportId();
            if (supportIdList.contains(supportId)) {
                relation.setStatus(BaseStatusEnum.NORMAL);
            }else {
                relation.setStatus(BaseStatusEnum.DEL);
            }
            existSupportIdList.add(supportId);
        });

        // 更新新加的
        List<String> notExistsSupportIdList = ListUtil.subtract(supportIdList, existSupportIdList);
        if (notExistsSupportIdList != null && notExistsSupportIdList.size() > 0) {
            notExistsSupportIdList.forEach( id -> {
                CourseSupportRelation relation = new CourseSupportRelation();
                relation.setCourseId(courseId);
                relation.setSupportId(id);
                relation.setStatus(BaseStatusEnum.NORMAL);
                relationList.add(relation);
            });
        }

        // 排序
        relationList.forEach( relation -> {
            String supportId = relation.getSupportId();
            Integer sort = supportIdList.indexOf(supportId);
            relation.setSort(sort);
        });

        relationRepository.save(relationList);
    }
}
