package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.data.api.product.dto.GoodsDTO;
import cn.rankin.data.api.product.entity.Goods;
import cn.rankin.data.api.product.entity.Product;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.productservice.repository.GoodsRepository;
import cn.rankin.productservice.repository.ProductRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Service
public class GoodsService {

    @Autowired
    private GoodsRepository goodsRepository;

    @Autowired
    private ProductRepository productRepository;

    // 拿商品id获取商品
    public APIResult<Goods> getGoods(String goodsId) {
        Goods goods = goodsRepository.find(goodsId);
        if (goods == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }
        return APIResult.ok(goods);
    }

    // 创建一个带价格的商品
    @Transactional
    public APIResult<Goods> create(GoodsDTO goodsDTO) {
        String productId = goodsDTO.getPid();
        Product product = productRepository.findByPid(productId);
        if (product == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }

        Goods goods = convert(goodsDTO);
        goods.setCpId(product.getCpId());
        goods.setType(product.getType());
        goods.setSort(0);
        Goods result = goodsRepository.save(goods);
        return APIResult.ok(result);
    }

    // 修改商品
    @Transactional
    public APIResult<Goods> update(GoodsDTO goodsDTO) {
        Goods goods = convert(goodsDTO);
        Goods result = goodsRepository.update(goods);
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String goodsId) {
        Integer count = goodsRepository.deleteById(goodsId);
        if (count > 0) {
            return APIResult.ok(true);
        }
        return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
    }

    public APIResult<List<Goods>> findByIds(List<String> goodsIdList) {
        List<Goods> goodsList = goodsRepository.findByIds(goodsIdList);
        return APIResult.ok(goodsList);
    }

    public APIResult<List<Goods>> findPackageGoods(String pkgId, String merchantId) {
        List<Goods> goodsList = goodsRepository.findByPkgIdAndMerchantId(pkgId, merchantId);
        return APIResult.ok(goodsList);
    }

    public List<Goods> findByPidAndMerchantId(String pId, String merchantId) {
        List<Goods> goodsList = goodsRepository.findByPidAndMerchantId(pId, merchantId);
        return goodsList;
    }
}
