package cn.rankin.productservice.service;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.entity.LessonWareRelation;
import cn.rankin.productservice.repository.LessonWareRelationRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class LessonWareRelationService {

    @Autowired
    private LessonWareRelationRepository relationRepository;

    public List<LessonWareRelation> findByWareId(String wareId) {
        return relationRepository.findByWareId(wareId, BaseStatusEnum.NORMAL);
    }

    public List<LessonWareRelation> findByLessonId(String lessonId) {
        return relationRepository.findByLessonId(lessonId, BaseStatusEnum.NORMAL);
    }

    @Transactional
    public Integer deleteByLessonId(String lessonId) {
        return relationRepository.deleteByLessonId(lessonId, BaseStatusEnum.DEL);
    }

    @Transactional
    public void update(String lessonId, List<String> wareIdList) {
        if (wareIdList == null) {
            return;
        }else if (wareIdList.size() == 0) {
            deleteByLessonId(lessonId);
            return;
        }

        List<LessonWareRelation> relationList = relationRepository.findAllByLessonId(lessonId);
        List<String> existWareIdList = new ArrayList<>();

        // 更新现存的
        relationList.forEach( relation -> {
            String wareId = relation.getWareId();
            if (wareIdList.contains(wareId)) {
                relation.setStatus(BaseStatusEnum.NORMAL);
            }else {
                relation.setStatus(BaseStatusEnum.DEL);
            }
            existWareIdList.add(wareId);
        });

        // 更新新加的
        List<String> notExistsWareIdList = ListUtil.subtract(wareIdList, existWareIdList);
        if (notExistsWareIdList != null && notExistsWareIdList.size() > 0) {
            notExistsWareIdList.forEach( id -> {
                LessonWareRelation relation = new LessonWareRelation();
                relation.setLessonId(lessonId);
                relation.setWareId(id);
                relation.setStatus(BaseStatusEnum.NORMAL);
                relationList.add(relation);
            });
        }

        // 排序
        relationList.forEach( relation -> {
            String wareId = relation.getWareId();
            Integer sort = wareIdList.indexOf(wareId);
            relation.setSort(sort);
        });

        relationRepository.save(relationList);

    }
}
