package cn.rankin.productservice.service;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.entity.MerchantProductTagRelation;
import cn.rankin.productservice.repository.MerchantProductTagRelationRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class MerchantProductTagRelationService {

    @Autowired
    private MerchantProductTagRelationRepository relationRepository;

    public List<MerchantProductTagRelation> findByTagId(String tagId) {
        return relationRepository.findByTagId(tagId);
    }

    public List<MerchantProductTagRelation> findByTagId(String tagId, BaseStatusEnum status) {
        return relationRepository.findByTagId(tagId, status);
    }

    public List<MerchantProductTagRelation> findByPidsAndMerchantIds(List<String> pids, List<String> merchantIds) {
        List<MerchantProductTagRelation> merchantProductTagRelationList = relationRepository.findByPidsAndMerchantIds(pids, merchantIds);
        return merchantProductTagRelationList;
    }

    public List<MerchantProductTagRelation> findByPidAndMerchantId(String pid, String merchantId) {
        return relationRepository.findByPidAndMerchantId(pid, merchantId, BaseStatusEnum.NORMAL);
    }

    @Transactional
    public void updateByPid(String productId, String merchantId, List<String> tagIdList) {
        if (tagIdList == null) {
            return;
        }else if (tagIdList.size() == 0) {
            relationRepository.deleteByPidAndMerchantId(productId, merchantId);
            return;
        }

        List<MerchantProductTagRelation> relationList = relationRepository.findByPidAndMerchantId(productId, merchantId);

        List<String> currentIdList = new ArrayList<>();
        relationList.forEach( relation -> {
            String tagId = relation.getTagId();
            if (!tagIdList.contains(tagId)) {
                relation.setStatus(BaseStatusEnum.DEL);
            }else {
                relation.setStatus(BaseStatusEnum.NORMAL);
            }
            currentIdList.add(tagId);
        });

        List<String> notExistTagIdList = ListUtil.subtract(tagIdList, currentIdList);
        if (notExistTagIdList != null && notExistTagIdList.size() > 0) {
            notExistTagIdList.forEach( tagId -> {
                MerchantProductTagRelation relation = new MerchantProductTagRelation();
                relation.setPid(productId);
                relation.setTagId(tagId);
                relation.setMerchantId(merchantId);
                relation.setSort(0);
                relation.setStatus(BaseStatusEnum.NORMAL);
                relationList.add(relation);
            });
        }

        relationRepository.save(relationList);
    }

    @Transactional
    public void updateByTagId(String tagId, String merchantId, List<String> productIdList) {
        if (productIdList == null) {
            return;
        }else if (productIdList.size() == 0) {
            relationRepository.deleteByTagId(tagId);
            return;
        }

        List<MerchantProductTagRelation> relationList = relationRepository.findByTagId(tagId);

        List<String> currentIdList = new ArrayList<>();
        relationList.forEach( relation -> {
            String productId = relation.getPid();
            if (productIdList.contains(productId)) {
                relation.setStatus(BaseStatusEnum.NORMAL);
            }else {
                relation.setStatus(BaseStatusEnum.DEL);
            }
            currentIdList.add(productId);
        });

        List<String> notExistItemIdList = ListUtil.subtract(productIdList, currentIdList);
        if (notExistItemIdList != null && notExistItemIdList.size() > 0) {
            notExistItemIdList.forEach( productId -> {
                MerchantProductTagRelation relation = new MerchantProductTagRelation();
                relation.setPid(productId);
                relation.setTagId(tagId);
                relation.setSort(0);
                relation.setMerchantId(merchantId);
                relation.setStatus(BaseStatusEnum.NORMAL);
                relationList.add(relation);
            });
        }

        // 排序
        relationList.forEach( relation -> {
            String productId = relation.getPid();
            Integer index = productIdList.indexOf(productId);
            relation.setSort(index);
        });

        relationRepository.save(relationList);
    }
}
