package cn.rankin.productservice.service;

import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.dto.PosterDTO;
import cn.rankin.data.api.product.entity.Poster;
import cn.rankin.productservice.repository.PosterRepository;
import cn.rankin.productservice.utils.DTOConverter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Slf4j
@Service
public class PosterService {

    @Autowired
    private PosterRepository posterRepository;

    public List<Poster> findByIds(List<String> posterIds) {
        List<Poster> posterList = posterRepository.findByIds(posterIds);

        if (posterList == null || posterList.size() == 0) {
            return posterList;
        }

        Map<String, Poster> posterMap = ListUtil.convert(posterList, "id", Poster.class);
        List<Poster> sortPosterList = new ArrayList<>();
        posterIds.forEach( id -> {
            Poster poster = posterMap.get(id);
            if (poster != null) {
                sortPosterList.add(poster);
            }
        });

        return sortPosterList;
    }

    public List<Poster> search(Poster poster, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = posterRepository.count(poster);

        if (count == 0) {
            return new ArrayList<>();
        }

        List<Poster> posterList = posterRepository.find(poster, JpaSortUtil.sort(sort));
        return posterList;
    }

    public Poster findById(String id) {
        Poster poster = posterRepository.find(id);
        return poster;
    }

    @Transactional
    public Poster create(PosterDTO posterDTO) {
        Poster poster = convert(posterDTO);
        Poster result = posterRepository.save(poster);

        return result;
    }

    @Transactional
    public Poster update(PosterDTO posterDTO) {
        Poster poster = DTOConverter.convert(posterDTO);
        Poster result = posterRepository.update(poster);

        return result;
    }

    @Transactional
    public Boolean delete(String id) {
        Integer count = posterRepository.deleteById(id);
        if (count > 0) {
            return true;
        }
        return false;
    }

    public List<Poster> getByMerchantId(String merchantId) {
        List<Poster> posterList = posterRepository.findByMerchantId(merchantId);
        if (CollectionUtils.isEmpty(posterList)) {
            return new ArrayList<>();
        }

        return posterList;
    }

    public List<Poster> updatePosters(String merchantId, List<String> posterIdList) {
        if (CollectionUtils.isEmpty(posterIdList)) {
            return null;
        }

        List<Poster> posterList = posterRepository.findByMerchantId(merchantId);

        // 排序
        posterList.forEach( poster -> {
//            String productId = poster.getPid();
            Integer sort = posterIdList.indexOf(poster.getId());
            poster.setSort(sort);
        });

        posterRepository.save(posterList);
        List<Poster> result = posterRepository.findByStatus(merchantId, BaseStatusEnum.NORMAL);
        return result;
    }

    public boolean isExist(PosterDTO posterDTO) {
        Poster poster =posterRepository.findByMerchantIdPid(posterDTO.getMerchantId(),posterDTO.getPid());
        if(poster != null && !poster.getId().equals(posterDTO.getId())){
            return true;
        }else{
            return false;
        }
    }
}
