package cn.rankin.productservice.service;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.entity.SupportReference;
import cn.rankin.productservice.repository.SupportReferenceRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.List;

@Service
public class SupportReferenceService {

    @Autowired
    private SupportReferenceRepository relationRepository;

    @Transactional
    public Integer deleteBySupportId(String supportId) {
        return relationRepository.deleteByFid(supportId);
    }

    public List<SupportReference> findByFid(String fid) {
        return relationRepository.findByFid(fid, BaseStatusEnum.NORMAL);
    }

    @Transactional
    public void update(String supportId, List<String> tidList) {
        if (tidList == null) {
            return;
        }else if (tidList.size() == 0) {
            deleteBySupportId(supportId);
            return;
        }

        //去除自身
        List<String> supportIdList = new ArrayList<>();
        tidList.forEach( id -> {
            if (supportId.equals(id)){
               return;
            }
            supportIdList.add(id);
        });


        List<SupportReference> relationList = relationRepository.findAllByFid(supportId);
        List<String> existSupportIdList = new ArrayList<>();

        // 更新现存的
        relationList.forEach( relation -> {
            String tid = relation.getTid();
            if (supportIdList.contains(tid)) {
                relation.setStatus(BaseStatusEnum.NORMAL);
            }else {
                relation.setStatus(BaseStatusEnum.DEL);
            }
            existSupportIdList.add(tid);
        });

        // 更新新加的
        List<String> notExistsSupportIdList = ListUtil.subtract(supportIdList, existSupportIdList);
        if (notExistsSupportIdList != null && notExistsSupportIdList.size() > 0) {
            notExistsSupportIdList.forEach( id -> {
                SupportReference relation = new SupportReference();
                relation.setFid(supportId);
                relation.setTid(id);
                relation.setStatus(BaseStatusEnum.NORMAL);
                relationList.add(relation);
            });
        }

        // 排序
        relationList.forEach( relation -> {
            String tid = relation.getTid();
            Integer sort = supportIdList.indexOf(tid);
            relation.setSort(sort);
        });

        relationRepository.save(relationList);
    }
}
