package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.data.api.app.vo.ItemVo;
import cn.rankin.data.api.product.dto.TagTypeDTO;
import cn.rankin.data.api.product.entity.MerchantProduct;
import cn.rankin.data.api.product.entity.Tag;
import cn.rankin.data.api.product.entity.TagType;
import cn.rankin.productservice.repository.TagTypeRepository;
import cn.rankin.productservice.service.app.ItemService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.LinkedHashMap;
import java.util.List;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Service
public class TagTypeService {

    @Autowired
    private TagTypeRepository tagTypeRepository;

    @Autowired
    private TagService tagService;


    public boolean exists(String code) {
        Long count = tagTypeRepository.countByCode(code);
        return count > 0;
    }

    public Page<TagType> search(TagType tagType, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = tagTypeRepository.count(tagType);

        Page<TagType> page = new Page<>(count, pageNo, pageSize);
        if (count == 0) {
            return page;
        }

        List<TagType> tagTypeList = tagTypeRepository.find(tagType, page.getStart(), page.getPageSize(), JpaSortUtil.sort(sort));
        page.setList(tagTypeList);

        return page;
    }

    @Transactional
    public TagType create(TagTypeDTO tagTypeDTO) {
        String code = tagTypeDTO.getCode();
        if (exists(code)) {
            //已存在返回null
            return null;
        }

        TagType tagType = convert(tagTypeDTO);
        TagType result = tagTypeRepository.save(tagType);
        return result;
    }

    @Transactional
    public TagType update(TagTypeDTO tagTypeDTO) {
        String typeCode = tagTypeDTO.getCode();
        TagType tagType = convert(tagTypeDTO);
        // 更新标签组
        TagType result = tagTypeRepository.update(tagType);
        // 给组内的标签排序
        List<String> tagIdList = tagTypeDTO.getTagList();
        if (!CollectionUtils.isEmpty(tagIdList)) {
            tagService.sortTagByTypeCode(typeCode, tagIdList);
        }
        return result;
    }

    @Transactional
    public Boolean delete(String id) {
        Integer count = tagTypeRepository.deleteById(id);
        if (count > 0) {
            return true;
        }
        return false;
    }

    public TagType getTagType(String tagTypeId) {
        TagType tagType = tagTypeRepository.find(tagTypeId);
        return tagType;
    }

    public TagType getTagTypeByCode(String typeCode) {
        TagType tagType = tagTypeRepository.findFirstByCode(typeCode);
        return tagType;
    }

    public TagType findTagTypeByCode(String code) {
        TagType tagType = tagTypeRepository.findFirstByCode(code);
        if (tagType == null) {
            return null;
        }
        String tagTypeId = tagType.getId();
        return this.getTagType(tagTypeId);
    }
}
