package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.CourseWareDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.entity.CourseWare;
import cn.rankin.productservice.repository.WareRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Slf4j
@Service
public class WareService {

    @Autowired
    private WareRepository wareRepository;

    public APIResult<Page<CourseWare>> search(CourseWare CourseWare, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = wareRepository.count(CourseWare);
        Page<CourseWare> page = new Page(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<CourseWare> courseWareList = wareRepository.find(CourseWare, page.getStart(), pageSize, JpaSortUtil.sort(sort));
        page.setList(courseWareList);

        return APIResult.ok(page);
    }

    public boolean exists(String code) {
        Long count = wareRepository.countByCode(code);
        return count > 0L;
    }

    @Transactional
    public APIResult<CourseWare> create(CourseWareDTO courseWareDTO) {
        CourseWare CourseWare = convert(courseWareDTO);
        if (exists(CourseWare.getCode())) {
            return APIResult.error(APICode.ALREADY_EXISTS);
        }
        CourseWare result = wareRepository.save(CourseWare);
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<CourseWare> update(CourseWareDTO courseWareDTO) {
        CourseWare CourseWare = convert(courseWareDTO);
        CourseWare result = wareRepository.update(CourseWare);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = wareRepository.deleteAllById(id, BaseStatusEnum.DEL);
        if (count > 0) {
            return APIResult.ok(true);
        }
        return APIResult.error(APICode.NOT_EXISTS);
    }

    public APIResult<CourseWare> findById(String id) {
        CourseWare CourseWare = wareRepository.find(id);
        if (CourseWare == null) {
            return APIResult.error(APICode.NOT_EXISTS);
        }
        return APIResult.ok(CourseWare);
    }

    public APIResult<List<CourseWare>> findByIds(List<String> ids) {
        List<CourseWare> courseWareList = wareRepository.findByIdIn(ids, BaseStatusEnum.NORMAL);

        if (courseWareList == null || courseWareList.size() == 0) {
            return APIResult.ok(courseWareList);
        }

        Map<String, CourseWare> wareMap = ListUtil.convert(courseWareList, "id", CourseWare.class);
        List<CourseWare> sortCourseWareList = new ArrayList<>();
        ids.forEach( id -> {
            CourseWare CourseWare = wareMap.get(id);
            if (CourseWare != null) {
                sortCourseWareList.add(CourseWare);
            }
        });

        return APIResult.ok(sortCourseWareList);
    }
}
