package cn.rankin.productservice.service.auth;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.model.BaseCode;
import cn.rankin.common.utils.constant.Constant;
import cn.rankin.data.api.auth.dto.AuthQueryDTO;
import cn.rankin.data.api.auth.vo.AuthResult;
import cn.rankin.data.api.auth.vo.AuthVo;
import cn.rankin.data.api.product.entity.MerchantProduct;
import cn.rankin.data.api.product.entity.PackageProductRelation;
import cn.rankin.productservice.repository.MerchantProductRepository;
import cn.rankin.productservice.repository.PackageProductRelationRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

@Service
public class AuthService {

    @Autowired
    private AuthClient authClient;

    @Autowired
    private PackageProductRelationRepository packageProductRelationRepository;

    @Autowired
    private MerchantProductRepository merchantProductRepository;

    public APIResult<AuthResult> auth(String userId, String merchantId, String itemId) {
        AuthResult authResult = new AuthResult();
        authResult.setPid(itemId);
        authResult.setUserId(userId);
        authResult.setType(Constant.ProductType.COURSE);

        List<String> authIdList = new ArrayList<>();
        authIdList.add(itemId);

        List<PackageProductRelation> relationList = packageProductRelationRepository.findByPid(itemId);
        for (PackageProductRelation relation : relationList) {
            String productId = relation.getPkgId();
            authIdList.add(productId);
        }
        List<MerchantProduct> merchantProductList = merchantProductRepository.findByPidsAndMerchantId(authIdList, merchantId);
        if (CollectionUtils.isEmpty(merchantProductList)) {
            return APIResult.ok(authResult);
        }

        List<String> validIdList = new ArrayList<>();
        merchantProductList.forEach(merchantProduct -> validIdList.add(merchantProduct.getPid()));
        AuthQueryDTO authQueryDTO = new AuthQueryDTO();
        authQueryDTO.setUid(userId);
        authQueryDTO.setPids(validIdList);
        authQueryDTO.setBizCode(Constant.BIZ_CODE);
        authQueryDTO.setType(Constant.ProductType.COURSE);

        APIResult<List<AuthVo>> apiResult = authClient.auth(authQueryDTO);
        if (!apiResult.getSuccess()) {
            return APIResult.error(new BaseCode(apiResult.getCode(), apiResult.getMessage()));
        }
        List<AuthVo> authVoList = apiResult.getData();
        if (CollectionUtils.isEmpty(authVoList)) {
            return APIResult.ok(authResult);
        }
        Collections.sort(authVoList, new Comparator<AuthVo>() {
            @Override
            public int compare(AuthVo o1, AuthVo o2) {
                Long diff = o1.getStartTime() - o2.getStartTime();
                if (diff > 0L) {
                    return 1;
                }
                return 0;
            }
        });
        for (int i = 0; i < authVoList.size(); i++) {
            AuthVo authVo = authVoList.get(i);
            if (i == 0) {
                authResult.setStartTime(authVo.getStartTime());
                authResult.setEndTime(authVo.getEndTime());
                continue;
            }
            Long preEndTime = authResult.getEndTime();
            Long curStartTime = authVo.getStartTime();
            if (preEndTime < curStartTime) {
                break;
            }
            Long curEndTime = authVo.getEndTime();
            if (curEndTime > preEndTime) {
                authResult.setEndTime(curEndTime);
            }
        }
        return APIResult.ok(authResult);
    }
}
