package cn.rankin.resourceservice.controller;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.resource.vo.OSSSignature;
import com.aliyun.oss.OSSClient;
import com.aliyun.oss.common.utils.BinaryUtil;
import com.aliyun.oss.model.MatchMode;
import com.aliyun.oss.model.PolicyConditions;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.sql.Date;

@Slf4j
@RestController
@RequestMapping(value = "oss")
public class AliOSSController {

    @Value("${ali.oss.endpoint}")
    private String endpoint;

    @Value("${ali.oss.accessKeyId}")
    private String accessId;

    @Value("${ali.oss.accessKeySecret}")
    private String accessKey;

    @Value("${ali.oss.bucket}")
    private String bucket;

    @Value("${ali.oss.rankin.dir}")
    private String dir;

    public String getHost() {
        return  "http://" + bucket + "." + endpoint;
    }

    private final static long expireTime = 2 * 60;

    @RequestMapping(value = "signature", method = RequestMethod.GET)
    public APIResult<OSSSignature> getSignature (String fileName) {

        OSSSignature signature = new OSSSignature(accessId, dir);
        signature.setPath(getPath(fileName));
        signature.setHost(getHost());

        try {
            OSSClient client = new OSSClient(endpoint, accessId, accessKey);

            long expireEndTime = System.currentTimeMillis() + expireTime * 1000;
            signature.setExpire(String.valueOf(expireEndTime / 1000));

            // generate policy
            PolicyConditions policyConds = new PolicyConditions();
            policyConds.addConditionItem(PolicyConditions.COND_CONTENT_LENGTH_RANGE, 0, 1048576000);
            policyConds.addConditionItem(MatchMode.StartWith, PolicyConditions.COND_KEY, dir);
            String postPolicy = client.generatePostPolicy(new Date(expireEndTime), policyConds);
            byte[] binaryData = postPolicy.getBytes("utf-8");
            String encodedPolicy = BinaryUtil.toBase64String(binaryData);

            signature.setPolicy(encodedPolicy);

            String postSignature = client.calculatePostSignature(postPolicy);
            signature.setSignature(postSignature);

        }catch (Exception e) {
            log.error("OSS Signature error: {}", e);
            return APIResult.error(APICode.OPERATE_ERROR);
        }

        return APIResult.ok(signature);
    }

    public static String getPath(String fileName) {
        String path;
        if (fileName == null || fileName.length() == 0) {
            return null;
        }

        if (fileName.contains("-")) {
            path = fileName.replaceAll("-","/");
        }else {
            path = fileName;
        }

        int dotPos = path.indexOf(".");
        if (dotPos > -1) {
            String suffix = path.substring(dotPos);
            path = path.replace(suffix, suffix.toLowerCase());
        }

        return path;
    }
}

