package cn.rankin.resourceservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.exception.DuplicateValueException;
import cn.rankin.common.utils.exception.NotFoundException;
import cn.rankin.common.utils.exception.UnsupportedOperationException;
import cn.rankin.data.api.resource.entity.AudioImg;
import cn.rankin.data.api.resource.entity.Resource;
import cn.rankin.resourceservice.dto.ResourceDetail;
import cn.rankin.resourceservice.repository.AudioImgRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.*;


@Slf4j
@Service
public class AudioImgService {

    @Autowired
    private AudioImgRepository audioImgRepository;

    public boolean exists(String code) {
        Long count = audioImgRepository.countByCode(code);
        return count > 0L;
    }

    public AudioImg get(String id) {
        return audioImgRepository.find(id);
    }

    public AudioImg save(AudioImg audioImg) {
        return audioImgRepository.save(audioImg);
    }

    public Page<AudioImg> search(AudioImg ai, Integer pageNo, Integer pageSize) {
        Long count = audioImgRepository.count(ai);
        Page<AudioImg> page = new Page(count, pageNo, pageSize);

        if (count == 0) {
            return page;
        }

        List<AudioImg> audioImgList = audioImgRepository.find(ai, page.getStart(), pageSize, new Sort(Sort.Direction.DESC, "gmtModified"));
        page.setList(audioImgList);

        return page;
    }

    /**
     *
      * @param audioImg
     * @throws DuplicateValueException
     * @throws NotFoundException
     */
    public AudioImg update(AudioImg audioImg) throws DuplicateValueException, NotFoundException, UnsupportedOperationException {
        if (null == audioImg){
            log.error("audioImg is null");
            return null;
        }

        AudioImg result = audioImgRepository.update(audioImg);
        return result;
    }

    public List<AudioImg>  findMixByIds(List<String> ids) {

        // 取本地
        List<AudioImg> audioImgs = audioImgRepository.findByIds(ids);
        return audioImgs;
    }
}

