package cn.rankin.task.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.product.entity.Goods;
import cn.rankin.data.api.product.entity.Package;
import cn.rankin.data.api.product.entity.Product;
import cn.rankin.task.code.TaskAPICode;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

@FeignClient(name = "${service.product.name}", fallback = ProductService.ProductServiceHystrix.class)
public interface ProductService {

    @RequestMapping(value = "/product/package/{pkgId}", method = RequestMethod.GET)
    APIResult<Package> getPackage(@PathVariable("pkgId") String pkgId);

    @RequestMapping(value = "/product/ids", method = RequestMethod.GET)
    APIResult<List<Product>> findProductByIds(@RequestParam("id") List<String> ids);

    @RequestMapping(value = "/goods/ids", method = RequestMethod.GET)
    APIResult<List<Goods>> findGoodsByIds(@RequestParam("id") List<String> ids);

    @RequestMapping(value = "goods/package/{pkgId}", method = RequestMethod.GET)
    APIResult<List<Goods>> findPackageGoods(@PathVariable("pkgId") String pkgId, @RequestParam("merchantId") String merchantId);

    @Component
    class ProductServiceHystrix implements ProductService {

        @Override
        public APIResult getPackage(String pkgId) {
            return APIResult.error(TaskAPICode.SERVER_ERROR);
        }

        @Override
        public APIResult findProductByIds(List<String> ids) {
            return APIResult.error(TaskAPICode.SERVER_ERROR);
        }

        @Override
        public APIResult findGoodsByIds(List<String> ids) {
            return APIResult.error(TaskAPICode.SERVER_ERROR);
        }

        @Override
        public APIResult findPackageGoods(String pkgId, String merchantId) {
            return APIResult.error(TaskAPICode.SERVER_ERROR);
        }
    }
}
