package cn.rankin.task.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.user.entity.Merchant;
import cn.rankin.data.api.user.vo.CampusVo;
import cn.rankin.data.api.user.vo.TerminalUserVo;
import cn.rankin.task.code.TaskAPICode;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;

@FeignClient(name = "${service.user.name}", fallback = UserService.UserServiceHystrix.class)
public interface UserService {

    @RequestMapping(value = "/user/ids", method = RequestMethod.GET)
    APIResult<List<TerminalUserVo>> findUserByIds(@RequestParam("id") List<String> userIdList);

    @RequestMapping(value = "/campus/ids", method = RequestMethod.GET)
    APIResult<Map<String, CampusVo>> findCampusByIds(@RequestParam("id") List<String> campusIdList);

    @RequestMapping(value = "/merchant/ids", method = RequestMethod.GET)
    APIResult<Map<String, Merchant>> findMerchantByIds(@RequestParam("id") List<String> merchantIds);

    @Component
    class UserServiceHystrix implements UserService {

        @Override
        public APIResult findUserByIds(List<String> userIdList) {
            return APIResult.error(TaskAPICode.SERVER_ERROR);
        }

        @Override
        public APIResult findCampusByIds(List<String> campusIdList) {
            return APIResult.error(TaskAPICode.SERVER_ERROR);
        }

        @Override
        public APIResult findMerchantByIds(List<String> merchantIds) {
            return APIResult.error(TaskAPICode.SERVER_ERROR);
        }
    }
}
