package cn.rankin.task.task.order;

import cn.rankin.data.api.trade.entity.Order;
import cn.rankin.task.service.OrderService;
import cn.rankin.task.service.ProductService;
import cn.rankin.task.service.UserService;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Component
public class OrderCompleteTask {

    public final static long INTERVAL = 10 * 1000;

    public final static Integer SIZE = 100;

    @Autowired
    private ProductService productService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private UserService userService;

    @Scheduled(fixedDelay = INTERVAL)
    public void run() {
        log.info("{} start", this.getClass().getName());

        List<Order> orderList = orderService.findOrderForCompleteList(SIZE);
        if (orderList.size() == 0) {
            log.info("load order for complete empty");
            return;
        }
        List<String> orderIdList = new ArrayList<>();
        orderList.forEach(order -> orderIdList.add(order.getId()));

        List<String> completeOrderIdList = orderService.findCompleteOrderFromOrderDetail(orderIdList);
        if (!CollectionUtils.isEmpty(completeOrderIdList)) {
            log.info("order complete, orderIdList={}", JSON.toJSONString(completeOrderIdList));
            orderService.completeOrder(completeOrderIdList);
        }else {
            log.info("no complete order");
        }

        log.info("{} complete", this.getClass().getName());
    }
}