package cn.rankin.tradeservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.data.api.trade.dto.OrderSnapshotSearchDTO;
import cn.rankin.data.api.trade.entity.OrderProductSnapshot;
import cn.rankin.tradeservice.repository.OrderProductSnapshotRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;

@Service
public class OrderProductSnapshotService {

    @Autowired
    private OrderProductSnapshotRepository orderProductSnapshotRepository;

    public APIResult<Page<OrderProductSnapshot>> search(OrderSnapshotSearchDTO searchDTO) {
        Specification<OrderProductSnapshot> specification = new Specification<OrderProductSnapshot>() {
            @Override
            public Predicate toPredicate(Root<OrderProductSnapshot> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
                return OrderProductSnapshotService.this.toPredicate(root, query, cb, searchDTO);
            }
        };

        Long count = orderProductSnapshotRepository.count(specification);
        Page<OrderProductSnapshot> page = new Page<>(count, searchDTO.getPageNo(), searchDTO.getPageSize());

        Long start = page.getStart();
        if (count == 0) {
            return APIResult.ok(page);
        }

        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);
        List<OrderProductSnapshot> orderList = orderProductSnapshotRepository.find(specification, start, page.getPageSize(), JpaSortUtil.sort(sort));
        page.setList(orderList);

        return APIResult.ok(page);
    }

    public Predicate toPredicate(Root<OrderProductSnapshot> root, CriteriaQuery<?> query, CriteriaBuilder cb, OrderSnapshotSearchDTO searchDTO) {
        // 存放查询条件
        List<Predicate> list = new ArrayList<Predicate>();
        Predicate tmp = null;

        String userId = searchDTO.getUid();
        if (!StringUtils.isEmpty(userId)) {
            tmp = cb.equal(root.get("uid"), userId);
            list.add(tmp);
        }

        String userCode = searchDTO.getUserCode();
        if (!StringUtils.isEmpty(userCode)) {
            tmp = cb.like(root.get("userCode"), "%" + userCode + "%");
            list.add(tmp);
        }

        String campusId = searchDTO.getCampusId();
        if (!StringUtils.isEmpty(campusId)) {
            tmp = cb.equal(root.get("campusId"), campusId);
            list.add(tmp);
        }

        String campusCode = searchDTO.getCampusCode();
        if (!StringUtils.isEmpty(campusCode)) {
            tmp = cb.like(root.get("campusCode"), "%" + campusCode + "%");
            list.add(tmp);
        }

        String campusName = searchDTO.getCampusName();
        if (!StringUtils.isEmpty(campusName)) {
            tmp = cb.like(root.get("campusName"), "%" + campusName + "%");
            list.add(tmp);
        }

        String pkgId = searchDTO.getPkgId();
        if (!StringUtils.isEmpty(pkgId)) {
            tmp = cb.equal(root.get("pkgId"), pkgId);
            list.add(tmp);
        }

        String productId = searchDTO.getPid();
        if (!StringUtils.isEmpty(productId)) {
            tmp = cb.equal(root.get("pid"), productId);
            list.add(tmp);
        }

        String productCode = searchDTO.getProductCode();
        if (!StringUtils.isEmpty(productCode)) {
            tmp = cb.like(root.get("productCode"), productCode);
            list.add(tmp);
        }

        String cpId = searchDTO.getCpId();
        if (!StringUtils.isEmpty(cpId)) {
            tmp = cb.equal(root.get("cpId"), cpId);
            list.add(tmp);
        }

        String merchantId = searchDTO.getMerchantId();
        if (!StringUtils.isEmpty(merchantId)) {
            tmp = cb.equal(root.get("merchantId"), merchantId);
            list.add(tmp);
        }

        Date beginTime = searchDTO.getBeginTime();
        if (beginTime != null) {
            tmp = cb.greaterThanOrEqualTo(root.get("gmtCreated"), beginTime);
            list.add(tmp);
        }

        Date endTime = searchDTO.getEndTime();
        if (endTime != null) {
            tmp = cb.lessThanOrEqualTo(root.get("gmtCreated"), endTime);
            list.add(tmp);
        }
        // where条件
        return list.isEmpty() ? null : cb.and(list.toArray(new Predicate[list.size()]));
    }

}
