package cn.rankin.userservice.controller;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.user.dto.CmsUserDTO;
import cn.rankin.data.api.user.dto.CmsUserSearchDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.userservice.code.UserServiceAPICode;
import cn.rankin.data.api.user.entity.CmsUser;
import cn.rankin.data.api.user.entity.Merchant;
import cn.rankin.userservice.service.CmsUserService;
import cn.rankin.userservice.service.MerchantService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = "/cms")
public class CmsUserController {

    @Autowired
    private CmsUserService cmsUserService;

    @Autowired
    private MerchantService merchantService;

    @RequestMapping(value = "/user", method = RequestMethod.GET)
    public APIResult<CmsUser> getUser(@RequestParam("name") String name) {
        APIResult<CmsUser> result = cmsUserService.loadCmsUser(name);
        if (!result.getSuccess() || result.getData() == null) {
            return result;
        }

        CmsUser user = result.getData();
        setMerchantName(user);
        return result;
    }

    @RequestMapping(value = "/user/list", method = RequestMethod.GET)
    public APIResult<Page<CmsUser>> search(CmsUserSearchDTO searchDTO) {
        CmsUser user = new CmsUser();
        String name = searchDTO.getName();
        if (name != null) {
            user.setName("%" + name + "%");
        }

        String merchantId = searchDTO.getMerchantId();
        if (merchantId != null) {
            user.setMerchantId(merchantId);
        }

        user.setStatus(searchDTO.getStatus());

        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);

        APIResult<Page<CmsUser>> result = cmsUserService.search(user, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);
        if (result.getSuccess() && result.getData().getList() != null) {
            List<CmsUser> cmsUserList = result.getData().getList();
            setMerchantName(cmsUserList);
        }
        return result;
    }

    @RequestMapping(value = "/user", method = RequestMethod.POST)
    public APIResult<CmsUser> create(@RequestBody CmsUserDTO userDTO) {
        String merchantId = userDTO.getMerchantId();
        if (StringUtils.isEmpty(merchantId)) {
            return APIResult.error(UserServiceAPICode.PARAMETER_ERROR);
        }
        Merchant merchant = merchantService.findNormalById(merchantId).getData();
        if (merchant == null) {
            return APIResult.error(UserServiceAPICode.NOT_EXISTS);
        }
        APIResult<CmsUser> result = cmsUserService.create(userDTO);
        if (result.getSuccess()) {
            CmsUser user = result.getData();
            setMerchantName(user);
        }

        return result;
    }

    @RequestMapping(value = "/user", method = RequestMethod.PUT)
    public APIResult<CmsUser> update(@RequestBody CmsUserDTO userDTO) {
        if (StringUtils.isEmpty(userDTO.getId())) {
            return APIResult.error(UserServiceAPICode.PARAMETER_ERROR);
        }
        APIResult<CmsUser> result = cmsUserService.update(userDTO);
        if (result.getSuccess()) {
            CmsUser user = result.getData();
            setMerchantName(user);
        }
        return result;
    }

    @RequestMapping(value = "/user/{id}", method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@PathVariable String id) {
        return cmsUserService.delete(id);
    }

    public void setMerchantName(CmsUser user) {
        if (user == null) {
            return;
        }
        String merchantId = user.getMerchantId();
        Merchant merchant = merchantService.findOne(merchantId);
        if (merchant == null) {
            return;
        }
        user.setMerchantName(merchant.getName());
    }

    public void setMerchantName(List<CmsUser> userList) {
        if (CollectionUtils.isEmpty(userList)) {
            return;
        }

        List<String> merchantIdList = new ArrayList<>();
        userList.forEach(user -> {
            String merchantId = user.getMerchantId();
            if (StringUtils.isNotEmpty(merchantId)) {
                merchantIdList.add(merchantId);
            }
        });

        Map<String, Merchant> merchantMap = merchantService.getMerchantMap(merchantIdList);
        for (CmsUser user : userList) {
            Merchant merchant = merchantMap.get(user.getMerchantId());
            if (merchant == null) {
                continue;
            }
            user.setMerchantName(merchant.getName());
        }
    }
}
