package cn.rankin.userservice.controller;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.user.dto.MerchantDTO;
import cn.rankin.data.api.user.dto.MerchantSearchDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.data.api.user.entity.Merchant;
import cn.rankin.data.api.user.entity.CmsUser;
import cn.rankin.userservice.service.MerchantService;
import cn.rankin.userservice.service.CmsUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = "merchant")
public class MerchantController {

    @Autowired
    private MerchantService merchantService;

    @Autowired
    private CmsUserService cmsUserService;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<Merchant>> search(MerchantSearchDTO searchDTO) {
        Merchant merchant = new Merchant();
        String code = searchDTO.getCode();
        if (code != null) {
            merchant.setCode("%" + code + "%");
        }

        String name = searchDTO.getName();
        if (name != null) {
            merchant.setName("%" + name + "%");
        }

        String merchantId = searchDTO.getMerchantId();
        if (merchantId != null) {
            merchant.setId(merchantId);
        }

        merchant.setStatus(searchDTO.getStatus());
        merchant.setDomain(searchDTO.getDomain());

        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);

        return merchantService.search(merchant, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);
    }

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<Merchant> create(@RequestBody @Valid MerchantDTO merchantDTO) {
        return merchantService.create(merchantDTO);
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<Merchant> update(@RequestBody MerchantDTO merchantDTO) {
        if (merchantDTO.getId() == null) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }

        return merchantService.update(merchantDTO);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@PathVariable String id) {
        return merchantService.delete(id);

    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<Merchant> getMerchant(@PathVariable String id) {
        APIResult<Merchant> result = merchantService.getMerchant(id);
        Merchant merchant = result.getData();

        if (!result.getSuccess() || merchant == null) {
            return result;
        }

        List<CmsUser> adminCmsUserList = cmsUserService.findMerchantAdminList(id);
        merchant.setAdminList(adminCmsUserList);

        return result;
    }

    @RequestMapping(value = "/ids", method = RequestMethod.GET)
    public APIResult<Map<String, Merchant>> getMerchantList(@RequestParam("id") List<String> idList) {
        return merchantService.findByIds(idList);
    }
}
