package cn.rankin.userservice.controller;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.user.dto.TerminalDeviceDTO;
import cn.rankin.data.api.user.entity.TerminalDevice;
import cn.rankin.userservice.code.UserServiceAPICode;
import cn.rankin.userservice.service.TerminalDeviceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@RestController
@RequestMapping(value = "/device")
public class TerminalDeviceController {

    @Autowired
    private TerminalDeviceService terminalDeviceService;

    @RequestMapping(value = "/bind", method = RequestMethod.POST)
    public APIResult<TerminalDevice> bind(@Valid @RequestBody TerminalDeviceDTO terminalDeviceDTO) {
        return terminalDeviceService.bind(terminalDeviceDTO);
    }

    @RequestMapping(value = "/updatebind", method = RequestMethod.PUT)
    public APIResult<TerminalDevice> updatebind(@Valid @RequestBody TerminalDeviceDTO terminalDeviceDTO) {
        return terminalDeviceService.updatebind(terminalDeviceDTO);
    }

    @RequestMapping(value = "/unbind", method = RequestMethod.DELETE)
    public APIResult<Boolean> unbind(@RequestParam("userId") String userId) {
        return terminalDeviceService.unbind(userId);
    }

    @RequestMapping(value = "/uid/{userId}", method = RequestMethod.GET)
    public APIResult<TerminalDevice> findByUserId(@PathVariable("userId") String userId) {
//        return terminalDeviceService.findByUserId(userId);
        TerminalDevice terminalDevice = terminalDeviceService.findByUserId(userId);
        if (terminalDevice == null) {
            return APIResult.error(UserServiceAPICode.NOT_EXISTS);
        }
        return APIResult.ok(terminalDevice);
    }

    @RequestMapping(value = "/deviceCode/{deviceCode}", method = RequestMethod.GET)
    public List<TerminalDevice> findByDeviceCode(@PathVariable("deviceCode") String deviceCode) {
        return terminalDeviceService.findByDeviceCode(deviceCode);
    }
}
