package cn.rankin.userservice.controller;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.user.dto.WhiteUserDTO;
import cn.rankin.data.api.user.dto.WhiteUserSearchDTO;
import cn.rankin.data.api.user.entity.Merchant;
import cn.rankin.data.api.user.entity.TerminalUser;
import cn.rankin.data.api.user.entity.WhiteUser;
import cn.rankin.userservice.code.UserServiceAPICode;
import cn.rankin.userservice.service.MerchantService;
import cn.rankin.userservice.service.TerminalUserService;
import cn.rankin.userservice.service.WhiteUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping(value = "/white/user")
public class WhiteUserController {

    @Autowired
    private MerchantService merchantService;

    @Autowired
    private TerminalUserService terminalUserService;

    @Autowired
    private WhiteUserService whiteUserService;

    @RequestMapping(value = "/{userId}", method = RequestMethod.GET)
    public APIResult<WhiteUser> getWhiteUser(@PathVariable String userId) {
        WhiteUser whiteUser = whiteUserService.findUserByUserId(userId);
        if (whiteUser == null) {
            return APIResult.error(UserServiceAPICode.NOT_EXISTS);
        }
        setUserInfo(whiteUser);
        return APIResult.ok(whiteUser);
    }

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<WhiteUser> create(@RequestBody WhiteUserDTO whiteUserDTO) {
        String userId = whiteUserDTO.getUserId();
        TerminalUser terminalUser = terminalUserService.find(userId);
        if (terminalUser == null) {
            return APIResult.error(UserServiceAPICode.NOT_EXISTS);
        }
        whiteUserDTO.setMerchantId(terminalUser.getMerchantId());
        whiteUserDTO.setCode(terminalUser.getCode());

        WhiteUser whiteUser = whiteUserService.addUser(whiteUserDTO);
        if (whiteUser != null) {
            return APIResult.ok(whiteUser);
        }
        setUserInfo(whiteUser);
        return APIResult.error(UserServiceAPICode.error("创建失败"));
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<WhiteUser> update(@RequestBody WhiteUserDTO whiteUserDTO) {
        WhiteUser whiteUser = whiteUserService.updateUser(whiteUserDTO);
        if (whiteUser != null) {
            return APIResult.ok(whiteUser);
        }
        setUserInfo(whiteUser);
        return APIResult.error(UserServiceAPICode.error("更新失败"));
    }

    @RequestMapping(method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@RequestBody WhiteUserDTO whiteUserDTO) {
        String userId = whiteUserDTO.getUserId();
        Boolean result = whiteUserService.deleteUser(userId);
        if (Boolean.TRUE.equals(result)) {
            return APIResult.ok(result);
        }
        return APIResult.error(UserServiceAPICode.NOT_EXISTS);
    }

    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public APIResult<Page<WhiteUser>> search(@RequestBody WhiteUserSearchDTO searchDTO) {
        WhiteUser sample = new WhiteUser();
        String code = searchDTO.getCode();
        if (!StringUtils.isEmpty(code)) {
            sample.setCode("%" + code + "%");
        }
        String merchantId = searchDTO.getMerchantId();
        if (!StringUtils.isEmpty(merchantId)) {
            sample.setMerchantId(merchantId);
        }
        Integer pageNo = searchDTO.getPageNo();
        Integer pageSize = searchDTO.getPageSize();
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);
        Page<WhiteUser> page = whiteUserService.search(sample, pageNo, pageSize, sort);
        List<WhiteUser> whiteUserList = page.getList();
        setUserInfo(whiteUserList);
        return APIResult.ok(page);
    }

    public void setUserInfo(WhiteUser whiteUser) {
        if (whiteUser == null) {
            return;
        }
        String userId = whiteUser.getUserId();
//        TerminalUser terminalUser = terminalUserService.find(userId);
//        if (terminalUser != null) {
//            whiteUser.setCode(terminalUser.getCode());
//        }
        Merchant merchant = merchantService.findOne(whiteUser.getMerchantId());
        if (merchant != null) {
            whiteUser.setMerchantName(merchant.getName());
        }
    }

    public void setUserInfo(List<WhiteUser> whiteUserList) {
        if (CollectionUtils.isEmpty(whiteUserList)) {
            return;
        }

        List<String> userIdList = new ArrayList<>();
        List<String> merchantIdList = new ArrayList<>();
        for (WhiteUser whiteUser : whiteUserList) {
            userIdList.add(whiteUser.getUserId());
            String merchantId = whiteUser.getMerchantId();
            if (!merchantIdList.contains(merchantId)) {
                merchantIdList.add(merchantId);
            }
        }

//        List<TerminalUser> terminalUserList = terminalUserService.findByIds(userIdList);
//        Map<String, TerminalUser> terminalUserMap = ListUtil.convert(terminalUserList, "id", TerminalUser.class);
        Map<String, Merchant> merchantMap = merchantService.getMerchantMap(merchantIdList);

        for (WhiteUser whiteUser : whiteUserList) {
//            String userId = whiteUser.getUserId();
//            TerminalUser terminalUser = terminalUserMap.get(userId);
//            if (terminalUser != null) {
//                whiteUser.setCode(terminalUser.getCode());
//            }
            String merchantId = whiteUser.getMerchantId();
            Merchant merchant = merchantMap.get(merchantId);
            if (merchant != null) {
                whiteUser.setMerchantName(merchant.getName());
            }
        }
    }
}
