package cn.rankin.userservice.proxy;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.userservice.code.UserServiceAPICode;
import cn.rankin.userservice.dto.RemoteUser;
import cn.rankin.userservice.dto.UserLogin;
import cn.rankin.userservice.dto.UserPasswd;
import cn.rankin.userservice.dto.UserRegistry;
import cn.rankin.data.api.user.entity.TerminalUser;
import cn.rankin.common.utils.constant.Constant;
import cn.rankin.userservice.utils.DTOConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class RemoteUserProxy {

    @Autowired
    private RemoteUserFeign remoteUserFeign;

    public APIResult<RemoteUser> register(String eid, String password, String userIp, String appVer) {
//        UserRegistry user = DTOConverter.convert(terminalUser);
        UserRegistry user = new UserRegistry();
        user.setEntry(eid);
        user.setEntryType(Constant.EntryType.EID);
        user.setPasswd(password);
        return remoteUserFeign.register(user, userIp, appVer);
    }

    public APIResult<RemoteUser> login(TerminalUser terminalUser, String userIp, String appVer) {
        UserLogin userLogin = new UserLogin();
        userLogin.setEntry(terminalUser.getCode());
        userLogin.setEntryType(Constant.EntryType.EID);
        userLogin.setPasswd(terminalUser.getPassword());
        APIResult<List<RemoteUser>> result = remoteUserFeign.login(userLogin, userIp, appVer);
        if (result.getSuccess()) {
            return APIResult.ok(result.getData().get(0));
        }
        return APIResult.error(UserServiceAPICode.error(result.getMessage()));
    }

    public APIResult changePassword(TerminalUser terminalUser, String newPassword, String userIp, String appVer) {
        APIResult<RemoteUser> remoteUserAPIResult = login(terminalUser, userIp, appVer);
        if (!remoteUserAPIResult.getSuccess() || remoteUserAPIResult.getData() == null) {
            return APIResult.error(UserServiceAPICode.REMOTE_USER_LOGIN_ERROR);
        }

        RemoteUser user = remoteUserAPIResult.getData();

        UserPasswd userPasswd = new UserPasswd();
        userPasswd.setToken(user.getToken());
        userPasswd.setPasswd(newPassword);

        APIResult result = remoteUserFeign.passwd(terminalUser.getId(), userPasswd, userIp, appVer);
        return result;
    }
}
