package cn.rankin.userservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.data.api.user.entity.BuyMsg;
import cn.rankin.data.api.user.entity.ShopCart;
import cn.rankin.userservice.code.UserServiceAPICode;
import cn.rankin.userservice.repository.BuyMsgRepository;
import cn.rankin.userservice.repository.ShopCartRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.List;

@Service
public class BuyMsgService {

    @Autowired
    private BuyMsgRepository buyMsgRepository;

    @Autowired
    private ShopCartRepository shopCartRepository;

    @Transactional
    public APIResult<Boolean> push(String userId) {
        ShopCart sample = new ShopCart();
        sample.setUserId(userId);
        sample.setStatus(BaseStatusEnum.NORMAL);
        List<ShopCart> shopCartList = shopCartRepository.find(sample);
        if (!CollectionUtils.isEmpty(shopCartList)) {
            List<BuyMsg> buyMsgList = convert(shopCartList);
            buyMsgRepository.save(buyMsgList);
            shopCartRepository.deleteByUserId(userId);
            return APIResult.ok(Boolean.TRUE);
        }
        return APIResult.error(UserServiceAPICode.NOT_EXISTS);
    }

    public APIResult<List<BuyMsg>> get(String userId) {
        BuyMsg buyMsg = new BuyMsg();
        buyMsg.setUserId(userId);
        buyMsg.setStatus(BaseStatusEnum.NORMAL);
        List<BuyMsg> buyMsgList = buyMsgRepository.find(buyMsg);
        return APIResult.ok(buyMsgList);
    }

    @Transactional
    public APIResult<Boolean> delete(String userId) {
        Integer count = buyMsgRepository.deleteByUserId(userId);
        if (count > 0) {
            return APIResult.ok(Boolean.TRUE);
        }
        return APIResult.error(UserServiceAPICode.NOT_EXISTS);
    }

    public static List<BuyMsg> convert(List<ShopCart> shopCartList) {
        List<BuyMsg> buyMsgList = new ArrayList<>();
        for (ShopCart shopCart : shopCartList) {
            BuyMsg buyMsg = new BuyMsg();
            buyMsg.setUserId(shopCart.getUserId());
            buyMsg.setGoodsId(shopCart.getGoodsId());
            buyMsg.setPid(shopCart.getPid());
            buyMsg.setStatus(BaseStatusEnum.NORMAL);
            buyMsgList.add(buyMsg);
        }
        return buyMsgList;
    }
}
