package cn.rankin.userservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.user.dto.CampusDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.userservice.code.UserServiceAPICode;
import cn.rankin.data.api.user.entity.Campus;
import cn.rankin.userservice.repository.CampusRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import static cn.rankin.userservice.utils.DTOConverter.convert;

@Slf4j
@Service
public class CampusService {

    @Autowired
    private CampusRepository campusRepository;

    public Integer getSerialNo(String merchantId, String pCode) {
        Integer serialNo = campusRepository.findMaxNo(merchantId, pCode);
        if (serialNo == null) {
            return 1;
        }
        return serialNo + 1;
    }

    public Campus findOne(String id) {
        return campusRepository.findOne(id);
    }

    public Map<String, Campus> getCampusMap(List<String> idList) {
        List<Campus> campusList = campusRepository.findByIds(idList);

        Map<String, Campus> map = new HashMap<>();
        campusList.forEach(campus -> map.put(campus.getId(), campus));

        return map;
    }

    public APIResult<Page<Campus>> search(Campus campus, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = campusRepository.count(campus);
        Page<Campus> page = new Page<>(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Campus> campusList = campusRepository.find(campus, page.getStart(), page.getPageSize(), JpaSortUtil.sort(sort));
        page.setList(campusList);

        return APIResult.ok(page);
    }

    public APIResult<List<Campus>> find(Campus campus, LinkedHashMap<String, BaseOrderEnum> sort) {
        List<Campus> campusList = campusRepository.find(campus, JpaSortUtil.sort(sort));
        return APIResult.ok(campusList);
    }

    public APIResult<Campus> getCampus(String campusId) {
        Campus campus = campusRepository.find(campusId);
        if (campus != null) {
            return APIResult.ok(campus);
        }
        return APIResult.error(UserServiceAPICode.NOT_EXISTS);
    }

    @Transactional
    public APIResult<Campus> create(CampusDTO campusDTO) {
        Campus campus = convert(campusDTO);
        Campus result = campusRepository.save(campus);
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Campus> update(CampusDTO campusDTO) {
        Campus campus = convert(campusDTO);
        Campus result = campusRepository.update(campus);

        return APIResult.ok(result);
    }
}
