package cn.rankin.userservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.LedgerTypeEnum;
import cn.rankin.data.api.user.dto.MerchantDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.userservice.code.UserServiceAPICode;
import cn.rankin.data.api.user.entity.Merchant;
import cn.rankin.userservice.repository.MerchantRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.transaction.Transactional;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import static cn.rankin.userservice.utils.DTOConverter.convert;

@Service
public class MerchantService {

    @Autowired
    private MerchantRepository merchantRepository;

    public APIResult<Page<Merchant>> search(Merchant merchant, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = merchantRepository.count(merchant);
        Page<Merchant> page = new Page<>(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Merchant> merchantList = merchantRepository.find(merchant, page.getStart(), page.getPageSize(), JpaSortUtil.sort(sort));
        page.setList(merchantList);

        return APIResult.ok(page);
    }

    public Boolean exists(String code) {
        Long count = merchantRepository.countByCode(code);
        return count > 0;
    }

    @Transactional
    public APIResult<Merchant> create(MerchantDTO merchantDTO) {
        if (exists(merchantDTO.getCode())) {
            return APIResult.error(APICode.ALREADY_EXISTS);
        }

        Merchant merchant = convert(merchantDTO);
        Merchant result = merchantRepository.save(merchant);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Merchant> update(MerchantDTO merchantDTO) {
        Merchant merchant = convert(merchantDTO);
        Merchant result = merchantRepository.update(merchant);
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = merchantRepository.updateStatusById(id, BaseStatusEnum.DEL);
        if (count == 0) {
            return APIResult.error(APICode.NOT_EXISTS);
        }

        return APIResult.ok(true);
    }

    public APIResult<Merchant> getMerchant(String id) {
        Merchant merchant = merchantRepository.find(id);
        if (merchant == null) {
            return APIResult.error(APICode.NOT_EXISTS);
        }
        return APIResult.ok(merchant);
    }

    public APIResult<Merchant> findNormalById(String id) {
        Merchant merchant = merchantRepository.findByIdAndStatus(id, BaseStatusEnum.NORMAL);
        if (merchant == null) {
            return APIResult.error(UserServiceAPICode.NOT_EXISTS);
        }

        return APIResult.ok(merchant);
    }

    public APIResult<Map<String, Merchant>> findByIds(List<String> idList) {
        List<Merchant> merchantList = merchantRepository.findByIds(idList);
        Map<String, Merchant> merchantMap = new HashMap<>();
        merchantList.forEach( merchant -> merchantMap.put(merchant.getId(), merchant));
        return APIResult.ok(merchantMap);
    }

    public Map<String, Merchant> getMerchantMap(List<String> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            return new HashMap<>();
        }

        List<Merchant> merchantList = merchantRepository.findByIds(idList);
        Map<String, Merchant> merchantMap = new HashMap<>();

        merchantList.forEach(merchant -> merchantMap.put(merchant.getId(), merchant));
        return merchantMap;
    }

    public Merchant findOne(String id) {
        return merchantRepository.find(id);
    }

    public Merchant findMerchantByName(String name) {
        return merchantRepository.findByName(name);
    }
}
