package cn.rankin.userservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.LedgerTypeEnum;
import cn.rankin.data.api.user.entity.Merchant;
import cn.rankin.userservice.code.UserServiceAPICode;
import cn.rankin.userservice.repository.MerchantRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.math.BigDecimal;

@Service
public class PayingService {

    @Autowired
    private MerchantRepository merchantRepository;

    @Autowired
    private LedgerService ledgerService;

    @Transactional
    public APIResult<Boolean> pay(String merchantId, BigDecimal quantity, LedgerTypeEnum type, String receiptId, String note) {
        return this.change(merchantId, quantity.negate(), type, receiptId, note);
    }

    @Transactional
    public APIResult<Boolean> change(String merchantId, BigDecimal quantity, LedgerTypeEnum type, String receiptId, String note) {
        Merchant merchant = merchantRepository.find(merchantId);
        if (merchant == null || !merchant.getStatus().equals(BaseStatusEnum.NORMAL)) {
            return APIResult.error(UserServiceAPICode.NOT_EXISTS);
        }

        BigDecimal balance = merchant.getBalance();
        if (balance.add(quantity).compareTo(BigDecimal.ZERO) < 0) {
            return APIResult.error(UserServiceAPICode.MONEY_NOT_ENOUGH);
        }

        Integer count = merchantRepository.updateBalance(merchantId, quantity);
        if (count <= 0) {
            return APIResult.error(UserServiceAPICode.PAY_FAILED);
        }

        // 插入流水
        ledgerService.insert(merchantId, quantity, merchant.getBalance(), type, receiptId, note);

        return APIResult.ok();
    }

    @Transactional
    public APIResult<Boolean> charge(String merchantId, BigDecimal quantity, String note) {
        return this.change(merchantId, quantity, LedgerTypeEnum.CHARGE, null, note);
    }
}
