package cn.rankin.userservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.data.api.user.dto.ShopCartDTO;
import cn.rankin.data.api.user.entity.ShopCart;
import cn.rankin.userservice.code.UserServiceAPICode;
import cn.rankin.userservice.repository.ShopCartRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.*;

@Service
public class ShopCartService {

    @Autowired
    private ShopCartRepository shopCartRepository;

    public APIResult<List<ShopCart>> findByIds(String userId, List<String> goodsIdList) {
        List<ShopCart> shopCartList = shopCartRepository.findByGoodsIds(goodsIdList, userId);
        return APIResult.ok(shopCartList);
    }

    public APIResult<Page<ShopCart>> getPage(String userId, Integer pageNo, Integer pageSize) {
        ShopCart sample = new ShopCart();
        sample.setUserId(userId);
        sample.setStatus(BaseStatusEnum.NORMAL);
        Long total = shopCartRepository.count(sample);
        Page<ShopCart> page = new Page(total, pageNo, pageSize);
        if (total == 0) {
            return APIResult.ok(page);
        }
        List<ShopCart> shopCartList = shopCartRepository.find(sample, page.getStart(), pageSize);
        page.setList(shopCartList);
        return APIResult.ok(page);
    }

    public APIResult<Map<String, Long>> getTotalNum(String userId) {
        ShopCart sample = new ShopCart();
        sample.setUserId(userId);
        sample.setStatus(BaseStatusEnum.NORMAL);
        Long total = shopCartRepository.count(sample);
        Map<String, Long> result = new HashMap<>();
        result.put("totalNum", total);
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> add(String userId, String goodsId, String productId) {
        ShopCart shopCart = new ShopCart();
        shopCart.setUserId(userId);
        shopCart.setPid(productId);

        ShopCart shopCartBefore = shopCartRepository.findFirst(shopCart);
        if (shopCartBefore != null) {
            shopCartBefore.setGoodsId(goodsId);
            shopCartBefore.setStatus(BaseStatusEnum.NORMAL);
            shopCartRepository.save(shopCartBefore);
        }else {
            shopCart.setGoodsId(goodsId);
            shopCart.setStatus(BaseStatusEnum.NORMAL);
            shopCartRepository.save(shopCart);
        }
        return APIResult.ok();
    }

    @Transactional
    public APIResult<Boolean> delete(String userId, String goodsId) {
        Integer count = shopCartRepository.deleteByUserIdAndGoodsId(userId, goodsId);
        if (count > 0) {
            return APIResult.ok(Boolean.TRUE);
        }
        return APIResult.error(UserServiceAPICode.NOT_EXISTS);
    }
}
