package cn.rankin.userservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.user.dto.TerminalUserDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.userservice.code.UserServiceAPICode;
import cn.rankin.data.api.user.entity.TerminalUser;
import cn.rankin.userservice.repository.TerminalUserRepository;
import cn.rankin.userservice.utils.DTOConverter;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;

import java.util.LinkedHashMap;
import java.util.List;

import static cn.rankin.userservice.utils.DTOConverter.convert;

@Service
public class TerminalUserService {

    @Autowired
    private TerminalUserRepository terminalUserRepository;

    public Integer getSerialNo(String campusId) {
        Integer serialNo = terminalUserRepository.findMaxNo(campusId);
        if (serialNo == null) {
            return 1;
        }

        return serialNo + 1;
    }

    public TerminalUser find(String id) {
        return terminalUserRepository.find(id);
    }

    public TerminalUser loadUserByCode(String code) {
        return terminalUserRepository.findByCode(code);
    }

    public List<TerminalUser> find(TerminalUser terminalUser, LinkedHashMap<String, BaseOrderEnum> sort) {
        List<TerminalUser> terminalUserList = terminalUserRepository.find(terminalUser, JpaSortUtil.sort(sort));
        return terminalUserList;
    }

    public List<TerminalUser> findByIds(List<String> ids) {
        return terminalUserRepository.findByIds(ids);
    }

    public List<TerminalUser> findByCampusIds(List<String> campusIds) {
        List<TerminalUser> terminalUserList = terminalUserRepository.findByCampusIdIn(campusIds);
        return terminalUserList;
    }

    public APIResult<Page<TerminalUser>> search(TerminalUser terminalUser, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = terminalUserRepository.count(terminalUser);
        Page<TerminalUser> page = new Page<>(count, pageNo, pageSize);
        if (count == 0) {
            return APIResult.ok(page);
        }

        List<TerminalUser> terminalUserList = terminalUserRepository.find(terminalUser, page.getStart(), pageSize, JpaSortUtil.sort(sort));
        page.setList(terminalUserList);

        return APIResult.ok(page);
    }

    @Transactional
    public APIResult<TerminalUser> create(TerminalUserDTO terminalUserDTO) {
        TerminalUser terminalUser = convert(terminalUserDTO);
        String newPassword = terminalUserDTO.getPassword();
        if (StringUtils.isEmpty(newPassword)) {
            return APIResult.error(UserServiceAPICode.PARAMETER_ERROR);
        }

        // md5加密密码, 因需要登录远程用户中心, 只能明文存储
        // terminalUser.setPassword(SecurityUtil.MD5(newPassword));
        TerminalUser result = terminalUserRepository.save(terminalUser);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<TerminalUser> update(TerminalUserDTO terminalUserDTO) {
        TerminalUser terminalUser = DTOConverter.convert(terminalUserDTO);
        TerminalUser result = terminalUserRepository.update(terminalUser);
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = terminalUserRepository.updateStatusById(BaseStatusEnum.DISABLE, id);
        if (count > 0) {
            return APIResult.ok(true);
        }
        return APIResult.error(UserServiceAPICode.NOT_EXISTS);
    }

}
