package cn.rankin.userservice.service;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.ListUtil;

import cn.rankin.data.api.product.entity.MerchantProduct;
import cn.rankin.data.api.user.entity.UserRecommend;
import cn.rankin.userservice.repository.UserRecommendRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class UserRecommendService {

    @Autowired
    private UserRecommendRepository userRecommendRepository;

    public List<UserRecommend> get(String uid) {
        List<UserRecommend> recommendList = userRecommendRepository.findByStatus(uid, BaseStatusEnum.NORMAL);
        if (CollectionUtils.isEmpty(recommendList)) {
            return new ArrayList<>();
        }

        return recommendList;
    }

    public List<UserRecommend> put(String uid, List<MerchantProduct> products) {
        List<String> productIdList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(products)) {
            products.forEach(product -> productIdList.add(product.getPid()) );
        }

        List<UserRecommend> userRecommendList = userRecommendRepository.findByUid(uid);
        List<String> existRecommendIdList = new ArrayList<>();

        // 更新现存的
        userRecommendList.forEach(recommend -> {
            String pid = recommend.getPid();
            if (productIdList.contains(pid)) {
                recommend.setStatus(BaseStatusEnum.NORMAL);
            }else {
                recommend.setStatus(BaseStatusEnum.DEL);
            }
            existRecommendIdList.add(pid);
        });

        // 更新新加的
        List<String> notExistsRecommendIdList = ListUtil.subtract(productIdList, existRecommendIdList);
        if (notExistsRecommendIdList != null && notExistsRecommendIdList.size() > 0) {
            Map<String, MerchantProduct> merchantProductMap = ListUtil.convert(products, "pid", MerchantProduct.class);
            for (String productId : notExistsRecommendIdList) {
                MerchantProduct merchantProduct = merchantProductMap.get(productId);
                UserRecommend recommend = new UserRecommend();
                recommend.setPid(productId);
                recommend.setUid(uid);
                recommend.setStatus(BaseStatusEnum.NORMAL);
                recommend.setType(merchantProduct.getType());
                userRecommendList.add(recommend);
            }

        }

        // 排序
        userRecommendList.forEach( recommend -> {
            String productId = recommend.getPid();
            Integer sort = productIdList.indexOf(productId);
            recommend.setSort(sort);
        });

        userRecommendRepository.save(userRecommendList);
        List<UserRecommend> result = userRecommendRepository.findByUid(uid);
        return result;
    }

    public List<UserRecommend> getUserRecommendCourses(String uid) {
        List<UserRecommend> recommendList = userRecommendRepository.findByStatus(uid, BaseStatusEnum.NORMAL);
        if (CollectionUtils.isEmpty(recommendList)) {
            return new ArrayList<>();
        }

        return recommendList;
    }
}
