package cn.rankin.userservice.utils;

import cn.rankin.common.utils.constant.Constant;
import cn.rankin.common.utils.util.ProvinceUtil;
import org.apache.commons.lang.StringUtils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/* 学号构成:
    左 -> 右
    4位: App编码, 1502
    4位: 渠道编码
    2位: 省份编码
    2位: 校区流水号, 递增
    3位: 教室流水号, 递增
*/
public class StudentNumberUtil {

    public final static String APP_CODE = Constant.APP_CODE;

    public final static String MRegEx = Constant.MRegEx;

    public final static int MAX_CAMPUS = 99;    // 校区目前最多限制两位

    public final static int MAX_ROOM = 999; // 教室最多三位

    public final static String[] LOW = { "零", "一", "二", "三", "四", "五", "六", "七", "八", "九", "十"};
    public final static String[] HIGH = { "十", "百", "千", "万"};

    public static boolean isValidCampusNo(Integer campusNo) {
        if (0 < campusNo && campusNo <= MAX_CAMPUS) {
            return true;
        }
        return false;
    }

    public static boolean isValidRoomNo(Integer roomNo) {
        if (roomNo > 0 && roomNo <= MAX_ROOM) {
            return true;
        }
        return false;
    }

    public static boolean isValidMerchantCode(String mCode) {
        Pattern pattern = Pattern.compile(MRegEx);
        Matcher matcher = pattern.matcher(mCode);
        return matcher.matches();
    }

    public static StudentNo getStudentNo(String merchantCode, String pCode, Integer campusNo, Integer roomNo) {
        if (!isValidCampusNo(campusNo) || !isValidRoomNo(roomNo) || ProvinceUtil.get(pCode) == null
                || !isValidMerchantCode(merchantCode)) {
            return null;
        }

        return new StudentNo(APP_CODE, merchantCode, pCode, campusNo, roomNo);
    }

    public static StudentNo getStudentNo(String merchantCode, String pCode, Integer campusNo) {
        if (!isValidCampusNo(campusNo) || ProvinceUtil.get(pCode) == null || !isValidMerchantCode(merchantCode)) {
            return null;
        }

        return new StudentNo(APP_CODE, merchantCode, pCode, campusNo, null);
    }

    public static String getStudentNo(String campusCode, Integer roomNo) {
        if (roomNo == null || StringUtils.isEmpty(campusCode)) {
            return null;
        }
        return String.format("%s%03d", campusCode, roomNo);
    }

    public static void main(String[] args) {
//        StudentNo a = getStudentNo("0222", "11", 99);
//        System.out.println(a.getCampusNo());
        int[] a = {0, 1, 9, 10, 11, 19, 20, 50, 71, 99, 100, 101, 120, 121, 301, 501, 900, 901, 999, 1000, 10000, 10001, 11001, 11100, 11101, 11110, 11111};
//        System.out.println('9' - '0');
        for (int i : a) {
            System.out.println(toChinese(i));
        }
        System.out.println('2' - '0');
    }

    public static String toChinese(Integer value, String prefix) {
        return prefix + toChinese(value);
    }
    /*
    10000以内的整数转化汉字
     */
    public static String toChinese(Integer value) {
        String string = String.valueOf(value);

        String result = new String("");

        if (value == 0) {
            return  "零";
        }else if (value > 0 && value <= 10) {
            result = LOW[value];
        }else if (value > 10 && value < 20) {
            result = HIGH[0] + LOW[value % 10];
        }else {
            int n = string.length();
            for (int i = 0; i < string.length(); i++) {
                int num = string.charAt(i) - '0';
                if (i != n-1 && num != 0) {
                    result += LOW[num] + HIGH[n - 2 - i];
                }else {
                    if (!LOW[num].equals("零") || !result.endsWith("零")) {
                        result += LOW[num];
                    }
                }
            }
        }

        if (result.endsWith("零")) {
            result = result.substring(0, result.length() - 1);
        }

        return result;
    }

}
